** -----------------------------------------------------------------------------
** -----------------------------------------------------------------------------
** This file creates dataset to estimate moments, BHPS-DataForMoments.dta
** -----------------------------------------------------------------------------
** -----------------------------------------------------------------------------


do "paths.do"


** read data
** -----------------------------------------------------------------------------
drop _all
clear matrix
use "$data\BHPS.dta", replace
sort pid wave
gen y = 1990+wave


** clean 
** -----------------------------------------------------------------------------
drop if wklife==0
drop if ~inrange(age,19,59)
drop if s==3 & age<22
drop if noint==1
sort pid wave
by pid: gen todrop=sum(dSE==1)
drop if todrop~=0
drop todrop n N
by pid: gen n=_n
by pid: gen N=_N


** delete NI and construct indicators for scotland and wales
** -----------------------------------------------------------------------------
sort pid wave
by pid: gen region1=region[1] if inrange(region[1],0,20)
forvalues w=2/18 {
	by pid: replace region1 = region[`w'] if region1==. & inrange(region[`w'],0,20)
}
keep if region1~=19 
gen wales=(region1==17)
gen scotland=(region1==18)
replace region=region1
drop region1


** construct (real) wage index
** -----------------------------------------------------------------------------

qui ta s, g(s)
qui ta wave, g(w)

forvalues s=1/3 {
	qui regress s`s' w1-w18 if lms==1 & inrange(age,25,59), noc
	global s`s' = 0
	forvalues w=1/18 {
		global s`s' = ${s`s'} + _b[w`w']/18
	}
}

centile jb1w if lms==1 & jb1w>0, c(1 99)
gen w1to99 = jb1w if lms==1 & inrange(jb1w,r(c_1),r(c_2))

forvalues s=1/3 {
	qui regress w1to99 w1-w18 wales scotland if inrange(age,25,59) & s==`s', noco
	gen Widx`s' = .
	forvalues i=1/18 {
		replace Widx`s' = _b[w`i']/_b[w18] if wave==`i'
	}
}
gen Widx = ${s1} * Widx1 + ${s2} * Widx2 + ${s3}*Widx3
label var Widx "wage index to detrend real wage rates" 



** Extra variables and more cleaning
** -----------------------------------------------------------------------------

keep if female==1

* transitions
by pid: gen byte trUE = dE5p if dE5p[_n-1]==0 & wave[_n-1]==wave-1
by pid: gen byte trEU = 1-dE5p if dE5p[_n-1]==1 & wave[_n-1]==wave-1

** de-trended log wage
gen lW = ln(jb1wage/Widx) if dE5p==1
centile lW, c(2 99)
** keep record of trimmed observations
gen trimmed = (dE5p==1 & lW~=. & ~inrange(lW,r(c_1),r(c_2)))
replace lW=. if ~inrange(lW,r(c_1),r(c_2))
label var lW "real detrended log wage rate in 1st job (central 98%, 2008 prices & wages)"


** Background info and regime
** -----------------------------------------------------------------------------

keep if f1 ~= .

centile f1 if n==1, c(50)
gen H1 = (f1>r(c_1))
global m1 = r(c_1)
centile f2 if n==1, c(50)
gen H2 = (f2>r(c_1))
global m2 = r(c_1)

gen t00to08 = (wave>9)


** Family composition
** -----------------------------------------------------------------------------

gen nkd=(ownkids>0)
replace nkd=1   if nkd==0 & inrange(yngk,16,18) & age>=35
replace nkd=0   if yngk==-1
replace yngk=-1 if nkd==0

gen family = 1*(couple==0 & nkd==0) + 2*(couple==1 & nkd==0) + 3*(couple==0 & nkd==1) + 4*(couple==1 & nkd==1)
label define family 1 "singles, no kids" 2 "couple no kids" 3 "lone mothers" 4 "mothers in couples"
label val family family

gen dm0k0 = (couple==0 & nkd==0)
gen dm1k0 = (couple==1 & nkd==0)
gen dm0k1 = (couple==0 & nkd==1)
gen dm1k1 = (couple==1 & nkd==1)

gen dk0to2   = inrange(yngkid,0,2)*(nkd==1)
gen dk3to5   = inrange(yngkid,3,5)*(nkd==1)
gen dk6to10  = inrange(yngkid,6,10)*(nkd==1)
gen dk11to18 = inrange(yngkid,11,18)*(nkd==1)
gen dpwk     = (pwk==1)*couple

label var dm0k0    "single"
label var dm1k0    "couple"
label var dm0k1    "lone mother"
label var dm1k1    "mother in couple"
label var dk0to2   "yng kid 0-2"
label var dk3to5   "yng kid 3-5"
label var dk6to10  "yng kid 6-10"
label var dk11to18 "yng kid 11-18"
label var dpwk     "partner working"


** clean and save
** -----------------------------------------------------------------------------
sort pid wave
gen age1 = 19*(s<=2)+22*(s==3)
by pid: gen todrop = sum(inrange(kidage1,age-age1+3,18))
drop if todrop~=0
drop todrop age1

keep if inrange(age,19,50)
drop n N
sort pid wave
by pid: gen n=_n
by pid: gen N=_N

drop w1-w18 s1-s3 w1to99 Widx1-Widx3

save "$data\DataForMoments-WL.dta", replace



** construct residual from parental income
** -----------------------------------------------------------------------------
drop _all
clear matrix
use "$data\BHPS.dta", replace
sort pid wave

* construct region dummies
forvalues n=2/18 {
	qui replace region = region[_n-1] if region>=. & region[_n-1]<. & n==`n'
}
forvalues n=18(-1)2 {
	qui replace region = region[_n+1] if region>=. & region[_n+1]<. & n[_n+1]==`n'
}
by pid: gen region1=region[1] if inrange(region[1],0,20)

keep if region1<19 
gen wales=(region1==17)
gen scotland=(region1==18)
replace region=region1
drop region1

* construct residuals
drop n N
gen priority = (age==16) + 2*(age==15) + 3*(age==17) + 4*(age==18) + 5*(age>18)
sort pid priority
by pid: gen n=_n
keep if n==1 & priority<=4
keep if female==1 & insample==1 & f1~=.

gen noparent = (mumpresent==0 & dadpresent==0)
regress parentY f1 f2 mumpresent dadpresent noparent wales scotland if insample==1
predict parentYres if e(sample), r

keep if yob>=1970 & s23~=. & e(sample) 

gen H1 = (f1>$m1)
gen H2 = (f2>$m2)

* clean & save
keep pid wave age female f1 f2 H1 H2 yob parentY* insamplewp insample s23 pid   ///
	 yob mumpresent dadpresent scotland wales region
gen sampleS=1
gen wklife=0

sort pid wave
save "$data\DataForMoments-Educ.dta", replace


** Merge the two datasets together and prepare data for fortran
** -----------------------------------------------------------------------------

use "$data\DataForMoments-WL.dta", replace		/* old version */
append using "$data\DataForMoments-Educ.dta"
sort pid wave
by pid: egen everwl=sum(wklife)
replace everwl=(everwl>0)

label var sampleS "sample to estimate education choice moments"
label var everWL  "indicator: observed during working life"

save "$data\BHPS-DataForMoments.dta", replace



