** -----------------------------------------------------------------------------
** -----------------------------------------------------------------------------
** This file estimates data moments used in structural estimation
** Then exports them to files and calculates inverse of optimal wegihting matrix
** -----------------------------------------------------------------------------
** -----------------------------------------------------------------------------


** Locate data and output
** -----------------------------------------------------------------------------

clear all
drop _all
do "paths.do"
global datafile "${data}\BHPS-DataForMoments.dta"      
global BSdatafile "${data}\BHPS-DataForMoments-BS.dta"



** Get data ready
** -----------------------------------------------------------------------------

use "${datafile}", replace
sort pid wave

* distribution of log wages during WL
cap drop rlW
gen rlW = .
forvalues s=1/3 {
	qui regress lW wales scotland if wklife==1 & s==`s' & dE5p==1
	qui replace rlW = lW - _b[wales]*wales - _b[scotland]*scotland if e(sample)
	qui centile rlW if s==`s' & dE5p==1 & dPT5to20==0, c(10 25 50 75 90)
	global Q10lW_FTs`s' = r(c_1)
	global Q25lW_FTs`s' = r(c_2)
	global Q50lW_FTs`s' = r(c_3)
	global Q75lW_FTs`s' = r(c_4)
	global Q90lW_FTs`s' = r(c_5)
	qui centile rlW if s==`s' & dE5p==1 & dPT5to20==1, c(10 25 50 75 90)
	global Q10lW_PTs`s' = r(c_1)
	global Q25lW_PTs`s' = r(c_2)
	global Q50lW_PTs`s' = r(c_3)
	global Q75lW_PTs`s' = r(c_4)
	global Q90lW_PTs`s' = r(c_5)
}
by pid: gen rlW_1 = rlW[_n-1] if wklife[_n-1]==1
centile rlW, c(10 50 90)
global Q10lW = r(c_1)
global Q50lW = r(c_2)
global Q90lW = r(c_3)

* distribution of log wage rates at entrance in WL
cap drop rlW1
gen rlW1 = .
forvalues s=1/3 {
	qui regress lW wales scotland if n==1 & student==1 & inrange(age,19,25) & s==`s' & dE5p==1
	qui replace rlW1 = lW - _b[wales]*wales - _b[scotland]*scotland if e(sample)
	qui centile rlW1 if s==`s', c(25 50 75)
	global Q25lW1_s`s' = r(c_1)
	global Q50lW1_s`s' = r(c_2)
	global Q75lW1_s`s' = r(c_3)
}

* family demmographics
gen family1 = (family<=2) + 2*(family==3)+3*(family==4)

* employment variables
rename dE5p E
rename dPT5to20 PT
by pid: gen E_1 = E[_n-1] if wklife[_n-1]==1
by pid: gen PT_1 = PT[_n-1] if wklife[_n-1]==1

* variables for wage regressions
gen le = log(e+1) if wklife==1
by pid: gen le_1   = le[_n-1] if wave==wave[_n-1]+1 & wklife[_n-1]==1
by pid: gen lW_1   = lW[_n-1] if wave==wave[_n-1]+1 & wklife[_n-1]==1
gen dlW = lW-lW_1
gen dle = le-le_1

cap drop dlWcompl
by pid: gen tsincelastW = 1           if E==1 & E_1==1 & wave==wave[_n-1]+1 & wklife[_n-1]==1
by pid: gen dlWcompl    = lW-lW[_n-1] if tsincelastW==1
forvalues gap=2/17 {
	qui by pid: replace tsincelastW = `gap'           if tsincelastW==. & E==1 & E[_n-`gap']==1 & wave==wave[_n-`gap']+`gap'
	qui by pid: replace dlWcompl    = lW-lW[_n-`gap'] if tsincelastW==`gap'
}
gen ydlW = dlWcompl/tsincelastW
by pid: gen dFT_1 = (E_1==1 & PT_1==0) if wklife[_n-1]==1
by pid: gen dPT_1 = (E_1==1 & PT_1==1) if wklife[_n-1]==1
by pid: gen dNW_1 = (E_1==0) if wklife[_n-1]==1

* CC costs
gen dCC = inrange(jb1ccarecost,1,10000) if wklife==1 & E==1 & nkd==1 & yngkid<=10 & inrange(jb1ccare,0,1)
replace dCC = . if ~inrange(jb1ccarecost,0,10000) & jb1ccare==1
regress dCC wales scotland
gen rdCC = dCC-_b[wales]*wales-_b[scotland]*scotland

* variables for education moments
gen g80 = inrange(yob, 1982, 1987) if inrange(yob, 1970, 1987) & sampleS==1
gen g80_f1 = g80 * f1
gen g80_f2 = g80 * f2

ta s23 if sampleS==1, g(s23_)
gen dparentsearn0 = (parentY==0) if sampleS==1

gen a1=age/10

* save data
save "${BSdatafile}", replace



** Routines to estimate moments
** -----------------------------------------------------------------------------

** Moments used to estimate WL parameters
cap program drop momentsWL
program define momentsWL

	local i=$i
	
	forvalues s=1/3 {
		
		global condition "wklife==1 & scotland==0 & wales==0"

		
		** Employment: means
		** -----------------
		global kage "0to2 3to5 6to10 11to18"
		foreach var in E PT {

			sum `var' if $condition & s==`s'
			global `var'_s`s'_`i' = r(mean)

			sum `var' if $condition & s==`s' & pwk==1
			global `var'_pwks`s'_`i' = r(mean)
			
			forvalues f = 1/4 {
				sum `var' if $condition & s==`s' & family ==`f'
				global `var'_f`f's`s'_`i' = r(mean)
			}
			
			foreach a of global kage {
				sum `var' if $condition & s==`s' & dk`a'==1
				global `var'_k`a's`s'_`i' = r(mean)
			}

			sum `var' if $condition & s==`s' & H1==1
			global `var'_H1s`s'_`i' = r(mean)
			sum `var' if $condition & s==`s' & H2==1
			global `var'_H2s`s'_`i' = r(mean)
			
			regress `var' t00to08 scotland wales if wklife==1 & s==`s'
			global `var'_00to08s`s'_`i' = _b[t00to08]
		}

		
		** Transition rates into work
		** --------------------------
		local var trUE
		sum `var' if $condition & s==`s'
		global `var'_s`s'_`i' = r(mean)

		forvalues f1 = 1/3 {
			sum `var' if $condition & s==`s' & family1 ==`f1'
			global `var'_f`f1's`s'_`i' = r(mean)
		}
		
		
		** Transition rates out of work
		** ----------------------------
		local var trEU
		sum `var' if $condition & s==`s'
		global `var'_s`s'_`i' = r(mean)

		forvalues f1 = 1/3 {
			sum `var' if $condition & s==`s' & family1 ==`f1'
			global `var'_f`f1's`s'_`i' = r(mean)
		}
			
		forvalues c=10(40)90 {
			sum `var' if $condition & s==`s' & rlW_1<=${Q`c'lW}
			global `var'_c`c's`s'_`i' = r(mean)
		}
		
		
		** log wages during WL: regressions
		** --------------------------------
		global vars "H1 H2 lW_1 le le_1"
		regress lW ${vars} t00to08 wales scotland if wklife==1 & s==`s' & E==1 & E_1==1
		predict r if e(sample), r
		gen r2 = r*r
		sum r2
		global VrlW_s`s'_`i' = r(mean)
		by pid: gen rr_1 = r*r[_n-1]
		sum rr_1 
		global Err1_s`s'_`i' = r(mean) 
		drop r r2 rr_1
		foreach v of global vars {
			global lW`v'_s`s'_`i' = _b[`v']
		}
		global lWcons_s`s'_`i' = _b[_cons]

		regress dlW dle wales scotland if wklife==1 & s==`s' & E==1 & E_1==1, noc
		global dlWdle_s`s'_`i' = _b[dle]
		
		regress lW H1 H2 a1 wales scotland if wklife==1 & s==`s' & E==1
		global lWslopecons_s`s'_`i' = _b[_cons]
		global lWslopeH1_s`s'_`i'   = _b[H1]
		global lWslopeH2_s`s'_`i'   = _b[H2]
		global lWslopea_s`s'_`i'    = _b[a1]
		
		regress ydlW dFT_1 dPT_1 dNW_1 wales scotland if s==`s', noc
		global ydlWdFT_s`s'_`i' = _b[dFT_1]
		global ydlWdPT_s`s'_`i' = _b[dPT_1]
		global ydlWdNW_s`s'_`i' = _b[dNW_1]
		
		
		** distribution of log wages
		** -------------------------
		sum rlW if wklife==1 & s==`s' & E==1 & PT==0
		global ElW_FTs`s'_`i' = r(mean)
		sum rlW if wklife==1 & s==`s' & E==1 & PT==1
		global ElW_PTs`s'_`i' = r(mean)

		sum rlW if wklife==1 & s==`s' & E==1 & H1==1
		global ElW_H1s`s'_`i' = r(mean)
		sum rlW if wklife==1 & s==`s' & E==1 & H2==1
		global ElW_H2s`s'_`i' = r(mean)
		
		foreach c of numlist 10 25 50 75 90 {
			cap drop q
			gen q = (rlW<=${Q`c'lW_FTs`s'}) if wklife==1 & s==`s' & E==1 & PT==0 & rlW<.
			sum q
			global pQ`c'lW_FTs`s'_`i' = r(mean)
			cap drop q
			gen q = (rlW<=${Q`c'lW_PTs`s'}) if wklife==1 & s==`s' & E==1 & PT==1 & rlW<.
			sum q
			global pQ`c'lW_PTs`s'_`i' = r(mean)
		}
		
		** Prevalence of CC costs
		** ----------------------
		sum rdCC if s==`s' & wklife==1
		global CC_s`s'_`i' = r(mean)
		
	}
		
end 


cap program drop momentsWLentry
program define momentsWLentry

	local i=$i
		
	forvalues s=1/3 {

		** Distribution of log wage rates at entrance in WL
		** ------------------------------------------------
		sum rlW1 if s==`s'
		global VlW1_s`s'_`i' = r(Var)
		global ElW1_s`s'_`i' = r(mean)

		sum rlW1 if s==`s' & H1==1
		global ElW1_H1s`s'_`i' = r(mean)
		sum rlW1 if s==`s' & H2==1
		global ElW1_H2s`s'_`i' = r(mean)
		
		forvalues c=25(25)75 {
			cap drop q
			gen q = (rlW1<=${Q`c'lW1_s`s'}) if rlW1<. & s==`s'
			sum q
			global pQ`c'lW1_s`s'_`i' = r(mean)
		}

	}
	
end 


cap program drop momentsEd
program define momentsEd

	local i=$i
	global Xvar "g80 f1 f2 g80_f1 g80_f2 parentYres"

	forvalues s=1/3   {

		sum s23_`s' if sampleS==1 & wales+scotland==0
		global s_s`s'_`i' = r(mean)

		forvalues H=0/1 {
			sum s23_`s' if sampleS==1 & wales+scotland==0 & H1==`H'
			global sH1`H'_s`s'_`i' = r(mean)
			sum s23_`s' if sampleS==1 & wales+scotland==0 & H2==`H'
			global sH2`H'_s`s'_`i' = r(mean)
		}

		regress s23_`s' $Xvar wales scotland if sampleS==1
		foreach v of global Xvar {
			global s`v'_s`s'_`i' = _b[`v']
		}
		global scons_s`s'_`i' = _b[_cons]
			
	}
	
end



** Estimate moments and SE, then export to file to be read by fortran code
** -----------------------------------------------------------------------------

* estimate moments
use "${BSdatafile}", replace
global i = 0
qui momentsWL
qui momentsWLentry
qui momentsEd

** bootstrap st errors
set seed 8705871
forvalues iter=1/1000 {
	
	global i =`iter'
	
	qui use "${BSdatafile}", replace
	qui bsample, cluster(pid) idcluster(j)
	qui sort pid j wave

	qui momentsWL
	qui momentsWLentry
	qui momentsEd
		
	di "BS iteration ", `iter'

}


** now calculate SE of estimated moments
global varSmean   "s_s sH10_s sH11_s sH20_s sH21_s"
global varSregr   "scons_s sg80_s sf1_s sf2_s sg80_f1_s sg80_f2_s sparentYres_s"
global varEmean   "E_s E_f1s E_f2s E_f3s E_f4s E_pwks E_k0to2s E_k3to5s E_k6to10s E_k11to18s E_H1s E_H2s E_00to08s"
global varPTmean  "PT_s PT_f1s PT_f2s PT_f3s PT_f4s PT_pwks PT_k0to2s PT_k3to5s PT_k6to10s PT_k11to18s PT_H1s PT_H2s PT_00to08s"
global varUE      "trUE_s trUE_f1s trUE_f2s trUE_f3s"
global varEU      "trEU_s trEU_f1s trEU_f2s trEU_f3s trEU_c10s trEU_c50s trEU_c90s"
global varlWentry "ElW1_s VlW1_s ElW1_H1s ElW1_H2s pQ25lW1_s pQ50lW1_s pQ75lW1_s"
global vardlWe    "dlWdle_s"
global varlW      "lWcons_s lWH1_s lWH2_s lWlW_1_s lWle_s lWle_1_s"
global varlWslope "lWslopecons_s lWslopeH1_s lWslopeH2_s lWslopea_s"
global vardlWy    "ydlWdFT_s ydlWdPT_s ydlWdNW_s"
global varlWdist  "VrlW_s Err1_s ElW_FTs ElW_PTs pQ10lW_FTs pQ25lW_FTs pQ50lW_FTs pQ75lW_FTs pQ90lW_FTs pQ10lW_PTs pQ25lW_PTs pQ50lW_PTs pQ75lW_PTs pQ90lW_PTs ElW_H1s ElW_H2s"
global varCC      "CC_s"
global lists      "varSmean varSregr varEmean varPTmean varUE varEU varlWentry vardlWe varlW varlWslope vardlWy varlWdist varCC"

qui drop _all
qui set obs 1000
gen n=_n

foreach list of global lists {
foreach v of global `list' {
	forvalues s=1/3 {
		qui gen `v'`s'=.
		forvalues i=1/1000 {
			qui replace `v'`s' = ${`v'`s'_`i'} if n==`i'
		}
		qui sum `v'`s'
		global SE`v'`s' = r(sd)
	}
}
}
save "${moments}\moments-BS.dta", replace


** export moments to text files
foreach list of global lists {
	foreach v of global `list' {
		matrix input `v' = (${`v'1_0},  ${SE`v'1}, ${`v'2_0},  ${SE`v'2}, ${`v'3_0},  ${SE`v'3})
		matrix rownames `v' = "`v'"
	}
}

estout matrix(s_s, fmt(5)) using "${moments}\moments-Sdist.txt", replace title("! education distribution") style(fixed) mlabels(none) collabels(none)
global vars "sH10_s sH11_s sH20_s sH21_s"
foreach v of global vars {
	estout matrix(`v', fmt(5)) using "${moments}\moments-Sdist.txt", append style(fixed) mlabels(none) collabels(none)
}

estout matrix(scons_s, fmt(5)) using "${moments}\moments-Sregr.txt", replace title("! education regressions") style(fixed) mlabels(none) collabels(none)
global vars "sg80_s sf1_s sf2_s sg80_f1_s sg80_f2_s sparentYres_s"
foreach v of global vars {
	estout matrix(`v', fmt(5)) using "${moments}\moments-Sregr.txt", append style(fixed) mlabels(none) collabels(none)
}

estout matrix(E_s, fmt(5)) using "${moments}\moments-Emean.txt", replace title("! E means") style(fixed) mlabels(none) collabels(none)
global vars "E_f1s E_f2s E_f3s E_f4s E_pwks E_k0to2s E_k3to5s E_k6to10s E_k11to18s E_H1s E_H2s E_00to08s"
foreach v of global vars {
	estout matrix(`v', fmt(5)) using "${moments}\moments-Emean.txt", append style(fixed) mlabels(none) collabels(none)
}

estout matrix(PT_s, fmt(5)) using "${moments}\moments-PTmean.txt", replace title("! PT employment means") style(fixed) mlabels(none) collabels(none)
global vars "PT_f1s PT_f2s PT_f3s PT_f4s PT_pwks PT_k0to2s PT_k3to5s PT_k6to10s PT_k11to18s PT_H1s PT_H2s PT_00to08s"
foreach v of global vars {
	estout matrix(`v', fmt(5)) using "${moments}\moments-PTmean.txt", append style(fixed) mlabels(none) collabels(none)
}

estout matrix(trUE_s, fmt(5)) using "${moments}\moments-transUE.txt", replace title("! transitions U to E") style(fixed) mlabels(none) collabels(none)
global vars "trUE_f1s trUE_f2s trUE_f3s"
foreach v of global vars {
	estout matrix(`v', fmt(5)) using "${moments}\moments-transUE.txt", append style(fixed) mlabels(none) collabels(none)
}

estout matrix(trEU_s, fmt(5)) using "${moments}\moments-transEU.txt", replace title("! transitions E to U") style(fixed) mlabels(none) collabels(none)
global vars "trEU_f1s trEU_f2s trEU_f3s trEU_c10s trEU_c50s trEU_c90s"
foreach v of global vars {
	estout matrix(`v', fmt(5)) using "${moments}\moments-transEU.txt", append style(fixed) mlabels(none) collabels(none)
}

estout matrix(ElW1_s, fmt(5)) using "${moments}\moments-W0.txt", replace title("! Wage rates at entrance in WL") style(fixed) collabels(none) mlabels(none) 
global vars "VlW1_s ElW1_H1s ElW1_H2s pQ25lW1_s pQ50lW1_s pQ75lW1_s"
foreach v of global vars {
	estout matrix(`v', fmt(5)) using "${moments}\moments-W0.txt", append style(fixed) mlabels(none) collabels(none)
}

estout matrix(dlWdle_s, fmt(5)) using "${moments}\moments-WregrdWLe.txt", replace title("! Wage regressions in 1st diffs on log experience (years worked)") style(fixed) mlabels(none) collabels(none)

estout matrix(lWcons_s, fmt(5)) using "${moments}\moments-WregrWL.txt", replace title("! Wage regression - WL") style(fixed) mlabels(none) collabels(none)
global vars "lWH1_s lWH2_s lWlW_1_s lWle_s lWle_1_s"
foreach v of global vars {
	estout matrix(`v', fmt(5)) using "${moments}\moments-WregrWL.txt", append style(fixed) mlabels(none) collabels(none)
}

estout matrix(lWslopecons_s, fmt(5)) using "${moments}\moments-WregrWLslope.txt", replace title("! More wage regressions - WL") style(fixed) mlabels(none) collabels(none)
global vars "lWslopeH1_s lWslopeH2_s lWslopea_s"
foreach v of global vars {
	estout matrix(`v', fmt(5)) using "${moments}\moments-WregrWLslope.txt", append style(fixed) mlabels(none) collabels(none)
}

estout matrix(ydlWdFT_s, fmt(5)) using "${moments}\moments-WregrdWLy.txt", replace title("! Wage regressions in 1st diffs on previous working status") style(fixed) mlabels(none) collabels(none)
global vars "ydlWdPT_s ydlWdNW_s"
foreach v of global vars {
	estout matrix(`v', fmt(5)) using "${moments}\moments-WregrdWLy.txt", append style(fixed) mlabels(none) collabels(none)
}

estout matrix(VrlW_s, fmt(5)) using "${moments}\moments-WdistWL.txt", replace title("! Wage distribution - WL") style(fixed) mlabels(none) collabels(none)
global vars "Err1_s ElW_FTs ElW_PTs pQ10lW_FTs pQ25lW_FTs pQ50lW_FTs pQ75lW_FTs pQ90lW_FTs pQ10lW_PTs pQ25lW_PTs pQ50lW_PTs pQ75lW_PTs pQ90lW_PTs ElW_H1s ElW_H2s"
foreach v of global vars {
	estout matrix(`v', fmt(5)) using "${moments}\moments-WdistWL.txt", append style(fixed) mlabels(none) collabels(none)
}

estout matrix(CC_s, fmt(5)) using "${moments}\moments-CC.txt", replace title("! Prevalence of +ve CC costs") style(fixed) mlabels(none) collabels(none)


** export quantiles to text files
foreach h in FT PT {
foreach q of numlist 10 25 50 75 90 {
	matrix input Q`q'`h' = (${Q`q'lW_`h's1}, ${Q`q'lW_`h's2}, ${Q`q'lW_`h's3})
	matrix rownames Q`q'`h' = "Q`q'`h'"
}
}
matrix input QWL = (${Q10lW}, ${Q50lW}, ${Q90lW})
foreach q of numlist 25 50 75 {
	matrix input Q`q'entry = (${Q`q'lW1_s1}, ${Q`q'lW1_s2}, ${Q`q'lW1_s3})
	matrix rownames Q`q'entry = "Q`q' entry"
}

foreach h in FT PT {
	estout matrix(Q10`h', fmt(3)) using "${moments}\Q`h's.txt", replace style(fixed) mlabels(none) collabels(none)
	foreach q of numlist 25 50 75 90 {
		estout matrix(Q`q'`h', fmt(3)) using "${moments}\Q`h's.txt", append style(fixed) mlabels(none) collabels(none)
	}
}
estout matrix(QWL, fmt(3)) using "${moments}\QWL.txt", replace style(fixed) mlabels(none) collabels(none)

estout matrix(Q25entry, fmt(3)) using "${moments}\Qentry.txt", replace style(fixed) mlabels(none) collabels(none)
foreach q of numlist 50 75 {
		estout matrix(Q`q'entry, fmt(3)) using "${moments}\Qentry.txt", append style(fixed) mlabels(none) collabels(none)
}



** calculate varcov matrix: exclude sources of multicollinearity
** -----------------------------------------------------------------------------


** Education moments
** -----------------

global allEd ""
global nEd=0
forvalues s=2/3 {
	foreach v of global varSmean {
		global nEd=$nEd+1
		global allEd "${allEd} `v'`s'"
	}
	foreach v of global varSregr {
		global nEd=$nEd+1
		global allEd "${allEd} `v'`s'"
	}
}


** Wl moments
** ----------
global allWL_s "${varEmean} ${varPTmean} ${varUE} ${varEU} ${varlWentry} ${vardlWe} ${varlW} ${varlWslope} ${vardlWy} ${varlWdist} ${varCC}"

forvalues s=1/3 {
	global allWL`s' ""
	global nWL`s' = 0
	foreach v of global allWL_s {
		global nWL`s'=${nWL`s'}+1
		global allWL`s' "${allWL`s'} `v'`s'"
	}
}


** now using all moments together
** ------------------------------
qui corr $allEd $allWL1 $allWL2 $allWL3, cov
matrix V = r(C)

* set to zero covariances between education groups 
local imin = $nEd+$nWL1+1
local imax = $nEd+$nWL1+$nWL2+$nWL3
local jmin = $nEd+1
local jmax = $nEd+$nWL1
di `imin', `imax', `jmin', `jmax'
forvalues i=`imin'/`imax' {
forvalues j=`jmin'/`jmax' {
	mat V[`i',`j'] = 0
	mat V[`j',`i'] = 0
}
}

local imin = $nEd+$nWL1+$nWL2+1
local imax = $nEd+$nWL1+$nWL2+$nWL3
local jmin = $nEd+$nWL1+1
local jmax = $nEd+$nWL1+$nWL2
di `imin', `imax', `jmin', `jmax'
forvalues i=`imin'/`imax' {
forvalues j=`jmin'/`jmax' {
	mat V[`i',`j'] = 0
	mat V[`j',`i'] = 0
}
}

mat invV = invsym(V)
drop _all
svmat invV
saveold ${moments}\invV.dta, replace



