** -----------------------------------------------------------------------------
** -----------------------------------------------------------------------------
** This file uses model simulations to calculate the elasticities of LS 
** with respect to changes in earnings (table 14 and figure 9)
** Calculates both Frisch and Marshall elasticities
** -----------------------------------------------------------------------------
** -----------------------------------------------------------------------------


drop _all
clear all
do "paths.do"
global frischdta   "$sim\frisch_touse.dta"
global marshalldta "$sim\marshall_touse.dta"

global factor = 200
local year = 1999


** Prepare data
** -----------------------------------------------------------------------------

use "$sim\frisch.dta", clear
qui gen a1=19*(s<=2)+22*(s==3)
qui keep if a>=a1
qui sort iSim a
qui compress
qui save "$frischdta", replace

use "$sim\marshall.dta", clear
qui gen a1=19*(s<=2)+22*(s==3)
qui keep if a>=a1
qui sort iSim a
qui compress
qui save "$marshalldta", replace


** Routines to compute elasticities or income effects
** -----------------------------------------------------------------------------

** compute elasticities by age
cap program drop computeElast_byage
program define computeElast_byage
	qui sum dE if $condition
	global dE = r(mean)
	qui sum dEage if $condition
	global dEage = r(mean)
	global Ea = $factor*($dEage-$dE)/$dE
end

** compute elasticities, all ages together
cap program drop computeElast_allages
program define computeElast_allages

	qui sum dE0 if $condition
	global dE0 = r(mean)
	qui sum dE1 if $condition
	global dE1 = r(mean)
	global E  = $factor*($dE1-$dE0)/$dE0
	global dE = $factor*($dE1-$dE0)

	qui sum h0 if $condition & dE0==1
	global h0 = r(mean)
	qui sum h1 if $condition & dE0==1
	global h1 = r(mean)
	global h = $factor*($h1-$h0)/$h0	
	
end


** Frisch elasticities by age and education: figure 9, lhs panel
** -----------------------------------------------------------------------------

use "$frischdta", replace
sort iSim a
gen byte dE  = (inrange(l,2,3))
gen byte f   = 1*(m==0 & kd==0) + 2*(m==0 & kd==1) + 3*(m==1 & kd==0) + 4*(m==1 & kd==1)
by iSim: gen byte f_1  = f[_n-1]

* calculate elasticities
forvalues a=23/50 { 

	qui gen byte dEage  = (inrange(l`a',2,3))

	global condition "a==`a'"
	computeElast_byage
	global Ea`a'  = $Ea

	forvalues s=1/3 { 
		global condition "a==`a' & s==`s'"
		computeElast_byage
		global Es`s'a`a'  = $Ea
	}
		
	di "age `a'"
	qui drop dEage 
	
}

** prepare data
drop _all
set obs 29
gen a = _n+21

global varlist "E Es1 Es2 Es3"
foreach var of global varlist {
	gen `var'  = .
	forvalues a=23/50 { 
		qui replace `var'  = ${`var'a`a'} if a==`a'
	} 
}

* draw graph
local elast "frisch"
twoway (lpoly E   a if a>22, bwidth(5) degree(1) lcolor(navy) lpattern(solid))         ///
	   (lpoly Es1 a if a>22, bwidth(5) degree(1) lcolor(purple) lpattern(shortdash))   ///
	   (lpoly Es2 a if a>22, bwidth(5) degree(1) lcolor(lavender) lpattern(dash))      ///
	   (lpoly Es3 a if a>22, bwidth(5) degree(1) lcolor(pink) lpattern(longdash)),     ///
	   legend(label(1 "all") label(2 "secondary") label(3 "high-school") label(4 "university") region(lcolor(none))) ///
	   graphregion(fcolor(white) lcolor(white))  ///
	   xtitle("age") title("Frisch")
more


** Marshall elasticities by age and education: figure 9, rhs panel
** -----------------------------------------------------------------------------

use "$marshalldta", replace
sort iSim a
gen byte dE  = (inrange(l,2,3))
gen byte f   = 1*(m==0 & kd==0) + 2*(m==0 & kd==1) + 3*(m==1 & kd==0) + 4*(m==1 & kd==1)
by iSim: gen byte f_1  = f[_n-1]

* calculate elasticities
forvalues a=23/50 { 

	qui gen byte dEage  = (inrange(l`a',2,3))

	global condition "a==`a'"
	computeElast_byage
	global Ea`a'  = $Ea

	forvalues s=1/3 { 
		global condition "a==`a' & s==`s'"
		computeElast_byage
		global Es`s'a`a'  = $Ea
	}
		
	di "age `a'"
	qui drop dEage 
	
}

** prepare data
drop _all
set obs 29
gen a = _n+21

global varlist "E Es1 Es2 Es3"
foreach var of global varlist {
	gen `var'  = .
	forvalues a=23/50 { 
		qui replace `var'  = ${`var'a`a'} if a==`a'
	} 
}

* draw graph
twoway (lpoly E   a if a>=22, bwidth(5) degree(1) lcolor(navy) lpattern(solid))    ///
	   (lpoly Es1 a if a>=22, bwidth(5) degree(1) lcolor(purple) lpattern(shortdash))   ///
	   (lpoly Es2 a if a>=22, bwidth(5) degree(1) lcolor(lavender) lpattern(dash)) ///
	   (lpoly Es3 a if a>=22, bwidth(5) degree(1) lcolor(pink) lpattern(longdash)),    ///
	   legend(label(1 "all") label(2 "secondary") label(3 "further") label(4 "higher") region(lcolor(none))) ///
	   graphregion(fcolor(white) lcolor(white)) ylabel(0 .2 .4 .6 .8 1) xtitle("age") title("Marshall")
more



** Frisch elasticities by education and family composition and exports to tex
** file: table 14, lhs panel
** -----------------------------------------------------------------------------

** prepare data
drop _all
use "$frischdta", replace
keep iSim rep a th s m kd akd l l* 
rename l lbase
reshape long l, i(iSim rep a) j(aFrisch)
rename lbase l0

keep if inrange(a-aFrisch,-1,0)
sort iSim rep aFrisch a

gen byte dE0  = (inrange(l0,2,3))
gen byte dE1  = (inrange(l,2,3))
gen h0        = (l0-1)*20
gen h1        = (l-1)*20

gen byte f = 1*(m==0 & kd==0) + 2*(m==0 & kd==1) + 3*(m==1 & kd==0) + 4*(m==1 & kd==1)
gen dh = (h1-h0)/h0


** compute elasticities for different groups

* all women
global condition "aFrisch==a"
computeElast_allages
global Eall  = $E
global dEall = $dE
global hall = $h

* by education
forvalues s=1/3 { 
    global condition "aFrisch==a & s==`s'"
    computeElast_allages
    global Es`s'  = $E
    global dEs`s' = $dE
 	global hs`s'  = $h
}

* by family composition
forvalues f=1/4 { 
    global condition "aFrisch==a & f==`f'"
    computeElast_allages
    global Ef`f'  = $E
    global dEf`f' = $dE
    global hf`f'  = $h
}

* write to tex file

cap file close Ofile
file open  Ofile using "$tables\elasticities.tex", write replace

file write Ofile _n 
file write Ofile _n 
file write Ofile "\begin{table}[h!]\centering" _n	
file write Ofile "\caption{Elasticities of labor supply: Frisch}" _n
file write Ofile "{\small\begin{tabular}{lccc}" _n
file write Ofile "\hline\hline" _n
file write Ofile "& \multicolumn{3}{c}{Frisch} &  & \multicolumn{3}{c}{Marshall} \\" _n
file write Ofile "\cline{2-4}\cline{6-8}" _n
file write Ofile "& elasticity & derivative & elasticity &  & elasticity& derivative & elasticity \\" _n
file write Ofile "\hline\hline" _n
	file write Ofile %30s  "All women          & " %6.3f (${Eall}) %3s " & " %6.3f (${dEall}) %3s " & " %6.3f (${hall}) %3s " \\"  _n
	file write Ofile "\hline" _n	
	file write Ofile %30s  "Secondary          & " %6.3f (${Es1})  %3s " & " %6.3f (${dEs1})  %3s " & " %6.3f (${hs1})  %3s " \\"  _n
	file write Ofile %30s  "High school        & " %6.3f (${Es2})  %3s " & " %6.3f (${dEs2})  %3s " & " %6.3f (${hs2})  %3s " \\"  _n
	file write Ofile %30s  "University         & " %6.3f (${Es3})  %3s " & " %6.3f (${dEs3})  %3s " & " %6.3f (${hs3})  %3s " \\"  _n
	file write Ofile "\hline" _n	
	file write Ofile %30s  "Singles, no kids   & " %6.3f (${Ef1})  %3s " & " %6.3f (${dEf1})  %3s " & " %6.3f (${hf1})  %3s " \\"  _n
	file write Ofile %30s  "Single mothers     & " %6.3f (${Ef2})  %3s " & " %6.3f (${dEf2})  %3s " & " %6.3f (${hf2})  %3s " \\"  _n
	file write Ofile %30s  "Couples, no kids   & " %6.3f (${Ef3})  %3s " & " %6.3f (${dEf3})  %3s " & " %6.3f (${hf3})  %3s " \\"  _n
	file write Ofile %30s  "Mothers in couples & " %6.3f (${Ef4})  %3s " & " %6.3f (${dEf4})  %3s " & " %6.3f (${hf4})  %3s " \\"  _n
file write Ofile "\hline\hline" _n	
file write Ofile "\end{tabular}}" _n	
file write Ofile "\end{table}" _n	



** Marshall elasticities by education and family composition and exports to tex
** file: table 14, lhs panel
** -----------------------------------------------------------------------------

** prepare data
drop _all
use "$marshalldta", replace
keep iSim rep a th s m kd akd l l* 
rename l lbase
reshape long l, i(iSim rep a) j(aMarshall)
rename lbase l0

keep if inrange(a-aMarshall,-1,0)
sort iSim rep aMarshall a

gen byte dE0  = (inrange(l0,2,3))
gen byte dE1  = (inrange(l,2,3))
gen h0        = (l0-1)*20
gen h1        = (l-1)*20

gen byte f = 1*(m==0 & kd==0) + 2*(m==0 & kd==1) + 3*(m==1 & kd==0) + 4*(m==1 & kd==1)
gen dh = (h1-h0)/h0


** compute elasticities for different groups

* all women
global condition "aMarshall==a"
computeElast_allages
global Eall  = $E
global dEall = $dE
global hall = $h

* by education
forvalues s=1/3 { 
    global condition "aMarshall==a & s==`s'"
    computeElast_allages
    global Es`s'  = $E
    global dEs`s' = $dE
 	global hs`s'  = $h
}

* by family composition
forvalues f=1/4 { 
    global condition "aMarshall==a & f==`f'"
    computeElast_allages
    global Ef`f'  = $E
    global dEf`f' = $dE
    global hf`f'  = $h
}


* write to tex file

file write Ofile _n 
file write Ofile _n 
file write Ofile "\begin{table}[h!]\centering" _n	
file write Ofile "\caption{Elasticities of labor supply: Marshall}" _n
file write Ofile "{\small\begin{tabular}{lccc}" _n
file write Ofile "\hline\hline" _n
file write Ofile "& \multicolumn{3}{c}{Frisch} &  & \multicolumn{3}{c}{Marshall} \\" _n
file write Ofile "\cline{2-4}\cline{6-8}" _n
file write Ofile "& elasticity & derivative & elasticity &  & elasticity& derivative & elasticity \\" _n
file write Ofile "\hline\hline" _n
	file write Ofile %30s  "All women          & " %6.3f (${Eall}) %3s " & " %6.3f (${dEall}) %3s " & " %6.3f (${hall}) %3s " \\"  _n
	file write Ofile "\hline" _n	
	file write Ofile %30s  "Secondary          & " %6.3f (${Es1})  %3s " & " %6.3f (${dEs1})  %3s " & " %6.3f (${hs1})  %3s " \\"  _n
	file write Ofile %30s  "High school        & " %6.3f (${Es2})  %3s " & " %6.3f (${dEs2})  %3s " & " %6.3f (${hs2})  %3s " \\"  _n
	file write Ofile %30s  "University         & " %6.3f (${Es3})  %3s " & " %6.3f (${dEs3})  %3s " & " %6.3f (${hs3})  %3s " \\"  _n
	file write Ofile "\hline" _n	
	file write Ofile %30s  "Singles, no kids   & " %6.3f (${Ef1})  %3s " & " %6.3f (${dEf1})  %3s " & " %6.3f (${hf1})  %3s " \\"  _n
	file write Ofile %30s  "Single mothers     & " %6.3f (${Ef2})  %3s " & " %6.3f (${dEf2})  %3s " & " %6.3f (${hf2})  %3s " \\"  _n
	file write Ofile %30s  "Couples, no kids   & " %6.3f (${Ef3})  %3s " & " %6.3f (${dEf3})  %3s " & " %6.3f (${hf3})  %3s " \\"  _n
	file write Ofile %30s  "Mothers in couples & " %6.3f (${Ef4})  %3s " & " %6.3f (${dEf4})  %3s " & " %6.3f (${hf4})  %3s " \\"  _n
file write Ofile "\hline\hline" _n	
file write Ofile "\end{tabular}}" _n	
file write Ofile "\end{table}" _n	

file close Ofile
