** -----------------------------------------------------------------------------
** -----------------------------------------------------------------------------
** This file uses model simulations to calculate impact of giveaway reforms
**   through a drop in the basic tax rate, an increase in the tax credit award
**   or an increase in the income support award (values in table 18)
** -----------------------------------------------------------------------------
** -----------------------------------------------------------------------------

drop _all
clear all
do "paths.do"


** prepare data
** -----------------------------------------------------------------------------

global varlist "s m kd y ym inc l EV EVpreS EVpostSs1-EVpostSs3"

* baseline system: 2002
use $sim\sys02.dta, clear
keep i rep a $varlist
gen system  = 1
compress
save "$sim\giveaway.dta", replace

* change basic tax rate
use $sim\sys02_tax.dta, clear
keep i rep a $varlist
gen system = 2
compress
append using "$sim\giveaway.dta"
save "$sim\giveaway.dta", replace

* change TC award
use $sim\sys02_TC.dta, clear
keep i rep a $varlist
gen system = 3
compress
append using "$sim\giveaway.dta"
save "$sim\giveaway.dta", replace
	
* change TC award
use $sim\sys02_IS.dta, clear
keep i rep a $varlist
gen system = 4
compress
append using "$sim\giveaway.dta"

label define system 1 "2002" 2 "basic tax" 3 "TC award" 4 "IS award"
label values system system

* demographic and employment variables
gen a1=19+3*(s==3)
gen byte dE  = (l>=2) if a>=a1
gen byte dFT = (l==3) if a>=a1
gen byte dPT = (l==2) if a>=a1
gen byte f = 1*(m==0 & kd==0) + 2*(m==0 & kd==1) + 3*(m==1 & kd==0) + 4*(m==1 & kd==1)
drop l m kd

* labels
label var s "education attainment"
label define s 1 "second" 2 "high" 3 "univ", modify
label values s s
	
sort i rep system a
by i rep: gen s2002=s[1]
label var s2002 "education in 2002"
label values s2002 s

label var f "family composition"
label define f 1 "single" 2 "single mother" 3 "couple" 4 "couple w/ kids", modify
label values f f
	
save "$sim\giveaway.dta", replace



** Calculate effects on total earned income and welfare
** -----------------------------------------------------------------------------

local year = 2002


** prepare data
** ------------
use "$sim\giveaway.dta", replace
		
** wealth
gen ginc = y+ym
rename inc ninc
global interest = 1/1.015
qui sort system i rep a
qui gen dginc = $interest^(a-19)*ginc
qui gen dninc = $interest^(a-19)*ninc
qui by system i rep: egen gW = sum(dginc)
qui by system i rep: egen nW = sum(dninc)
drop y ym ginc ninc dginc dninc

qui keep if a==a1
qui sort i rep system
	
qui by i rep: gen gW`year'=gW[1]
qui by i rep: gen nW`year'=nW[1]
qui gen cgW = 100*(gW-gW`year')/gW`year'
qui gen cnW = 100*(nW-nW`year')/nW`year'	
		
** consumption compensation
gen EVpostS = (s==1)*EVpostSs1+(s==2)*EVpostSs2+(s==3)*EVpostSs3
by i rep: gen EVpreS`year'     = EVpreS[1]
by i rep: gen EVpostS`year'    = (s[1]==1)*EVpostSs1[1]+(s[1]==2)*EVpostSs2[1]+(s[1]==3)*EVpostSs3[1]
	
gen cCpreS   = 100*(1-(EVpreS`year'/EVpreS)^(-1/0.56))
gen cCpostS  = 100*(1-(EVpostS`year'/EVpostS)^(-1/0.56))


	
** calculate effects and store
** ---------------------------

global varlist "cgW cnW cCpostS"
global Nsys = 4

forvalues sys=1/$Nsys {
foreach var of global varlist {
	forvalues s=1/3 {
		qui sum `var' if system==`sys' & s2002==`s'
		global `var'`sys's`s' = r(mean)
		if `sys'>1  global d`var'`sys's`s' = ${`var'`sys's`s'}-${`var'1s`s'}
	}
	qui sum cCpreS if system==`sys'
	global cCpreS`sys' = r(mean)
	if `sys'>1  global dcCpreS`sys' = ${cCpreS`sys'}-${cCpreS1}
}
}


** Export tables to text file (latex format)
** -----------------------------------------------------------------------------

* table 18

cap file close Ofile
file open  Ofile using "$tables\giveaway.tex", write replace
file write Ofile _n
file write Ofile _n
file write Ofile "\begin{table}[h!]\centering" _n	
file write Ofile "\caption{Impacts of an exogenous increase in public spending distributed through alternative routes}" _n
file write Ofile "{\small\begin{tabular}{ll*{3}{c}}" _n
file write Ofile "\hline\hline" _n
file write Ofile "&& Secondary  & High School & University \\" _n
file write Ofile "\hline\hline" _n
local s = 2
	file write Ofile "\multicolumn{5}{l}{Adjustment in basic tax rate} \\" _n
	file write Ofile %40s "(1) & Lifetime gross earnings    & " %6.2f (${dcgW`s's1})     %3s " & " %6.2f (${dcgW`s's2})     %3s " & " %6.2f (${dcgW`s's3})     %3s " \\" _n 
	file write Ofile %40s "(2) & Lifetime disposable income & " %6.2f (${dcnW`s's1})     %3s " & " %6.2f (${dcnW`s's2})     %3s " & " %6.2f (${dcnW`s's3})     %3s " \\" _n 
	file write Ofile %40s "(3) & Welfare (post-education)   & " %6.2f (${dcCpostS`s's1}) %3s " & " %6.2f (${dcCpostS`s's2}) %3s " & " %6.2f (${dcCpostS`s's3}) %3s " \\" _n 
	file write Ofile %50s "(3) & Welfare (pre-education)    & \multicolumn{3}{c}{" %6.2f (${dcCpreS`s'}) %4s "} \\" _n 
	file write Ofile "\hline" _n
local s = 3
	file write Ofile "\multicolumn{11}{l}{Adjustment in tax credits maximum award} \\" _n
	file write Ofile %40s "(1) & Lifetime gross earnings    & " %6.2f (${dcgW`s's1})     %3s " & " %6.2f (${dcgW`s's2})     %3s " & " %6.2f (${dcgW`s's3})     %3s " \\" _n 
	file write Ofile %40s "(2) & Lifetime disposable income & " %6.2f (${dcnW`s's1})     %3s " & " %6.2f (${dcnW`s's2})     %3s " & " %6.2f (${dcnW`s's3})     %3s " \\" _n 
	file write Ofile %40s "(3) & Welfare (post-education)   & " %6.2f (${dcCpostS`s's1}) %3s " & " %6.2f (${dcCpostS`s's2}) %3s " & " %6.2f (${dcCpostS`s's3}) %3s " \\" _n 
	file write Ofile %50s "(3) & Welfare (pre-education)    & \multicolumn{3}{c}{" %6.2f (${dcCpreS`s'}) %4s "} \\" _n 
	file write Ofile "\hline" _n
local s = 4
	file write Ofile "\multicolumn{11}{l}{Adjustment in IS award} \\" _n
	file write Ofile %40s "(1) & Lifetime gross earnings    & " %6.2f (${dcgW`s's1})     %3s " & " %6.2f (${dcgW`s's2})     %3s " & " %6.2f (${dcgW`s's3})     %3s " \\" _n 
	file write Ofile %40s "(2) & Lifetime disposable income & " %6.2f (${dcnW`s's1})     %3s " & " %6.2f (${dcnW`s's2})     %3s " & " %6.2f (${dcnW`s's3})     %3s " \\" _n 
	file write Ofile %40s "(3) & Welfare (post-education)   & " %6.2f (${dcCpostS`s's1}) %3s " & " %6.2f (${dcCpostS`s's2}) %3s " & " %6.2f (${dcCpostS`s's3}) %3s " \\" _n 
	file write Ofile %50s "(3) & Welfare (pre-education)    & \multicolumn{3}{c}{" %6.2f (${dcCpreS`s'}) %4s "} \\" _n 
file write Ofile "\hline\hline" _n
file write Ofile "\end{tabular}}" _n	
file write Ofile "\end{table}" _n	
file close Ofile
