** -----------------------------------------------------------------------------
** -----------------------------------------------------------------------------
** This file uses model simulations to calculate impact of two large reforms 
** to the 2002 WFTC:
** 1. removing tax credits altogether (values in tables 15 and 16)
** 2. assessing tax credits eligibility on individual (rather than family) 
**    income (values in table 17)
**
** Results exported to a .tex file
** -----------------------------------------------------------------------------
** -----------------------------------------------------------------------------


drop _all
clear all
do "paths.do"



** Construct dataset with all systems
** -----------------------------------------------------------------------------

global varlist "e s m kd akd y ym inc l w EV EVpreS EVpostSs1-EVpostSs3 sampleS insample scotland wales year k"

** read first simulations under 2002 system (system=1)
use "$sim\sys02.dta", clear
keep i rep a $varlist
gen system = 1
qui compress
save "$sim\largereforms.dta", replace

** append simulations under 2002 exc TC system (system=2)
use "$sim\sys02excTC_RN02.dta", clear
keep i rep a $varlist
gen system = 2
qui compress
append using "$sim\largereforms.dta"
save "$sim\largereforms.dta", replace

** append simulations under 2002 with TC eligibility assessed on indiv income (system=3)
use "$sim\sys02indTC_RN02.dta", clear
keep i rep a $varlist
gen system = 3
qui compress
append using "$sim\largereforms.dta"
save "$sim\largereforms.dta", replace


** create some new variables, clean and save
** -----------------------------------------------------------------------------

* age
gen a1=19+3*(s==3)
keep if a>=a1

* employment
gen byte dE  = (l>=2) if a>=a1
gen byte dFT = (l==3) if a>=a1
gen byte dPT = (l==2) if a>=a1

* family composition
gen byte f = 1*(m==0 & kd==0) + 2*(m==0 & kd==1) + 3*(m==1 & kd==0) + 4*(m==1 & kd==1)
label var f "family composition"
label define f 1 "single" 2 "single mother" 3 "couple" 4 "couple w/ kids", modify
label values f f

* age of youngest child
sort system rep i a
forvalues a=23/59 {
	qui replace akd=akd[_n-1]+1 if kd==0 & akd[_n-1]>0 & a==`a'
}
	
* cumulated years as LM or mother in couple
by system rep i: gen yrsLM = sum(f==2)
by system rep i: gen yrsCM = sum(f==4)

* labels
sort system rep i a
label define system 1 "2002 (baseline)" 2 "2002 without TC" 3 "2002 with individual TC"
label values system system

label var s "education"
label define s 1 "sec" 2 "HS" 3 "uni", modify
label values s s
	
sort i rep a system
by i rep a: gen s2002 = s[1]
label var s2002 "education under 2002 system"
label values s2002 s
	
compress
save "$sim\largereforms.dta", replace



** Set conditions
** -----------------------------------------------------------------------------

global m0  "f==2"
global m1  "f==4"
global m2  "inlist(f,2,4)"

global LM18 "akd>18 & yrsCM==0"



** Employment effects
** -----------------------------------------------------------------------------

use "$sim\largereforms.dta", replace
keep if inrange(a,22,50) // scotland==0 & wales==0 &
global vars "E PT FT" 

foreach var of global vars {

	di "var ", "`var'"

	forvalues sys=1/3 {
	forvalues s=1/3 {
	
		* mothers of dependent children
		forvalues m=0/1 {
			qui sum d`var' if ${m`m'} & inrange(akd,0,18) & s2002==`s' & system==`sys' 
			global `var'`sys's`s'm`m'kd = r(mean)
			if `sys'==2  global d`var'`sys's`s'm`m'kd = -${`var'`sys's`s'm`m'kd}+${`var'1s`s'm`m'kd}
			if `sys'==3  global d`var'`sys's`s'm`m'kd =  ${`var'`sys's`s'm`m'kd}-${`var'1s`s'm`m'kd}
		}

		* mothers of adult children
		qui sum d`var' if akd>18 & s2002==`s' & system==`sys'
		global `var'`sys's`s'kd19 = r(mean)
		if `sys'==2  global d`var'`sys's`s'kd19 = -${`var'`sys's`s'kd19}+${`var'1s`s'kd19}
		if `sys'==3  global d`var'`sys's`s'kd19 =  ${`var'`sys's`s'kd19}-${`var'1s`s'kd19}

		* mothers of adult children who have always been a lone mother
		qui sum d`var' if ${LM18} & s2002==`s' & system==`sys'
		global `var'`sys's`s'LM = r(mean)
		if `sys'==2  global d`var'`sys's`s'LM = -${`var'`sys's`s'LM}+${`var'1s`s'LM}
		if `sys'==3  global d`var'`sys's`s'LM =  ${`var'`sys's`s'LM}-${`var'1s`s'LM}

	}
	}
	
}


** Impacts on education
** -----------------------------------------------------------------------------

use "$sim\largereforms.dta", replace
keep if a==25 & wales==0 & scotland==0

qui ta s, g(s)
forvalues s=1/3 {
	forvalues sys=1/3 {
		qui sum s`s' if system==`sys'
		global s`s'sys`sys'=r(mean)
		if `sys'==2  global ds`s'sys`sys'= -${s`s'sys`sys'}+${s`s'sys1}
		if `sys'==3  global ds`s'sys`sys' = ${s`s'sys`sys'}-${s`s'sys1}
	}
}


** Effects on experience
** -----------------------------------------------------------------------------

use "$sim\largereforms.dta", replace
keep if akd>18
gen le = ln(e+1)

forvalues s=1/3 {
forvalues sys=1/3 {
	
	qui sum e if system==`sys' & s2002==`s' & akd==19
	global e`sys's`s'kd19 = r(mean)
	if `sys'==2  global de`sys's`s'kd19  = 100*(-ln(${e`sys's`s'kd19})+ln(${e1s`s'kd19}))
	if `sys'==3  global de`sys's`s'kd19  = 100*( ln(${e`sys's`s'kd19})-ln(${e1s`s'kd19}))

	qui sum e if system==`sys' & s2002==`s' & akd==19 & $LM18
	global e`sys's`s'LM = r(mean)
	if `sys'==2  global de`sys's`s'LM = 100*(-ln(${e`sys's`s'LM})+ln(${e1s`s'LM}))
	if `sys'==3  global de`sys's`s'LM = 100*( ln(${e`sys's`s'LM})-ln(${e1s`s'LM}))

}
}

global g1 = 0.152
global g2 = 0.229
global g3 = 0.306


** Effects on savings 
** -----------------------------------------------------------------------------

use "$sim\largereforms.dta", replace

keep if a>22
global k0 "akd==-1"
global k1 "inrange(akd,0,18)"
global k2 "akd>18"

forvalues s=1/3 {
forvalues sys=1/3 {

	forvalues kd=0/2 {
		qui sum k if system==`sys' & s2002==`s' & ${k`kd'}
		global k`sys's`s'kd`kd' = r(mean)
		if `sys'==2  global dk`sys's`s'kd`kd' = 100*( -ln(${k`sys's`s'kd`kd'})+ln(${k1s`s'kd`kd'}) )
		if `sys'==3  global dk`sys's`s'kd`kd' = 100*(  ln(${k`sys's`s'kd`kd'})-ln(${k1s`s'kd`kd'}) )
	}

	qui sum k if system==`sys' & s2002==`s' & akd==19 & $LM18
	global k`sys's`s'LM = r(mean)
	if `sys'==2  global dk`sys's`s'LM = 100*( -ln(${k`sys's`s'LM})+ln(${k1s`s'LM}) )
	if `sys'==3  global dk`sys's`s'LM = 100*(  ln(${k`sys's`s'LM})-ln(${k1s`s'LM}) )

}
}


** Effects on disposable income and welfare
** -----------------------------------------------------------------------------

use "$sim\largereforms.dta", replace

sort system i rep a
qui by system i rep: replace yrsLM = yrsLM[_N]
qui by system i rep: replace yrsCM = yrsCM[_N]

** disposable income
global interest = 1/1.015
qui sort system i rep a
qui gen dinc = $interest^(a-19)*inc
qui by system i rep: egen nW = sum(dinc)
drop inc dinc

qui keep if a==a1
qui sort i rep system
	
qui by i rep: gen nW2002 = nW[1]
qui gen     cnW = 100*(nW-nW2002)/nW2002 if system==3
qui replace cnW = 100*(-nW+nW2002)/nW2002 if system==2

** consumption compensation
qui gen EVpostS = (s==1)*EVpostSs1+(s==2)*EVpostSs2+(s==3)*EVpostSs3
qui by i rep: gen EVpostS2002 = (s[1]==1)*EVpostSs1[1] + (s[1]==2)*EVpostSs2[1] + (s[1]==3)*EVpostSs3[1]
qui gen cCpostS = 100*(1-(EVpostS2002/EVpostS)^(-1/0.56)) if system==3
qui replace cCpostS = 100*(1-(EVpostS/EVpostS2002)^(-1/0.56)) if system==2

** effects
global varlist "cnW cCpostS"
forvalues sys=2/3 {
forvalues s=1/3 {
	
	foreach var of global varlist {
		qui sum `var' if system==`sys' & s2002==`s'
		global `var'`sys's`s' = r(mean)
	}
	
	qui sum cnW if system==`sys' & s2002==`s' & yrsCM==0 & yrsLM>0
	global cnW`sys's`s'LM = r(mean)
	
}
}


** Export tables to text file (latex format)
** -----------------------------------------------------------------------------

* tables 15 & 17

cap file close Ofile
file open  Ofile using "$tables\LargeReforms.tex", write replace
forvalues sys = 2/3 {
	file write Ofile _n
	file write Ofile _n
	file write Ofile "\begin{table}[h!]\centering" _n	
	if `sys'==2  file write Ofile "\caption{Effects of tax credits}" _n
	if `sys'==3  file write Ofile "\caption{Effects of assessing tax credits at the individual level - integrated with teh 2002 tax and benefit system}" _n
	file write Ofile "{\small\begin{tabular}{ll*{8}{c}}" _n
	file write Ofile "\hline\hline" _n
	file write Ofile "&& \multicolumn{8}{c}{Pre-reform education choice} \\" _n
	file write Ofile "&& \multicolumn{2}{c}{Secondary} && \multicolumn{2}{c}{High School} && \multicolumn{2}{c}{University} \\" _n
	file write Ofile "\hline\hline" _n
	file write Ofile "&& \multicolumn{8}{c}{Impact on Employment: Mothers of Dependent Children (0-18)}\\" _n
	file write Ofile "&& Single & Married && Single & Married && Single & Married \\" _n
	file write Ofile "\cline{3-4}\cline{6-7}\cline{9-10}" _n
		file write Ofile %50s " & All (pp)       & " %6.1f (100*${dE`sys's1m0kd})  %3s " & " %6.1f (100*${dE`sys's1m1kd})  %4s " && " %6.1f (100*${dE`sys's2m0kd})  %3s " & " %6.1f (100*${dE`sys's2m1kd})  %4s " && " %6.1f (100*${dE`sys's3m0kd})  %3s " & " %6.1f (100*${dE`sys's3m1kd})  %3s " \\" _n
		file write Ofile %50s " & Full-time (pp) & " %6.1f (100*${dFT`sys's1m0kd}) %3s " & " %6.1f (100*${dFT`sys's1m1kd}) %4s " && " %6.1f (100*${dFT`sys's2m0kd}) %3s " & " %6.1f (100*${dFT`sys's2m1kd}) %4s " && " %6.1f (100*${dFT`sys's3m0kd}) %3s " & " %6.1f (100*${dFT`sys's3m1kd}) %3s " \\" _n
		file write Ofile %50s " & Part-time (pp) & " %6.1f (100*${dPT`sys's1m0kd}) %3s " & " %6.1f (100*${dPT`sys's1m1kd}) %4s " && " %6.1f (100*${dPT`sys's2m0kd}) %3s " & " %6.1f (100*${dPT`sys's2m1kd}) %4s " && " %6.1f (100*${dPT`sys's3m0kd}) %3s " & " %6.1f (100*${dPT`sys's3m1kd}) %3s " \\" _n
	file write Ofile "\hline\hline" _n
	file write Ofile "&& \multicolumn{8}{c}{&& \multicolumn{8}{c}{Impact on Employment: Mothers of Adult Children (19+)} \\" _n
		file write Ofile %50s " & All (pp)       & \multicolumn{2}{c}{" %6.1f (100*${dE`sys's1kd19})  %24s "} && \multicolumn{2}{c}{" %6.1f (100*${dE`sys's2kd19})  %24s "} && \multicolumn{2}{c}{" %6.1f (100*${dE`sys's3kd19})  %4s "} \\" _n
		file write Ofile %50s " & Full-time (pp) & \multicolumn{2}{c}{" %6.1f (100*${dFT`sys's1kd19}) %24s "} && \multicolumn{2}{c}{" %6.1f (100*${dFT`sys's2kd19}) %24s "} && \multicolumn{2}{c}{" %6.1f (100*${dFT`sys's3kd19}) %4s "} \\" _n
		file write Ofile %50s " & Part-time (pp) & \multicolumn{2}{c}{" %6.1f (100*${dPT`sys's1kd19}) %24s "} && \multicolumn{2}{c}{" %6.1f (100*${dPT`sys's2kd19}) %24s "} && \multicolumn{2}{c}{" %6.1f (100*${dPT`sys's3kd19}) %4s "} \\" _n
	file write Ofile "\hline\hline" _n
	file write Ofile "&& \multicolumn{8}{c}{&& \multicolumn{8}{c}{Impact on Education and Wages} \\" _n
		file write Ofile %50s " & Education (pp) & \multicolumn{2}{c}{" %6.2f (100*${ds1sys`sys'})   %24s "} && \multicolumn{2}{c}{" %6.2f (100*${ds2sys`sys'})   %24s "} && \multicolumn{2}{c}{" %6.2f (100*${ds3sys`sys'})   %4s "} \\" _n
		file write Ofile %50s " & Wages (\%)     & \multicolumn{2}{c}{" %6.2f ($g1*${de`sys's1kd19}) %24s "} && \multicolumn{2}{c}{" %6.2f ($g2*${de`sys's2kd19}) %24s "} && \multicolumn{2}{c}{" %6.2f ($g3*${de`sys's3kd19}) %4s "} \\" _n
	file write Ofile "\hline\hline" _n
	file write Ofile "&& \multicolumn{8}{c}{&& \multicolumn{8}{c}{Impact on Assets (\%)} \\" _n
		file write Ofile %50s " & No children            & \multicolumn{2}{c}{" %6.1f (${dk`sys's1kd0}) %24s "} && \multicolumn{2}{c}{" %6.1f (${dk`sys's2kd0}) %24s "} && \multicolumn{2}{c}{" %6.1f (${dk`sys's3kd0}) %4s "} \\" _n
		file write Ofile %50s " & Dependent child (0-18) & \multicolumn{2}{c}{" %6.1f (${dk`sys's1kd1}) %24s "} && \multicolumn{2}{c}{" %6.1f (${dk`sys's2kd1}) %24s "} && \multicolumn{2}{c}{" %6.1f (${dk`sys's3kd1}) %4s "} \\" _n
		file write Ofile %50s " & Adult child (19+)      & \multicolumn{2}{c}{" %6.1f (${dk`sys's1kd2}) %24s "} && \multicolumn{2}{c}{" %6.1f (${dk`sys's2kd2}) %24s "} && \multicolumn{2}{c}{" %6.1f (${dk`sys's3kd2}) %4s "} \\" _n
	file write Ofile "\hline\hline" _n
	file write Ofile "&& \multicolumn{8}{c}{&& \multicolumn{8}{c}{Impact on Lifetime Disposable Income and Welfare \\" _n
		file write Ofile %50s " & Disposable income (\%) & \multicolumn{2}{c}{" %6.2f (${cnW`sys's1})     %24s "} && \multicolumn{2}{c}{" %6.2f (${cnW`sys's2})     %24s "} && \multicolumn{2}{c}{" %6.2f (${cnW`sys's3})     %4s "} \\" _n
		file write Ofile %50s " & Consumption equiv (\%) & \multicolumn{2}{c}{" %6.2f (${cCpostS`sys's1}) %24s "} && \multicolumn{2}{c}{" %6.2f (${cCpostS`sys's2}) %24s "} && \multicolumn{2}{c}{" %6.2f (${cCpostS`sys's3}) %4s "} \\" _n
	file write Ofile "\hline\hline" _n	
	file write Ofile "\end{tabular}}" _n	
	file write Ofile "\end{table}" _n	
}
file close Ofile


* table 16

cap file close Ofile
file open  Ofile using "$tables\LargeReforms-LM.tex", write replace

file write Ofile _n 
file write Ofile _n 
file write Ofile "\begin{table}[h!]\centering" _n	
file write Ofile "\caption{Effects of tax credits on mothers of 19 year olds who have always been a lone-mother}" _n
file write Ofile "{\small\begin{tabular}{ll*{3}{c}}" _n
file write Ofile "\hline\hline" _n
file write Ofile "&& Secondary  & High School & University \\" _n
file write Ofile "\hline\hline" _n
	file write Ofile %50s " & All employment when child is 19+ (pp)       & " %6.1f (100*${dE2s1LM})  %3s " & " %6.1f (100*${dE2s2LM})  %3s " & " %6.1f (100*${dE2s3LM})  %3s " \\" _n
	file write Ofile %50s " & Part-time employment when child is 19+ (pp) & " %6.1f (100*${dPT2s1LM}) %3s " & " %6.1f (100*${dPT2s2LM}) %3s " & " %6.1f (100*${dPT2s3LM}) %3s " \\" _n
	file write Ofile %50s " & Full-time employment when child is 19+ (pp) & " %6.1f (100*${dFT2s1LM}) %3s " & " %6.1f (100*${dFT2s2LM}) %3s " & " %6.1f (100*${dFT2s3LM}) %3s " \\" _n
	file write Ofile %50s " & Wages when child is 19 (\%)                 & " %6.1f ($g1*${de2s1LM})  %3s " & " %6.1f ($g2*${de2s2LM})  %3s " & " %6.1f ($g3*${de2s3LM})  %3s " \\" _n
	file write Ofile %50s " & Assets when child is 19 (\%)                & " %6.1f (${dk2s1LM})      %3s " & " %6.1f (${dk2s2LM})      %3s " & " %6.1f (${dk2s3LM})      %3s " \\" _n
	file write Ofile %50s " & Lifetime disposable income (\%)             & " %6.1f (${cnW2s1LM})     %3s " & " %6.1f (${cnW2s2LM})     %3s " & " %6.1f (${cnW2s3LM})     %3s " \\" _n
file write Ofile "\hline\hline" _n	
file write Ofile "\end{tabular}}" _n	
file write Ofile "\end{table}" _n	
file close Ofile


