! -----------------------------------------------------------------------------------------------------------
! -----------------------------------------------------------------------------------------------------------
! This file contains the routines to simulate the effects of policy reforms
! Simulated life-cycle profiles exported to stata
! -----------------------------------------------------------------------------------------------------------
! -----------------------------------------------------------------------------------------------------------


module PolicyExperimentsFcs

use parametersfcs
use wrapperMod,    only : getstartedsimulation, getstartedsolution, SolveAndSim_1Sys, SolveAndSim_manySys
use IOpaths
use fortax_data
use simMod,        only : writesimstata
use precision,     only : wp, tol
use constants
use parameters
use state
use SimConditions

contains



    ! -------------------------------------------------------------------------------------------------------
    ! Immediate impact of 1999-2002 reforms

    subroutine simReforms1999ST
        implicit none

        ! Before policy reform
        sysN = 9
        if (allocated(sysD))  deallocate(sysD);   allocate(sysD(sysN))   !system dates
        if (allocated(sysY0)) deallocate(sysY0);  allocate(sysY0(sysN))  !1st year of system
        if (allocated(sysY1)) deallocate(sysY1);  allocate(sysY1(sysN))  !last year of system
        sysD  = (/ 19910407, 19920407, 19930407, 19940407, 19950407, 19960407, 19970407, 19980407, 19990407 /) 
        sysY0 = (/ 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999 /)
        sysY1 = (/ 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 2008 /)                              
        call SolveAndSim_manySys(1, simpath//'sys91to99.dta', len(simpath//'sys91to99.dta'), 1)
        write (*,*) 'systems 1991-1999'

        ! After policy reform
        sysN = 18
        if (allocated(sysD))  deallocate(sysD);   allocate(sysD(sysN))   !system dates
        if (allocated(sysY0)) deallocate(sysY0);  allocate(sysY0(sysN))  !1st year of system
        if (allocated(sysY1)) deallocate(sysY1);  allocate(sysY1(sysN))  !last year of system
        sysD  = (/ 19910407, 19920407, 19930407, 19940407, 19950407, 19960407, 19970407, 19980407, 19990407, &
                   20000407, 20010407, 20020407, 20030407, 20040407, 20050407, 20060407, 20070407, 20080407/) 
        sysY0 = (/ 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 /)
        sysY1 = (/ 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 /)                              
        call SolveAndSim_manySys(1, simpath//'sys91to08.dta', len(simpath//'sys91to08.dta'), 1)
        write (*,*) 'systems 1991-2008'

    end subroutine simReforms1999ST



    ! -------------------------------------------------------------------------------------------------------
    ! Impact of 1999-2002 reforms in the LT, under revenue neutrality
    
    subroutine simReforms1999LT
        implicit none
        integer            :: i, rep, MixedAcc, chooseS, dRN
        real(wp)           :: tax(nind,nrsim), PB0, finalt


        ! Initialise parameters, grids and simulation matrix
        sysN     = 1
        MixedAcc = 1
        dRN      = 1
        chooseS  = 1
        call getstartedsolution(1.0_wp)
        call getstartedsimulation(defaultseed)

        ! Solve and simulate under 1999 tax system
        sys = allsys(1999)                                                     ! load tax-benefits system
        call SolveAndSim_1Sys(chooseS, MixedAcc)                               ! solve and simulate problem
        call writesimstata(simpath//'sys99.dta',len(simpath//'sys99.dta'))     ! export simulated data to stata file
        forall (i=1:nind, rep=1:nrsim) &                                       ! calculate public budget
            tax(i,rep)  = sum(disc(mina:maxa) * sim(i,rep)%aa(mina:maxa)%Oh%tax) 
        PB0 = sum(tax(:,:))
        write (*,*) 'system 1999'
        
        ! Solve and simulate under 2002 tax system
        sys = allsys(2002)                                                     ! load tax-benefits system
        call SolveAndSim_forPolicyAnalysis(chooseS, MixedAcc, dRN, PB0, finalt)
        call writesimstata(simpath//'sys02_RN99.dta', len(simpath//'sys02_RN99.dta'))
        write (*,*) 'system 2002'
                        
    end subroutine simReforms1999LT



    ! -------------------------------------------------------------------------------------------------------
    ! Impact of large reforms: remove tax credits and make eligibility conditional on personnal income

    subroutine simLargeReforms
        implicit none
        integer            :: i, rep, MixedAcc, chooseS, dRN
        real(wp)           :: tax(nind,nrsim), PB0, finalt


        ! initialise parameters, grids and simulation matrix
        sysN     = 1
        MixedAcc = 1
        dRN      = 1
        chooseS  = 1
        call getstartedsolution(1.0_wp)
        call getstartedsimulation(defaultseed)

        ! for long-term effects: change parts of the 2002 system 
        sys  = allsys(2002)                                                    ! define system
        call SolveAndSim_1Sys(chooseS, MixedAcc)                               ! solve and simulate problem
        call writesimstata(simpath//'sys02.dta',len(simpath//'sys02.dta'))     ! export simulated data to stata file
        forall (i=1:nind, rep=1:nrsim) &                                       ! calculate public budget
            tax(i,rep)  = sum(disc(mina:maxa) * sim(i,rep)%aa(mina:maxa)%Oh%tax) 
        PB0 = sum(tax(:,:))
        write (*,*) 'system 2002', PB0
        write (*,*) 

        ! remove WFTC
        sys    = allsys(2002)                                                                   ! define system
        sys%FC = allsys(2003)%FC
        call SolveAndSim_forPolicyAnalysis(chooseS, MixedAcc, dRN, PB0, finalt)                 ! solve and simulate under revenue neutrality
        call writesimstata(simpath//'sys02excTC_RN02.dta',len(simpath//'sys02excTC_RN02.dta'))  ! export simulated data to stata file
        write (*,*) 'system 2002 exc TC', finalt
        write (*,*) 

        ! set WFTC entitlement at the individual level in 2002 system
        sys  = allsys(2002)                                                                     ! define system
        individualWFTC = 1
        call SolveAndSim_forPolicyAnalysis(chooseS, MixedAcc, dRN, PB0, finalt)                 ! solve and simulate under revenue neutrality
        call writesimstata(simpath//'sys02indTC_RN02.dta',len(simpath//'sys02indTC_RN02.dta'))  ! export simulated data to stata file
        individualWFTC = 0                                                                      ! recover eligibility at family level
        write (*,*) 'system 2002 with individual eligibility rule', finalt
        write (*,*) 

    end subroutine simLargeReforms



    ! -------------------------------------------------------------------------------------------------------
    ! Impact of increasing public deficit by 0.5% of product and redistribute it through alternative routes:
    !    basic income tax rate, TC award, IS  

    subroutine GiveawayReforms
        implicit none
        integer  :: i, adjustIS, adjustFC, rep, n, MixedAcc, chooseS
        real(wp) :: tax(nind,nrsim), LCearn(nind,nrsim), PB1, PB0, PB,  gdp, finalt, increment


        ! Initialise parameters, grids and simulation matrix
        MixedAcc = 1
        chooseS  = 1
        call getstartedsolution(1.0_wp)
        call getstartedsimulation(defaultseed)

        ! Solve and simulate individual's problem under baseline tax-benefit system (2002)
        call SolveAndSim_1Sys(chooseS, MixedAcc)                                                                  ! solve and simulate under 2002 system
        call writesimstata(simpath//'sys02.dta',len(simpath//'sys02.dta'))                                        ! export simulated data to stata file
        forall (i=1:nind, rep=1:nrsim) 
            tax(i,rep)    = sum(disc(mina:maxa) * sim(i,rep)%aa(mina:maxa)%Oh%tax)                                ! net tax liability
            LCearn(i,rep) = sum(disc(mina:maxa) * (sim(i,rep)%aa(mina:maxa)%Oh%y+sim(i,rep)%aa(mina:maxa)%Oh%ym)) ! gross LC earn
        end forall
        PB  = sum(tax(:,:))
        gdp = sum(LCearn(:,:))
        write (*,*) ' system 2002: public savings and gdp', PB, gdp
        PB0 = PB-0.005d0*gdp                                                                                      ! increase PB by 0.5% of GDP

        ! Distribute excess PB by reducing basic tax rate
        call SolveAndSim_forPolicyAnalysis(chooseS, MixedAcc, 1, PB0, finalt)
        call writesimstata(simpath//'sys02_tax.dta',len(simpath//'sys02_tax.dta'))
        write (*,*) ' giveaway: new basic tax rate is      ', finalt
        write (*,*)
        
        ! Distribute excess PB using IS award
        call fortax_loadsys_anydate(20020407)
        adjustIS = 1
        adjustFC = 0
        PB1      = PB
        call getRevenueNeutralSys(PB0, PB1, chooseS, MixedAcc, adjustIS, adjustFC, increment)
        call writesimstata(simpath//'sys02_IS.dta',len(simpath//'sys02_IS.dta'))
        write (*,*) ' giveaway: increment in IS is         ', increment
        write (*,*)
        
        ! Distribute excess PB using WFTC award
        call fortax_loadsys_anydate(20020407)
        adjustIS = 0
        adjustFC = 1
        PB1      = PB
        call getRevenueNeutralSys(PB0, PB1, chooseS, MixedAcc, adjustIS, adjustFC, increment)
        call writesimstata(simpath//'sys02_TC.dta',len(simpath//'sys02_TC.dta'))
        write (*,*) ' giveaway: increment in WFTC award is ', increment
        write (*,*) 

    end subroutine GiveawayReforms



    ! -------------------------------------------------------------------------------------------------------
    ! Routines to solve and simulate behaviour under single regime under the option of keeping 
    !    public revenue unchanged relative to baseline


    ! If dRN=1: reform is to be revenue neutral, in which case need to provide meaningful "PB0" 
    !   and will output "finalt" (basic tax rate that equates public budgets under the two regimes)
    subroutine SolveAndSim_forPolicyAnalysis (chooseS, MixedAcc, dRN, PB0, finalt) 
        implicit none
        real(wp),intent(in)  :: PB0                   ! public budget in baseline policy environment
        integer, intent(in)  :: chooseS, &            ! whether choosing education or using last set of simulations
                                MixedAcc, &           ! whether some indiv live in rented acc
                                dRN                   ! whether budget is to be kept unaltered
        real(wp),intent(out) :: finalt                ! final basic tax rate (calculated if dRN=1)
        real(wp) PB1, tax(nind,nrsim)
        integer i, a, rep

        
        ! Solve and simulate individual's problem under new tax system
        call SolveAndSim_1Sys(chooseS, MixedAcc)
        
        ! initialise final basic tax rate
        finalt = sys%inctax%rates(2)
        
        ! compute new basic tax rate if reform is revenue neutral
        if (dRN==1) then
        
            ! total tax liability per individual
            forall (i=1:nind, rep=1:nrsim) tax(i,rep) = sum(disc(mina:maxa) * sim(i,rep)%aa(mina:maxa)%Oh%tax) 

            ! Obtain new public budget        
            PB1 = sum(tax(:,:))
            write (*,*) 
            write(*,'(" initial budgets    ", 2F20.5, F10.7)') PB0, PB1, abs((PB0-PB1)/PB0)
            
            ! Get tax adjustment to keep budget at pre-reform level
            call getRevenueNeutralTax(PB0,PB1,chooseS,MixedAcc,finalt)
            write(*,'(" final budgets      ", 2F20.5, F10.7)') PB0, PB1, abs((PB0-PB1)/PB0)

        end if
        
    end subroutine SolveAndSim_forPolicyAnalysis


    ! Calculates change in tax rate required to compensate public revenue from change in expenditure
    subroutine getRevenueNeutralTax(PB0,PB1,chooseS,MixedAcc,finalt)
        implicit none
        real(wp),intent(in)    :: PB0      !Public budget (revenue-expenditure) under system 0 (baseline)
        real(wp),intent(inout) :: PB1      !Public budget (revenue-expenditure) under system 1 (new)
        integer, intent(in)    :: chooseS  !Whether adjust education choice to new tax regime
        integer, intent(in)    :: MixedAcc !Whether allow for population living in rented and non-rented accommodation
        real(wp),intent(out)   :: finalt
        real(wp) t1, t2, newt, tax(nind,nrsim), PB2
        integer  j, i, a, rep


        t1 = sys%inctax%rates(2)
        
        if (abs((PB0-PB1)/PB0) > tol*10.0_wp) then

            ! move tax rate in the direction to budget gap
            if (PB0-PB1 > 0.0_wp) t2 = sys%inctax%rates(2) + 0.01_wp
            if (PB0-PB1 < 0.0_wp) t2 = sys%inctax%rates(2) - 0.01_wp

            ! solve and calculate new budget position
            sys%inctax%rates(2) = t2
            call SolveAndSim_1Sys(chooseS, MixedAcc)
            forall (i=1:nind, rep=1:nrsim) tax(i,rep) = sum(disc(mina:maxa) * sim(i,rep)%aa(mina:maxa)%Oh%tax) 
            PB2 = sum(tax(:,:))
            j   = 1
            write(*,'("        ", I3, F9.5, 2F20.5, F10.7)') j, t2, PB0, PB2, abs((PB0-PB2)/PB0)
            
            ! iterate until budget gap is zero            
            do 
            
                ! new t obtained by linear interpolation
                newt = t2 - (PB0-PB2) * (t2-t1) / ((PB0-PB2)-(PB0-PB1))

                ! if convergence not achieved
                if (abs(newt-t2) > tol) then

                    j=j+1

                    sys%inctax%rates(2) = newt
                    call SolveAndSim_1Sys(chooseS, MixedAcc)
                    PB1 = PB2
                    forall (i=1:nind, rep=1:nrsim) tax(i,rep) = sum(disc(mina:maxa) * sim(i,rep)%aa(mina:maxa)%Oh%tax) 
                    PB2 = sum(tax(:,:))

                    t1  = t2
                    t2  = sys%inctax%rates(2)
                    write(*,'("        ", I3, F9.5, 2F20.5, F10.7)') j, t2, PB0, PB2, abs((PB0-PB2)/PB0)
                                    
                ! if convergence achieved
                else 
                    PB1 = PB2
                    exit
                end if
                
            end do
            
        end if
        
        finalt = sys%inctax%rates(2)
        
    end subroutine getRevenueNeutralTax


    ! Computes change in some system parameter selected to distribute and increase in public spending
    subroutine getRevenueNeutralSys(PB0, PB1, chooseS, MixedAcc, adjustIS, adjustFC, increment)
        implicit none 
        real(wp),intent(in)    :: PB0                ! Public budget (revenue-expenditure) under sys 0
        real(wp),intent(inout) :: PB1                ! Public budget (revenue-expenditure) under sys 1
        integer, intent(in)    :: chooseS            ! Whether adjust education choice to new tax regime
        integer, intent(in)    :: MixedAcc           ! Whether allow for population living in rented and non-rented accommodation
        integer, intent(in)    :: adjustIS, adjustFC ! parts of the transfer system where adjustment is done
        real(wp),intent(out)   :: increment
        type(sys_t) :: sys1, sys2, sysbase
        real(wp)    :: tax(nind, nrsim), PB2
        integer     :: j, i, a, rep


        sys1    = sys
        sys2    = sys
        sysbase = sys
        
        if (abs((PB0-PB1)/PB0) > tol*10.0_wp) then

            increment = 1.d0
            if (PB0-PB1 > 0.0_wp) increment=-1.d0

            if (adjustIS==1) then
                sys%incsup%maincou = sys2%incsup%maincou + increment
                sys%incsup%yngcou  = sys2%incsup%yngcou  + increment
                sys%incsup%mainlp  = sys2%incsup%mainlp  + increment
                sys%incsup%ynglp   = sys2%incsup%ynglp   + increment
                sys%incsup%mainsin = sys2%incsup%mainsin + increment
                sys%incsup%yngsin  = sys2%incsup%yngsin  + increment
            else if (adjustFC==1) then
                sys%FC%adult  = sys2%FC%adult + increment
            end if
            
            sys2 = sys
            call SolveAndSim_1Sys(chooseS, MixedAcc)
            forall (i=1:nind, rep=1:nrsim) tax(i,rep) = sum(disc(mina:maxa) * sim(i,rep)%aa(mina:maxa)%Oh%tax) 
            PB2 = sum(tax(:,:))
            
            j=1
            write(*,'("        ", I3, F9.5, 2F20.5, F10.7)') j, increment, PB0, PB2, abs((PB0-PB2)/PB0)
            
            do 
                if (adjustIS==1) then
                    increment = (PB0-PB2) * (sys2%incsup%maincou-sys1%incsup%maincou) / (PB2-PB1)
                else if (adjustFC==1) then
                    increment = (PB0-PB2) * (sys2%FC%adult-sys1%FC%adult) / (PB2-PB1)
                end if

                if (abs((PB0-PB2)/PB0) > tol*10.0_wp) then

                    j=j+1

                    if (adjustIS==1) then
                        sys%incsup%maincou = sys2%incsup%maincou + increment
                        sys%incsup%yngcou  = sys2%incsup%yngcou  + increment
                        sys%incsup%mainlp  = sys2%incsup%mainlp  + increment
                        sys%incsup%ynglp   = sys2%incsup%ynglp   + increment
                        sys%incsup%mainsin = sys2%incsup%mainsin + increment
                        sys%incsup%yngsin  = sys2%incsup%yngsin  + increment
                    else if (adjustFC==1) then
                        sys%FC%adult  = sys2%FC%adult +increment
                    end if

                    call SolveAndSim_1Sys(chooseS, MixedAcc)
                    PB1 = PB2
                    forall (i=1:nind, rep=1:nrsim) tax(i,rep) = sum(disc(mina:maxa) * sim(i,rep)%aa(mina:maxa)%Oh%tax) 
                    PB2 = sum(tax(:,:))
                    sys1 = sys2
                    sys2 = sys
                    write(*,'("        ", I3, F9.5, 2F20.5, F10.7)') j, increment, PB0, PB2, abs((PB0-PB2)/PB0)
                                    
                else 
                    PB1 = PB2
                    exit
                end if

            end do
                        
        end if
        
        ! calculate overall adjustments
        sys2 = sys                          ! final system
        sys = sysbase                       ! recover baseline system
        if (adjustIS==1) then
            increment = sys2%incsup%maincou-sys%incsup%maincou
        else if (adjustFC==1) then
            increment = sys2%FC%adult - sys%FC%adult
        end if
        
    end subroutine getRevenueNeutralSys
    

end module PolicyExperimentsFcs


