! -----------------------------------------------------------------------------------------------------------
! -----------------------------------------------------------------------------------------------------------
! Data modules: all the key constants, parameters, grids, prices and data structures to be used by 
!               several routines
! -----------------------------------------------------------------------------------------------------------
! -----------------------------------------------------------------------------------------------------------


! -----------------------------------------------------------------------------------------------------------
! Mathematical constants

module precision
    implicit none
    integer, parameter   :: wp       = kind(1.0d0)
    real(wp), parameter  :: tol      = 1.0e-5_wp
    real(wp), parameter  :: infty    = 1.0e300_wp
    real(wp), parameter  :: neginfty = -1.0e300_wp
end module precision



! -----------------------------------------------------------------------------------------------------------
! Fixed parameters

module constants
    use precision, only : wp
    implicit none

    ! female constants
    integer, parameter :: ntth = 2              ! types of theta (permanent unobserved heterogeneity)
    integer, parameter :: ntb1 = 2              ! family background, 1st factor
    integer, parameter :: ntb2 = 2              ! family background, 2nd factor
    integer, parameter :: nts  = 3              ! levels of schooling

    ! age constants
    integer, parameter, dimension(nts) :: as = (/19, 19, 22/)   ! age entering labour market (by education)
    integer, parameter                 :: mina = as(1)          ! minimum age (first period of working life for lowest educated)
    integer, parameter                 :: maxa = 59             ! maximum age (last period of working life)
    integer, parameter                 :: enda = 69             ! Last period of life
    integer, parameter, dimension(nts) :: minam = as            ! minimum age of marriage
    integer, parameter, dimension(nts) :: minabirth = as        ! minimum fertility age
    integer, parameter                 :: maxabirth = 42        ! maximum fertility age
    
    ! age of kids and CC
    integer, parameter  :: dokids  = 1           ! include kids
    integer,  parameter :: aSchSt  = 5           ! age school starts
    integer,  parameter :: aCcHi   = 10          ! max age with positive CC cost
    real(wp), parameter :: schHrs  = 18.0_wp     ! school hours
    integer,  parameter :: doCC=1                ! some women have +ve CC costs

    ! male constants
    integer, parameter  :: domales = 1           ! include males
    integer, parameter  :: ntsm = 3              ! number of levels of schooling for males

    ! fertility constants
    integer, parameter                  :: maxmaxakd = 18       ! maximum age of child (age kid = -1 if no kids)
    integer, dimension(mina:maxa,nts)   :: birthyrs             ! indicator for fertility period
    integer, dimension(mina-1:enda,nts) :: maxnkdvec, &         ! maximum number of kids by (a,s)
                                           minakd, maxakd       ! min and max possible age of kid of mother (a,s)

    ! family constants
    real(wp), parameter, dimension(0:domales,0:dokids) :: &     ! equivalence scale
         eqsc = reshape(source=(/ 1.0_wp, 1.6_wp, 1.4_wp, 2.0_wp /) , shape = (/2,2/))
    
    real(wp), parameter, dimension(nts,0:domales,0:dokids) :: initfam = &                      !Initial distr of family type: at 19 (s=1,2) and 22 (s=3)
                                                          (/ 0.608_wp, 0.815_wp, 0.715_wp,  &  !male=0, kd=0, s=1,2,3
                                                             0.139_wp, 0.128_wp, 0.273_wp,  &  !male=1, kd=0, s=1,2,3
                                                             0.089_wp, 0.026_wp, 0.006_wp,  &  !male=0, kd=1, s=1,2,3
                                                             0.164_wp, 0.031_wp, 0.006_wp /)   !male=1, kd=1, s=1,2,3

    ! Minimum (subsistence) consumption
    real(wp), parameter :: minc = 5.0_wp

    ! 99 wage level in 2008 prices: real wage index * price index
    real(wp), parameter :: p99 = 0.928984_wp * (165.4083_wp/214.825_wp)
    
end module constants



! -----------------------------------------------------------------------------------------------------------
! Location of I/O data files

module IOpaths

    character(*), parameter :: BHPSpath      = '../data/'
    character(*), parameter :: simpath       = '../sim/'
    character(*), parameter :: elastpath     = '../sim/'
    character(*), parameter :: fortaxpath    = '../fortaxcalcs/fortax/'
    character(*), parameter :: parampath     = 'parameters/'
    character(*), parameter :: mompath       = 'moments/'

end module IOpaths



! -----------------------------------------------------------------------------------------------------------
! Grids, mostly to discretise state space

module gridvars

    use precision, only : wp
    use constants, only : nts, mina, maxa, enda, ntth, ntsm
    implicit none

    ! Grid in k (assets)
    integer,  parameter                     :: ngpk = 6               ! # grid points in assets
    real(wp), dimension(mina:enda,ngpk,nts) :: gk                     ! grid in assets
    real(wp), parameter                     :: maxk = 20000.0_wp      ! largest value of k in grid
    real(wp), dimension(mina:enda+1,nts)    :: mink                   ! minimum assets
    real(wp), dimension(nts)                :: minkfloor

    ! Grid in e (experience)
    integer,  parameter                        :: maxngpe=5           ! max # grid points for experience
    real(wp), dimension(mina:maxa,nts)         :: maxe                ! max years of experience by (a,s)
    integer,  dimension(mina:maxa,nts)         :: ngpe                ! # grid points by (a,s)
    real(wp), dimension(mina:maxa,maxngpe,nts) :: ge                  ! grid in experience (maxngpe,a,s)

    ! Grid in p (female productivity shock)
    integer,  parameter                               :: ngpp = 12
    real(wp), dimension(ngpp,mina:maxa,nts,ntth)      :: gp           ! grid in female productivity shock by (a,s,th)
    real(wp), dimension(ngpp,ngpp,mina:maxa,nts,ntth) :: trp          ! transition probabilities: female productivity shock (gpp,gpp_1,a,s,th)

    ! Grid in pm (male productivity shock)
    integer,  parameter                               :: ngppm = 4    ! # grid points for partner's productivity
    real(wp), dimension(ngppm,ntsm,mina:maxa)         :: gpm          ! grid in male productivity shock by (sm,a)

    ! Grid in l (female working hours)
    integer,  parameter                   :: ngpl = 3                            ! # grid points in hours
    real(wp), parameter, dimension(ngpl)  :: gl   = (/0.0_wp, 18.0_wp, 38.0_wp/) ! grid in hours
    
    ! Grid in lm (male working hours)
    integer,  parameter                   :: ngplm = 2                           ! # grid points in male hours
    real(wp), parameter, dimension(ngplm) :: glm   = (/0.0_wp, 40.0_wp/)         ! grid in male hours

end module gridvars



! -----------------------------------------------------------------------------------------------------------
! Model parameters (not all estimated within the structural model)

module parameters

    use precision,      only : wp
    use constants,      only : nts, ntsm, ntth, domales, dokids, maxmaxakd, mina, maxa, enda, ntb1, ntb2, p99
    use gridvars,       only : ngpl, ngplm, ngppm
    implicit none

    ! constant and derived parameters: not to be estimated
    ! ----------------------------------------------------

    real(wp) :: R                           ! Interest rate
    real(wp) :: beta, disc(mina:maxa)       ! discount rate

    ! curvature parameter in utility
    real(wp) :: mu0             ! curvature 
    real(wp) :: omu0            ! (derived) reciprocal of mu0
    real(wp) :: omu0l1          ! (derived) Power for linearisation
    real(wp) :: mu0omu0l1       ! (derived) power in Euler equation
    real(wp) :: phi             ! (derived) EE ratio of consumption across retirement periods

    ! Male: wage equation 
    real(wp), dimension(ntsm)           :: mktwm          ! market wage values (2008 wage levels)
    real(wp), dimension(ntsm)           :: alpham         ! Coeff on log (female) age
    real(wp)                            :: rhom           ! Autocorrelation coeff
    real(wp), dimension(ntsm)           :: sepmas, seqm   ! SE productivity shock: initial and on-going
    real(wp), dimension(ntsm,mina:maxa) :: sepm           ! (derived) SE male unobserved productivity by (sm,a)

    ! Male: employment probability 
    real(wp), dimension(ntsm)           :: prlmnew        ! prob male in new couple is working, by (sm)
    real(wp), dimension(ntsm)           :: dlta0          ! prob male in ongoing couple is working if not working last period, by(sm)
    real(wp), dimension(ntsm)           :: dlta1          ! prob male in ongoing couple is working if working last period, by(sm)
    real(wp), dimension(ngplm,ntsm)     :: trlm           ! (derived) transition rate to employment for males in ongoing couples, by (lm_1, sm)

    ! family transitions: formation/dissolution couples
    real(wp), dimension(ntsm,mina:enda,nts) :: pr0m                             ! (imported) Prob new spouse of education sm, by (a,s)
    real(wp)                                :: trSS(mina:enda,nts)              ! transition matrix: single to single, by (a,s)
    real(wp)                                :: trCS(ntsm,0:dokids,mina:enda)    ! (imported) transition matrix: couple to single, by (sm_1,kd_1,a)
    type partner
        real(wp) :: m(ntsm,ngplm,ngppm)                                         ! transition matrix: to man (sm, lm, pm)
    end type partner
    type(partner) :: trSC(mina:maxa, nts)                                       ! transition matrix: single to couple, by (a,s)
    type(partner) :: trCC(ngppm, ngplm, 0:dokids, mina:maxa)                    ! transition matrix: couple to couple, by (pm_1,lm_1,kd_1,a)
    
    ! Kid arrival
    real(wp), dimension(mina:enda,nts,0:domales,-1:maxmaxakd)              :: prkd   ! (imported) probability new birth, by (a,s,m_1,akd_1)
    real(wp), dimension(-1:maxmaxakd,-1:maxmaxakd,mina:enda,nts,0:domales) :: trkd   ! (derived) transition: age of child (akd, akd_1, a, s, m_1)

    ! Cost of childcare
    real(wp) :: ccPerHr                                                              ! hourly rate of CC (2008 wage levels)
    real(wp), dimension(ngpl,0:domales,ngplm,-1:maxmaxakd) :: ccCost                 ! (derived) CC cost, by (l,m,lm,akd)

    ! university fees & rents
    real(wp) :: Ufee, fee        ! weekly fee for university education (1000*3yrs, 2008 wage levels); if variable over period use fee
    integer  :: dUfee            ! whether including University fees
    real(wp), parameter :: rent0=0.0_wp, rentp=90.0_wp     ! zero and +ve rent (2008 wage levels)


    ! parameters estimated within model
    ! ---------------------------------

    ! female wage equation
    real(wp), dimension(nts,ntb1,ntb2)      :: mktw     ! (estimate) Female market wages, by(s,f1,f2)
    real(wp), dimension(nts,ntb1,ntb2)      :: alpha    ! (estimate) Coeff on experience, by(s,f1,f2)
    real(wp), dimension(nts)                :: rho      ! (estimate) autoregressive coeff, by (s)
    real(wp), dimension(nts)                :: sepas    ! (estimate) SE productivity at entrance in working life, by (s)
    real(wp), dimension(nts,ntth)           :: Epas     ! (estimate) expected value of initial productivity shock, by (s,th)
    real(wp), dimension(nts,ntth,mina:maxa) :: Ep       ! (estimate) expected value of productivity shock, by (s,th,a)
    real(wp), dimension(nts)                :: seq      ! (estimate) se of innovation to productivity shock (nts)
    real(wp), dimension(nts)                :: Eq       ! expected value of innovation in productivity - assumed zero but change in risk anaylsis
    real(wp), dimension(nts,mina:maxa)      :: sep      ! (derived)  SE female productivity, by (s,a)
    real(wp), dimension(nts)                :: seME     ! (estimate) SE measurement error in log wages, by (s)

    ! female experience accumulation
    real(wp),dimension(ngpl,nts) :: delta0               ! (estimate) experience accumulation, by (l,s)
    real(wp),dimension(nts) :: delta1                    ! (estimate) depreciation rate, by (s)
                
    ! utility parameters: employment 
    real(wp) U(nts), Uk(nts), Um(nts), Umk(nts)          ! (estimate) interactions between education and family composition
    real(wp) Uk02, Uk35, Uk610, Uk1118                   ! (estimate) age of youngest kid
    real(wp) Ub1, Ub2, Umwk                              ! (estimate) family background types and whether present partner works
    
    ! utility parameters: PT employment 
    real(wp) UPT(nts), UkPT(nts), UmPT(nts), UmkPT(nts)  ! (estimate) interactions between education and family composition
    real(wp) Uk02PT, Uk35PT, Uk610PT, Uk1118PT           ! (estimate) age of youngest kid
    real(wp) Ub1PT, Ub2PT, UmwkPT                        ! (estimate) family background types and whether present partner works
    
    ! utility parameters: unobserved heterogeneity in preferences
    real(wp),dimension(ngpl,ntth) :: theta               ! (estimate) preference parameters by (l)
    real(wp) ptheta1                                     ! (estimate) uncoditional probability of type 1
    
    ! probability of positive CC costs
    real(wp) :: pposCC                                   ! (estimate) prob of positive CC costs if working

    ! derived utility scalers, by (l,th,lm,m,akd,s,f1,f2)
    type Ucoeff
        real(wp), dimension(ngplm,0:domales,-1:maxmaxakd,nts,ntb1,ntb2) :: b
    end type Ucoeff
    type(Ucoeff), dimension(ngpl,ntth) :: mu1, expmu1, EEpar
    
    ! preferences for education
    real(wp) :: vS(nts,6)                          ! (estimate) parameters in value function
    real(wp) :: seS(nts)                           ! SE unobserved preferences for education

    ! Data structure: estimating parameters
    integer, parameter :: nParamMax = 89           ! Maximum number of parameters 
    type param_t
        integer        :: n                        ! Number of parameters loaded from file
        character(12)  :: name(nParamMax)          ! Parameter names
        real(wp)       :: val(nParamMax)           ! Parameter values
        real(wp)       :: lbval(nParamMax)         ! tight lower bound for parameter values
        real(wp)       :: ubval(nParamMax)         ! tight upper bound for parameter values
        real(wp)       :: lb(nParamMax)            ! rescaled lower bounds
        real(wp)       :: ub(nParamMax)            ! rescaled upper bounds
        character(50)  :: text(nParamMax)          ! Labels for parameters
        real(wp)       :: bscale(nParamMax)        ! scale of the parameter (relative to unit interval)
        real(wp)       :: bmean(nParamMax)         ! mid point of feasibility interval
        real(wp)       :: b(nParamMax)             ! re-scaled estimating parameters
    end type param_t
    type(param_t)      :: param

end module parameters


! -----------------------------------------------------------------------------------------------------------
! Solution matrices: expected value function and marginal utility of consumption
!   (approx inverse transformation)

module lEmatrices
    use precision,  only : wp
    use constants
    use parameters, only : aCcHi
    use gridvars,   only : ngpl, maxngpe, ngpk, ngpp, ngplm, ngppm
    implicit none

    integer, parameter :: toti = (maxa-as(1)+1) + (maxabirth-as(1)+1) * ( (maxmaxakd+1) + (aCcHi+1) ) - 3 +          &  !indexes for (a,akd_1,dCC,s=1)
                                 ((maxabirth-as(1)-aCcHi) + (maxabirth-as(1)-maxmaxakd)) * (maxmaxakd-aCcHi+1) / 2 + &
                                 (maxa-as(2)+1) + (maxabirth-as(2)+1) * ( (maxmaxakd+1) + (aCcHi+1) ) - 3 +          &  !indexes for (a,akd_1,dCC,s=2)
                                 ((maxabirth-as(2)-aCcHi) + (maxabirth-as(2)-maxmaxakd)) * (maxmaxakd-aCcHi+1) / 2 + &
                                 (maxa-as(3)+1) + (maxabirth-as(3)+1) * ( (maxmaxakd+1) + (aCcHi+1) ) - 3 +          &  !indexes for (a,akd_1,dCC,s=3)
                                 ((maxabirth-as(3)-aCcHi) + (maxabirth-as(3)-maxmaxakd)) * (maxmaxakd-aCcHi+1) / 2 + 2
    type index
        integer :: ix(-1:maxmaxakd, mina:maxa, 0:doCC, 1:nts)    ! index in lE matrices: point (akd_1,a,iCC,s) 
        integer :: a(toti)                                       ! corresponding age in point i
        integer :: akd_1(toti)                                   ! corresponding age of kid at age i-1 in point i
        integer :: iCC(toti)                                     ! corresponding indicator for +ve CC cost in point i
        integer :: s(toti)                                       ! corresponding education in point i
    end type index
    type(index) :: iEmtx

    ! inner matrices: singles at a-1, by (k, e, p_1)
    type single_1
        real(wp) :: m0(ngpk,maxngpe,ngpp)
    end type single_1
    
    ! inner matrices: in couple at a-1, (k, e, p_1, sm_1, lm_1, pm_1)
    type couple_1
        real(wp) :: m1(ngpk,maxngpe,ngpp,ntsm,ngplm,ngppm)
    end type couple_1
    
    ! outer structures: working life periods
    type(single_1), allocatable, target :: lEV0(:,:,:,:), lEdu0(:,:,:,:)          ! if single at t-1 (ix(akd,a,dC,s), th, f1i, f2i)
    type(couple_1), allocatable, target :: lEV1(:,:,:,:), lEdu1(:,:,:,:)          ! if couple at t-1 (ix(akd,a,dC,s), th, f1i, f2i)

    ! similar matrices for post-working life period
    real(wp),dimension(nts,-1:maxmaxakd,0:domales,ntsm,ngpk) :: lEVr, lEDUr

    
contains
        

    ! -------------------------------------------------------------------------
    ! Populate matrix of indexes and corresponding arguments of lE/ldU matrices

    subroutine pop_Emtx
        implicit none
        integer a, akd_1, s, iCC, ix

        write(*,*) toti
        
        iEmtx%ix    = 0
        iEmtx%a     = 0
        iEmtx%akd_1 = 0
        iEmtx%iCC   = 0
        iEmtx%s     = 0

        ix = 0
        do s = 1,nts

            ! no CC costs
            iCC = 0
            if (s==nts) then
                ix = ix+1
                akd_1 = -1
                iEmtx%ix(akd_1,mina,iCC,s) = ix
                iEmtx%a(ix)     = mina
                iEmtx%akd_1(ix) = akd_1
                iEmtx%iCC(ix)   = iCC
                iEmtx%s(ix)     = s
            end if
            
            do a=as(s),maxa

                ix    = ix+1                      ! no kids last period (a-1)
                akd_1 = -1
                iEmtx%ix(akd_1,a,iCC,s) = ix
                iEmtx%a(ix)     = a
                iEmtx%akd_1(ix) = akd_1
                iEmtx%iCC(ix)   = iCC
                iEmtx%s(ix)     = s
                
                do akd_1 = minakd(a-1,s), maxakd(a-1,s)
                    ix    = ix+1                  ! kid aged akd_1 last period (a-1)
                    iEmtx%ix(akd_1,a,iCC,s) = ix
                    iEmtx%a(ix)     = a
                    iEmtx%akd_1(ix) = akd_1
                    iEmtx%iCC(ix)   = iCC
                    iEmtx%s(ix)     = s
                end do
                
            end do

            ! CC costs
            iCC = 1
            iEmtx%ix(:,:,1,s) = iEmtx%ix(:,:,0,s)
            if (s==nts) then
                ix = ix+1
                akd_1 = -1
                iEmtx%ix(akd_1,mina,iCC,s) = ix
                iEmtx%a(ix)     = mina
                iEmtx%akd_1(ix) = akd_1
                iEmtx%iCC(ix)   = iCC
                iEmtx%s(ix)     = s
            end if
            
            do a=as(s),maxabirth
            do akd_1 = -1, maxakd(a-1,s)

                ix    = ix+1                      ! no kids last period (a-1) - still possible to have a child today
                iEmtx%ix(akd_1,a,iCC,s) = ix
                iEmtx%a(ix)     = a
                iEmtx%akd_1(ix) = akd_1
                iEmtx%iCC(ix)   = iCC
                iEmtx%s(ix)     = s

            end do
            end do

            do a=maxabirth+1,maxa
            do akd_1 = minakd(a-1,s), aCcHi-1

                ix    = ix+1                      ! no kids last period (a-1) - still possible to have a child today
                iEmtx%ix(akd_1,a,iCC,s) = ix
                iEmtx%a(ix)     = a
                iEmtx%akd_1(ix) = akd_1
                iEmtx%iCC(ix)   = iCC
                iEmtx%s(ix)     = s

            end do
            end do

        end do
    
        write(*,*) 'iEmtx', toti, ix
        
    end subroutine pop_Emtx

end module lEmatrices



! -----------------------------------------------------------------------------------------------------------
! Information for simulations

module SimConditions
    use precision,      only : wp
    use fortax_type,    only : sys_t

    integer, parameter :: defaultseed=4571269
    integer            :: calcFrisch, aFrisch            ! for Frisch elasticities of LS
    integer            :: calcMarshall, aMarshall        ! for Marshall elasticities
    real(wp)           :: pchangeNetInc                  ! for elasticities: proportional change and absolute drop in net earnings (the latter by l)
    real(wp)           :: rent                           ! rent if living in rented accommodation
    integer            :: individualWFTC                 ! to simulate WFTC entitlement at individual (rather than family) level
    type(sys_t)        :: allsys(1991:2008)              ! the two systems being combined

end module SimConditions



! -----------------------------------------------------------------------------------------------------------
! Fortax types (tax system, family structure, net income structure)

module fortax_data
    use precision,      only : wp
    use fortax_type,    only : sys_t, fam_t, net_t
    use fortax_prices,  only : sysindex_t
    use IOpaths
    implicit none

    type(sys_t)         :: sys
    type(fam_t)         :: fam
    type(net_t)         :: net
    type(sysindex_t)    :: sysindex
    integer             :: sysdate
    integer             :: sysN                        ! Number of systems
    integer,parameter   :: maxsysN = 18                ! maximum number of systems (1 per year)
    integer,allocatable :: sysD(:), sysY0(:), sysY1(:) ! system dates and corresponding periods
    
    integer, parameter      :: datadate = 20080101

    character(*), parameter :: rpifile       = fortaxpath//'prices/rpi.csv'
    character(*), parameter :: WageIndexfile = fortaxpath//'prices/realWageIndex.csv'
    character(*), parameter :: sysindexfile  = fortaxpath//'systems/sysindex.csv'

contains


    ! Wrapper program to load Fortax tax system
    ! -----------------------------------------
    subroutine fortax_loadsys(system)
        implicit none
        integer,intent(in)          :: system

        call fortax_loadsys_anydate(sysD(system))

    end subroutine fortax_loadsys


    ! Load Fortax tax system
    ! ----------------------
    subroutine fortax_loadsys_anydate(date)
        use precision,      only : wp
        use fortax_prices,  only : loadsysindex, getsysindex, loadindex, upratefactor, upratesys
        use fortax_read,    only : ReadFortaxParams
        implicit none
        integer,intent(in)          :: date
        integer                     :: sysnum
        character(255)              :: sysfilepath
        real(wp)                    :: uprfac, adjustment

        ! Determine system date
        sysdate = date

        ! Load tax system index (records which tax system was in place when)
        call loadsysindex(sysindex,sysindexfile)

        ! Get the file path for the tax system in place on the specified date
        call getsysindex(sysindex,sysdate,'fortax',sysfilepath,sysnum)
        sysfilepath = fortaxpath//sysfilepath

        ! Read in the tax system (no need to initialise because ReadTaxParams does it)
        call readFortaxParams(sys,sysfilepath)

        ! Deflate tax system using (nominal) wage deflator
        call loadindex(WageIndexfile)                             ! Load real wage index
        uprfac = upratefactor(sysdate,datadate)                   ! Calculate real earnings uprating factor
        call loadindex(rpifile)                                   ! Load RPI index
        uprfac = uprfac*upratefactor(sysdate,datadate)            ! Calculate nominal earnings growth
        call upratesys(sys,uprfac,datadate)                       ! Uprate system

    end subroutine fortax_loadsys_anydate


    ! Select only part of a tax system: 2002 WFTC
    ! -------------------------------------------
    subroutine impose2002WFTC
      use precision,      only : wp
      use fortax_type,    only : sys_t
      use fortax_prices,  only : loadsysindex, getsysindex, loadindex, upratefactor, upratesys
      use fortax_read,    only : ReadFortaxParams
      implicit none
      integer, parameter          :: date2002 = 20020407
      integer                     :: sysnum
      character(255)              :: sysfilepath
      real(wp)                    :: uprfac
      type(sys_t)                 :: sys2002

      ! Get the file path for the tax system in place on the specified date
      call getsysindex(sysindex,date2002,'fortax',sysfilepath,sysnum)
      sysfilepath = fortaxpath//sysfilepath

      ! Read in the tax system (no need to initialise because ReadTaxParams does it)
      call readFortaxParams(sys2002,sysfilepath)

      ! Calculate uprating factor
      uprfac = upratefactor(date2002,datadate)

      ! Uprate system
      call upratesys(sys2002,uprfac,datadate)

      ! Replace FC with WFTC in tax system (only really works if sys contains tax system up to Apr-99)
      sys%FC = sys2002%FC

    end subroutine impose2002WFTC

end module fortax_data



! -----------------------------------------------------------------------------------------------------------
! Structure to hold simulated life-cycle profiles

module state
    use precision,      only : wp
    use constants,      only : mina, maxa, nts
    use gridvars,       only : ngpl
    use fortax_data,    only : maxsysN
    implicit none

    integer, parameter :: nind  = 3901
    integer, parameter :: nrsim = 5

    type Oh_t
        integer  :: id                  ! id
        integer  :: system              ! tax system 
        integer  :: insample            ! indicator for observation in data sample
        integer  :: year                ! year
        integer  :: a                   ! age
        real(wp) :: k                   ! assets
        real(wp) :: e                   ! accumulated experience
        real(wp) :: p                   ! productivity
        integer  :: nDl                 ! # periods worked so far
        real(wp) :: w                   ! gross hourly wage rate (woman)
        real(wp) :: y                   ! gross weekly earnings (woman)
        integer  :: m                   ! partner: indicator for presence
        integer  :: sm                  ! partner: education (1, 2 or 3)
        integer  :: lm                  ! partner: labour supply (1=not working, 2=working FT)
        real(wp) :: pm                  ! partner: productivity
        real(wp) :: wm                  ! partner: gross hourly wage rate
        real(wp) :: ym                  ! partner: gross weekly earnings
        real(wp) :: inc                 ! family: net income
        real(wp) :: incbyS(nts)         ! family: net income conditional on education choice
        real(wp) :: tax                 ! family: tax liability
        integer  :: kd                  ! kids: presence
        integer  :: akd                 ! kids: age of youngest
        real(wp) :: cc                  ! family: childcare cost
        real(wp) :: V                   ! expected present value of present and future utility, conditional on today's optimal choice (c,l)
        real(wp) :: returnExp(2)        ! to store simulations of value of exp at different ages (allows to compare 2 systems)
    end type Oh_t

    type Ch_t
        integer  :: l                   ! labour supply (1=not working, 2=working PT, 3=working FT)
        real(wp) :: c                   ! consumption
    end type Ch_t

    type OhCh_t
        type(Oh_t)  :: Oh
        type(Ch_t)  :: Ch
    end type OhCh_t

    type sim_t
        integer  :: id             ! BHPS: Individual (woman) identifier
        integer  :: yob            ! BHPS: year of birth
        integer  :: scotland
        integer  :: wales
        integer  :: g80            ! BHPS: 80s generation (against 70s)
        integer  :: sampleS        ! BHPS: dummy for observations used to estimate parameters in education choice 
        integer  :: student        ! BHPS: whether observed as student entering WL
        real(wp) :: parentYres     ! BHPS: res - parent earnings when woman aged ~16
        real(wp) :: f1             ! BHPS: background information - principal component 1 (continuous)
        real(wp) :: f2             ! BHPS: background information - principal component 2 (continuous)
        integer  :: f1i            ! BHPS: background information - principal component 1 (dummy for top 50%)
        integer  :: f2i            ! BHPS: background information - principal component 2 (dummy for top 50%)
        integer  :: th             ! preference type - taste for working (1 or 2)
        integer  :: s              ! education
        integer  :: drent          ! whether pays rent (in mixed simulations)
        integer  :: pveCC          ! whether faces CC costs if working mother
        real(wp) :: EVpreS         ! expected value of lifetime utility at mina, before education decision
        real(wp) :: EVpostS(nts)   ! expected value of lifetime utility at mina, by educ choice (excluding observ and unobserv cost/pref for educ, at 19)
        real(wp) :: EV16(nts)      ! expected value of lifetime utility at mina, by educ choice (including observ cost/pref for educ but excl unobserv pref/cost)
        real(wp) :: V16(nts)       ! expected value of lifetime utility at mina, by educ choice (including observ and unobserv cost/pref for educ)
        integer  :: sysA0(maxsysN) ! age at which first experiences each tax system
        integer  :: sysA1(maxsysN) ! age at which last experiences each tax system
        real(wp) :: lEdu_as        ! expected marginal utility of consumption at start of WL
        real(wp) :: prefS2         ! preference shock for s=2
        real(wp) :: prefS3         ! preference shock for s=3

        type(OhCh_t) :: aa(mina:maxa)  ! An array of Oh and Ch structures
    end type sim_t

    type(sim_t),target :: sim(nind,nrsim)
    
end module state

