! -----------------------------------------------------------------------------------------------------------
! -----------------------------------------------------------------------------------------------------------
! This file contains the routines to simulate women labour supply when there is an expected (Frisch) or 
! unexpected (Marshall) change in income
! Simulated life-cycle profiles exported to stata
! -----------------------------------------------------------------------------------------------------------
! -----------------------------------------------------------------------------------------------------------


module elasticitiesfcs

use precision,     only : wp, tol
use constants
use IOpaths
use parameters
use parametersfcs
use gridvars
use gridfcs,       only : popgp 
use lEmatrices,    only : lEdu0, iEmtx
use state
use fortax_data
use stataMod
use SimConditions
use slnMod,        only : solve
use simMod,        only : sim_forFrisch, simReadInit, simClean, sim_forElast, bounds
use wrapperMod,    only : getstartedsolution, getstartedsimulation, SolveAndSim_1Sys
use generalfns,    only : LinInterp1D, LinInterp3d

contains


    ! ----------------------------------------------------------------------------------------------------
    ! Frisch elasticities
    ! Compensated expected transitory change in net (female) earnings

    subroutine SolveSimFrisch
        implicit none
        integer, parameter  :: tota = maxa-mina+1
        integer  :: a, a1, i, s, th, dCC, f1i, f2i, ubk, ix, akd_1, rep
        real(wp) :: lEdu_as(nts,ntth,1-doCC:1,ntb1,ntb2,ngpk), initialk
        character(3),dimension(22:50) :: k,l


        ! name variables to export simulated data to stata
        do a=22,50
            l(a)(1:1) = 'l'
            k(a)(1:1) = 'k'
            write(l(a)(2:3), '(I2)') a
            write(k(a)(2:3), '(I2)') a
        end do

        ! Initialise parameters, grids and simulation matrix
        call getstartedsolution(1.0_wp)
        call getstartedsimulation(defaultseed)
        forall (a=mina:maxa) disc(a) = R**(mina-a)

        ! Solve and simulate individual's problem with normal earnings
        call SolveAndSim_1Sys(1, 0)
        call saveStata(elastpath//'frisch.dta', len(elastpath//'frisch.dta'), nind*nrsim*tota)
        call writeStata((/ ((spread(i,1,tota),             i=1, nind), rep=1,nrsim) /), 'iSim')
        call writeStata((/ ( spread(rep,1,tota*nind),                  rep=1,nrsim) /), 'rep')
        call writeStata((/ (((a, a=mina,maxa),             i=1, nind), rep=1,nrsim) /), 'a')
        call writeStata((/ ((spread(sim(i,rep)%th,1,tota), i=1, nind), rep=1,nrsim) /), 'th')
        call writeStata((/ ((spread(sim(i,rep)%s,1,tota),  i=1, nind), rep=1,nrsim) /), 's')
        call writeStata((/ (((sim(i,rep)%aa(:)%Oh%m),      i=1, nind), rep=1,nrsim) /), 'm')
        call writeStata((/ (((sim(i,rep)%aa(:)%Oh%sm),     i=1, nind), rep=1,nrsim) /), 'sm')
        call writeStata((/ (((sim(i,rep)%aa(:)%Oh%kd),     i=1, nind), rep=1,nrsim) /), 'kd')
        call writeStata((/ (((sim(i,rep)%aa(:)%Oh%akd),    i=1, nind), rep=1,nrsim) /), 'akd')
        call writeStata((/ (((sim(i,rep)%aa(:)%Oh%p),      i=1, nind), rep=1,nrsim) /), 'p')
        call writeStata((/ (((sim(i,rep)%aa(:)%Oh%k),      i=1, nind), rep=1,nrsim) /), 'k')
        call writeStata((/ (((sim(i,rep)%aa(:)%Oh%inc),    i=1, nind), rep=1,nrsim) /), 'inc')
        call writeStata((/ (((sim(i,rep)%aa(:)%Ch%l),      i=1, nind), rep=1,nrsim) /), 'l')
        write(*,*)
        write(*,*) 'Frisch elasticities'
        
        ! store expected marginal utility of consumption at entrance in WL
        akd_1 = -1
        do dCC = 1-doCC,1
        do s   = 1,nts
            ix = iEmtx%ix(akd_1, as(s), dCC, s)
            forall (th=1:ntth, f1i=1:ntb1, f2i=1:ntb2) lEdu_as(s,th,dCC,f1i,f2i,:) = lEdu0(ix,th,f1i,f2i)%m0(:,1,1)
        end do        
        end do        

        do rep=1,nrsim
        do i=1,nind

            s    = sim(i,rep)%s
            th   = sim(i,rep)%th
            dCC  = sim(i,rep)%pveCC     
            f1i = sim(i,rep)%f1i
            f2i = sim(i,rep)%f2i
            
            if (s .ne. nts) then
                ! marginal utility at as(s) evaluated at k=0 for s=1,2
                sim(i,rep)%lEdu_as = lEdu_as(s,th,dCC,f1i,f2i,1)
            else
                ! marginal utility at as(s) evaluated at initialk for s=3: interpolate
                a1       = as(s)
                initialk = sim(i,rep)%aa(a1)%Oh%k                
                ubk      = 2
                do
                    if (gk(a1,ubk,s) > initialk .or. ubk==ngpk) exit
                    ubk = ubk+1
                end do
                sim(i,rep)%lEdu_as = LinInterp1D(initialk, gk(a1,ubk-1,s), gk(a1,ubk,s), &
                                                 lEdu_as(s,th,dCC,f1i,f2i,ubk-1), lEdu_as(s,th,dCC,f1i,f2i,ubk))
            end if
            
        end do
        end do
        

        ! change net earnings at each age and calculate compensation in initial k that 
        !    keeps marginal utility of consumtpion constant
        calcFrisch = 1
        write(*,*)
        do aFrisch=22,50
            write(*,*) '   simulations for change in earnings at age', aFrisch
            call solve
            call sim_forFrisch
            call writeStata((/ (((sim(i,rep)%aa(:)%Ch%l), i=1, nind), rep=1,nrsim) /), l(aFrisch))
            call writeStata((/ (((sim(i,rep)%aa(:)%Oh%k), i=1, nind), rep=1,nrsim) /), k(aFrisch))
        end do
        calcFrisch = 0
        call closeSaveStata

    end subroutine SolveSimFrisch



    ! ----------------------------------------------------------------------------------------------------
    ! Marshall elasticities
    ! Uncompensated unexpected permanent change in net (female) earnings
    
    subroutine SolveSimMarshall
        implicit none
        integer, parameter :: tota = maxa-mina+1
        integer  a, i, s, rep
        character(3),dimension(22:50) :: k, c, l 


        ! Prepare name of variables to export simulated data to stata
        do a=22,50
            k(a)(1:1) = 'k';    write(k(a)(2:3), '(I2)') a
            c(a)(1:1) = 'c';    write(c(a)(2:3), '(I2)') a
            l(a)(1:1) = 'l';    write(l(a)(2:3), '(I2)') a
        end do


        ! Initialise parameters, grids and simulation matrix
        call getstartedsolution(1.0_wp)
        call getstartedsimulation(defaultseed)
        forall (a=mina:maxa) disc(a) = R**(mina-a)


        ! Solve and simulate individual's problem with normal earnings
        call SolveAndSim_1Sys(1, 0)
        call saveStata(elastpath//'marshall.dta', len(elastpath//'marshall.dta'), nind*nrsim*tota)
        call writeStata((/ ((spread(i,1,tota),             i=1,nind), rep=1,nrsim) /), 'iSim')
        call writeStata((/ ( spread(rep,1,tota*nind),                 rep=1,nrsim) /), 'rep' )
        call writeStata((/ (((a, a=mina,maxa),             i=1,nind), rep=1,nrsim) /), 'a')
        call writeStata((/ ((spread(sim(i,rep)%th,1,tota), i=1,nind), rep=1,nrsim) /), 'th')
        call writeStata((/ ((spread(sim(i,rep)%s, 1,tota), i=1,nind), rep=1,nrsim) /), 's')
        call writeStata((/ (((sim(i,rep)%aa(:)%Oh%m),      i=1,nind), rep=1,nrsim) /), 'm')
        call writeStata((/ (((sim(i,rep)%aa(:)%Oh%sm),     i=1,nind), rep=1,nrsim) /), 'sm')
        call writeStata((/ (((sim(i,rep)%aa(:)%Oh%kd),     i=1,nind), rep=1,nrsim) /), 'kd')
        call writeStata((/ (((sim(i,rep)%aa(:)%Oh%akd),    i=1,nind), rep=1,nrsim) /), 'akd')
        call writeStata((/ (((sim(i,rep)%aa(:)%Oh%p),      i=1,nind), rep=1,nrsim) /), 'p')
        call writeStata((/ (((sim(i,rep)%aa(:)%Oh%k),      i=1,nind), rep=1,nrsim) /), 'k')
        call writeStata((/ (((sim(i,rep)%aa(:)%Oh%inc),    i=1,nind), rep=1,nrsim) /), 'inc')
        call writeStata((/ (((sim(i,rep)%aa(:)%Ch%l),      i=1,nind), rep=1,nrsim) /), 'l')
        call writeStata((/ (((sim(i,rep)%aa(:)%Ch%c),      i=1,nind), rep=1,nrsim) /), 'c')
        call writeStata((/ (((sim(i,rep)%aa(:)%Oh%y),      i=1,nind), rep=1,nrsim) /), 'y')
        call writeStata((/ (((sim(i,rep)%aa(:)%Oh%w),      i=1,nind), rep=1,nrsim) /), 'w')
        write(*,*)
        write(*,*) 'Marshall elasticities'


        ! Simulate behaviour with permanent (uncompensated and unexpected) change in net earning
        calcMarshall = 1
        write(*,*)
        do aMarshall=50,22,-1
            write(*,*) '   simulations for change in earnings at age', aMarshall
            call solve
            call sim_forElast(aMarshall)
            call writeStata((/ (((sim(i,rep)%aa(:)%Ch%l), i=1, nind), rep=1,nrsim) /), l(aMarshall))
            call writeStata((/ (((sim(i,rep)%aa(:)%Oh%k),  i=1, nind), rep=1,nrsim) /), k(aMarshall))
            call writeStata((/ (((sim(i,rep)%aa(:)%Ch%c),  i=1, nind), rep=1,nrsim) /), c(aMarshall))
        end do
        calcMarshall = 0
        call closeSaveStata

    end subroutine SolveSimMarshall

end module elasticitiesfcs


