! -----------------------------------------------------------------------------------------------------------
! -----------------------------------------------------------------------------------------------------------
! This file contains the routines use to estimate the structural parameters of the model
! Use quadratic optimisation routine form NAG libraries
! -----------------------------------------------------------------------------------------------------------
! -----------------------------------------------------------------------------------------------------------


module EstimationMod

use precision
use parameters
use lEmatrices
use fortax_data
use SimConditions
use parametersfcs
use IOpaths
use gridfcs,       only : popgp 
use slnMod,        only : solve
use wrapperMod,    only : QformScale, SolveSimAndCalcMom
use momMod
use omp_lib 

integer, private :: ncallQA

contains



    !--------------------------------------------------------------------------------------------------
    ! Main estimation routine
    ! Estimate by quadratic approximation
    ! Maximisation needs to be with respect to 2 or more variables

    subroutine QAestimation(simfile, simchar, momfile, momchar, Qform)
        use generalfns, only : realToStr
        implicit none
        integer, intent(in)            :: simchar, momchar
        character(simchar), intent(in) :: simfile
        character(simchar), intent(in) :: momfile
        real(wp), intent(inout)        :: Qform
        integer                        :: iuser(1), ifail, ncall, maxIter
        real(wp)                       :: scaledQform, fmin, ruser(1)
        external                       :: e04jcf


        ! Re-scale objective function and parameters to facilitate estimation
        QformScale = 0.0_wp
        call scaleToRange(Qform, QformScale, scaledQform)
        call QAparameters(param%b, param%bmean, param%bscale, param%lb, param%ub)
        
        ! Estimate objective function
        ncallQA = 1
        ifail   = -1
        maxIter = merge(150, 1000, param%n<=12)
        call e04jcf(QAobjective, param%n, 2*param%n+1, param%b(1:param%n), param%lb(1:param%n), param%ub(1:param%n), &
                    0.1_wp, tol, QAmonitor, maxIter, fmin, ncall, iuser, ruser, ifail)

        ! Recover parameters in original scale
        param%val(1:param%n) = param%b(1:param%n) * param%bscale(1:param%n) + param%bmean(1:param%n)
        call paramSaveAll(param,parampath//'endParam.txt','Estimated parameters; (Qform=' // realToStr(fmin) // ')')

        ! Check how far (final) simulated moments are from data moments
        if (ifail==0) print *, "Routine converged"
        write(*,*)
        call SolveSimAndCalcMom(1, simfile, simchar, 1, momfile, momchar, Qform)
                
    end subroutine QAestimation



    !--------------------------------------------------------------------------------------------------
    ! Auxilliary routines
    
    ! Calculate objective function
    subroutine QAobjective(nToEstimate, b, Qform, iuser, ruser, inform)
        use generalfns,     only : realToStr
        implicit none
        integer, intent(in)     :: nToEstimate
        real(wp), intent(in)    :: b(nToEstimate)
        real(wp), intent(out)   :: Qform
        integer, intent(inout)  :: iuser(*)
        real(wp), intent(inout) :: ruser(*)
        integer,intent(out)     :: inform
        integer  i
        real(wp) iQform

        iQform = Qform
        write(*,*)

        ! recover parameters in original scale
        param%val(1:nToEstimate) = b * param%bscale(1:nToEstimate) + param%bmean(1:nToEstimate)
        call SolveSimAndCalcMom(0, ' ' , 1, 0, ' ', 1, Qform)
        write(*,'(I4, A30, 2F12.6)') ncallQA, "Qform at entrance and exit", iQform, Qform
        
        ! print ongoing set of parameters to file
        call paramSaveAll(param, parampath//'param-ongoing.txt', 'Estimated parameters; (Qform=' // realToStr(Qform) // ')')
        ncallQA = ncallQA+1
        
        ! needs to be +ve for estimation to continue
        inform = 1

    end subroutine QAobjective


    ! Monitor estimation
    subroutine QAmonitor(ntoestimate, ncall, b, Qform, rho, iuser, ruser, inform)
        implicit none
        integer,  intent(in)    :: ntoestimate, ncall
        real(wp), intent(in)    :: b(ntoestimate), Qform, rho
        integer,  intent(inout) :: iuser(1)
        real(wp), intent(inout) :: ruser(1)
        integer,  intent(out)   :: inform
        integer i

        write (*,'(I5,A30,F10.7)') ncall, ' - current Qform value = ', Qform
        write(*, '("    with parameters ")', advance='NO')
        do i=1,ntoestimate
            write (*,'(F10.6)', advance='NO') b(i)
        end do
        write(*,*)

        ! needs to be +ve for the estimation to continue
        inform = 1 

    end subroutine QAmonitor


    ! Re-scale estimation parameters to unit interval
    subroutine QAparameters(b, bmean, bscale , bl, bu)
        implicit none
        real(wp),intent(out),dimension((param%n)) :: b, bmean, bscale, bl, bu
        integer                                   :: i

        do i=1, param%n
            bmean(i)  = ( param%ubval(i)+param%lbval(i) ) * 0.5_wp
            bl(i)     = -1.0_wp
            bu(i)     =  1.0_wp
            bscale(i) = param%ubval(i)-bmean(i)
            b(i)      = (param%val(i)-bmean(i))/bscale(i)
        end do

    end subroutine QAparameters

end module EstimationMod


