! -----------------------------------------------------------------------------------------------------------
! -----------------------------------------------------------------------------------------------------------
! Routines to construct working grids
! -----------------------------------------------------------------------------------------------------------
! -----------------------------------------------------------------------------------------------------------


module gridfcs

use precision,  only : wp
use gridvars
use constants


contains


    ! -------------------------------------------------------------------------------------------------------
    ! Populate grid in assets k
    ! Algorithm: 
    !   sets mink to ensure subsistence consumption (minc) for rest of life is always affordable
    !   interval between grid points grows exponetially 
    !   set min income to IS for singles with no children in Apr 1999

    subroutine popgk
        use parameters,    only : R, Ufee, dUfee
        use incomefns,     only : netincome
        use fortax_data,   only : fortax_loadsys_anydate
        implicit none
        integer            :: a, s, ki
        real(wp)           :: minkval, miny, unit, UBwl
        real(wp),parameter :: ratio=4.0_wp


        ! calculate minimum income that the agent is guaranteed to earn in each period of wl (JSA/IS)
        call fortax_loadsys_anydate(19990407)
        UBwl = netincome(1,1,0.0_wp,0.0_wp,0,1,1,maxa,0.0_wp,-1,0,1,1,1)   

        ! grid in k: (gpk x a x s)
        mink(enda+1,:) = 0.0_wp
        do s=1, nts

            minkval = 0.0_wp                                                ! mink at age enda+1

            do a=enda, mina, -1

                ! calculate bounds in grids
                miny      = merge ( UBwl, 0.0_wp, a>=as(s) .and. a<=maxa )  ! min guaranteed weekly income
                minkval   = (minc - miny + minkval)/R                       ! cumulate minimum k
                mink(a,s) = max(minkval,minkfloor(s))
                if (a==mina) mink(a,s) = 0.0_wp                             ! does not start adult life indebt & university educ always affordable

                ! select grid points
                gk(a,1,s) = mink(a,s)
                unit      = (maxk-mink(a,s))*(1.0_wp-ratio)/(1.0_wp-ratio**(ngpk-1))
                do ki=2,ngpk
                    gk(a,ki,s) = gk(a,ki-1,s)+unit*ratio**(ki-2)
                end do

            end do
                        
        end do
       
    end subroutine popgk



    ! -------------------------------------------------------------------------------------------------------
    ! Populate grid in experience e

    subroutine popge
        implicit none
        integer s, a, ei
        real(wp),dimension(maxngpe),parameter :: auxge = (/0.0_wp, 2.0_wp, 6.0_wp, 16.0_wp, 40.0_wp/)


        ! maximum experience at (a,s)
        do s=1,nts
            forall (a=as(s):maxa) maxe(a,s) = a-as(s)
        end do
        maxe(mina:as(3),3) = 0    ! e during university years
        
        ! grid in e and nr grid points by (a,s)
        ge   = 0.0_wp
        ngpe = 1
        do s=1,nts
        do a = as(s)+1,maxa
            do ei=2,maxngpe
                if (maxe(a,s)<=auxge(ei)) then
                    ge(a,ei,s) = maxe(a,s)
                    ngpe(a,s)  = ei
                    exit
                else
                    ge(a,ei,s) = auxge(ei)
                end if
            end do
            if (ngpe(a,s) == 0) ngpe(a,s) = maxngpe
        end do
        end do

    end subroutine popge



    ! -------------------------------------------------------------------------------------------------------
    ! Populate grid in productivity shock p and compute transition matrix
    
    subroutine popgp
        use parameters, only : rho, seq, Eq, sep, Ep, Epas
        implicit none
        integer  :: a, th, s, pi_1, pi
        real(wp) :: gSN(ngpp)


        ! Initiate variables and grids
        call popnormgrid(ngpp,0.0_wp,1.0_wp,gSN)          ! grid for standard normal
        gp  = 0.0_wp
        trp = 0.0_wp

        ! construct grids and transition matrices
        do th=1,ntth
        do s=1,nts

            ! grid points by (th,s,a)
            do a=as(s),maxa
                gp(:,a,s,th) = gSN * sep(s,a) + Ep(s,th,a)
            end do !a

            ! transition probabilities at a>as: by (p_1,a-1,s,th)
            do a=as(s)+1,maxa
            do pi_1=1, ngpp
                call trprob(gp(pi_1,a-1,s,th), rho(s), Eq(s), seq(s), ngpp, gp(:,a,s,th), trp(:,pi_1,a,s,th))
            end do
            end do
            
            ! at a=as: unconditional normal
            a = as(s)
            call trprob(Ep(s,th,a), 1.0_wp, 0.0_wp, sep(s,a), ngpp, gp(:,a,s,th), trp(:,1,a,s,th))
            forall (pi=1:ngpp) trp(pi,2:ngpp,a,s,th) = trp(pi,1,a,s,th)

        end do
        end do
        
    end subroutine popgp
            


    ! -------------------------------------------------------------------------------------------------------
    ! Populate grid in spouse's productivity shock (pm)

    subroutine popgpm
        use parameters, only : sepm
        implicit none
        integer  :: a, sm, i
        real(wp) :: gSN(ngppm)

        call popnormgrid(ngppm,0.0_wp,1.0_wp,gSN)
        forall (sm=1:ntsm, a=mina:maxa, i=1:ngppm) gpm(i,sm,a) = gSN(i) * sepm(sm,a)

    end subroutine popgpm



    ! -------------------------------------------------------------------------------------------------------
    ! Populate vectors describing domain of kids variables by (a,s)

    subroutine popkdvec
        implicit none
        integer s, akd

        birthyrs  = 0       ! fertility period by (a,s)
        maxnkdvec = 0       ! max # kids in family
        minakd    = 0       ! min age of youngest child of mother (a,s)
        maxakd    = -1      ! max age of youngest child of mother (a,s): =-1 indicates no child can be around

        do s=1, nts
            birthyrs(minabirth(s):maxabirth, s)                   = 1
            maxnkdvec(minabirth(s):maxabirth+maxmaxakd, s)        = 1
            maxakd(minabirth(s)+maxmaxakd:maxabirth+maxmaxakd, s) = maxmaxakd
            do akd=0,maxmaxakd
                minakd(maxabirth+akd,s)    = akd
                maxakd(minabirth(s)+akd,s) = akd
            end do
        end do
        
    end subroutine popkdvec



    ! -------------------------------------------------------------------------------------------------------
    ! Mixed auxilliary routines

    ! centiles from normal distribution
    subroutine popnormgrid(ngp,mu,sigma,grid)
        use nag_g_ib,  only : g01faf
        implicit none
        integer, intent(in)   :: ngp
        real(wp), intent(in)  :: mu, sigma
        real(wp), intent(out) :: grid(ngp)
        integer               :: gpi, ifail
        real(wp)              :: p

        do gpi=1,ngp
            p         = (real(gpi,8)-0.5d0)/ngp
            ifail     = 0
            grid(gpi) = g01faf('L',p,ifail)*sigma + mu
        end do

    end subroutine popnormgrid


    ! transition probabilities for AR(1) process with normal innovations
    subroutine trprob(rv,rho,Einnov,sinnov,ngp,grd1,trp)
        use nag_g_ib,  only : g01eaf
        implicit none
        real(wp), intent(in)  :: rv          ! value of random variable
        real(wp), intent(in)  :: rho         ! AR parameter
        real(wp), intent(in)  :: Einnov      ! Expected value of innovation
        real(wp), intent(in)  :: sinnov      ! Standard error of innovation
        integer,  intent(in)  :: ngp         ! Number of grid points
        real(wp), intent(in)  :: grd1(ngp)   ! grid next period
        real(wp), intent(out) :: trp(ngp)    ! Output vector of transition probabilities
        real(wp) mean1                       ! mean of rv next period
        integer  i, ifail

        
        ! calculate mean of random variable next period
        mean1 = rho*rv + Einnov
        
        ! probability of transition to intervals where each grid point is the middle probability point
        ifail = 0
        trp(1) = (g01eaf('L',(grd1(2)-mean1)/sinnov,ifail) + g01eaf('L',(grd1(1)-mean1)/sinnov,ifail))/2.0_wp
        do i=2,ngp-1
            trp(i) = (g01eaf('L',(grd1(i+1)-mean1)/sinnov,ifail) - g01eaf('L',(grd1(i-1)-mean1)/sinnov,ifail))/2.0_wp
        end do
        trp(ngp) = 1.0_wp - (g01eaf('L',(grd1(ngp-1)-mean1)/sinnov,ifail) + g01eaf('L',(grd1(ngp)-mean1)/sinnov,ifail))/2.0_wp

    end subroutine trprob


end module gridfcs




