! -----------------------------------------------------------------------------------------------------------
! -----------------------------------------------------------------------------------------------------------
! Central routine: calls others to solve, simulate and calculate simulated moments
! -----------------------------------------------------------------------------------------------------------
! -----------------------------------------------------------------------------------------------------------

 program main

    use precision,            only : wp, infty, neginfty
    use IOpaths
    use lEmatrices
    use fortax_data
    use parametersfcs
    use state
    use EstimationMod,        only : QAestimation
    use PolicyExperimentsFcs, only : simReforms1999ST, simReforms1999LT, GiveawayReforms, simLargeReforms
    use riskFcs,              only : sensitivityrisk
    use elasticitiesfcs,      only : SolveSimFrisch, SolveSimMarshall
    use varMod,               only : getVariance
    use wrapperMod,           only : QformScale, SolveSimAndCalcMom, simEY, defaultSim
    use omp_lib 
    implicit none
    real(wp)     :: objfc
    integer      :: Munit, proceed, nsim



    ! -----------------------------------------------------------------------------------------------------
    ! Initialise structures to start solving and simulating

    ! Load initial information
    call defaultSim
    call CttParameters
    call DerivedCttParametersAndGrds
    call pop_Emtx
    
    ! Set size of big solution matrices 
    allocate (lEV0(toti,ntth,ntb1,ntb2))
    allocate (lEV1(toti,ntth,ntb1,ntb2))
    allocate (lEDU0(toti,ntth,ntb1,ntb2))
    allocate (lEDU1(toti,ntth,ntb1,ntb2))
    
    ! set initial scale for Qform - this only changes if estimating
    QformScale = 0.0_wp

    ! set number of processors to use in parallel computing
    call omp_set_num_threads(6) 
    !$omp parallel 
    if (omp_get_thread_num() == 0) write(*,'(a,i1)') 'Number of threads: ', omp_get_num_threads() 
    !$omp end parallel 


    ! -----------------------------------------------------------------------------------------------------
    ! Baseline simulations: use same 4 tax-benefit systems as in estimation

    write(*,*)
    write(*, '("Run baseline simulations (sequence of 4 systems as in estimation)? (Y=1/N=0) ")', ADVANCE='NO')
    read(*,*) proceed

    if (proceed==1) then
    
        sysN = 4
        if (allocated(sysD))  deallocate(sysD);   allocate(sysD(sysN));   sysD  = (/ 19950407, 19990407, 20020407, 20040407 /) !system dates
        if (allocated(sysY0)) deallocate(sysY0);  allocate(sysY0(sysN));  sysY0 = (/ 1991, 1997, 2000, 2003 /)                 !1st year of system
        if (allocated(sysY1)) deallocate(sysY1);  allocate(sysY1(sysN));  sysY1 = (/ 1996, 1999, 2002, 2008 /)                 !last year of system
        call paramLoadStarting
        call SolveSimAndCalcMom(1, simpath//'newsim.dta', len(simpath//'newsim.dta'), &
                                1, mompath//'simmom.tex', len(mompath//'simmom.tex'), objfc)
        
    end if


    ! -----------------------------------------------------------------------------------------------------
    ! Simulate expected lifetime income by tax-system (from 1991 to 2008) as if it remains unchanged over
    !     entire life, and by education

    write(*,*)
    write(*, '("Simulate expected lifetime income? (Y=1/N=0) ")', ADVANCE='NO')
    read(*,*) proceed

    if (proceed==1) then    
        sysN = 1        
        nsim = 20
        call paramLoadStarting
        call simEY(nsim, simpath//'EY.dta')
    end if


    ! -----------------------------------------------------------------------------------------------------
    ! Estimate structural parameters
    ! use 4 tax-benefit systems

    write(*,*)
    write(*, '("Estimate structural parameters? (Y=1/N=0) ")', ADVANCE='NO')
    read(*,*) proceed
    
    if (proceed==1) then

        sysN = 4
        if (allocated(sysD))  deallocate(sysD);   allocate(sysD(sysN));   sysD  = (/ 19950407, 19990407, 20020407, 20040407 /) !system dates
        if (allocated(sysY0)) deallocate(sysY0);  allocate(sysY0(sysN));  sysY0 = (/ 1991, 1997, 2000, 2003 /)                 !1st year of system
        if (allocated(sysY1)) deallocate(sysY1);  allocate(sysY1(sysN));  sysY1 = (/ 1996, 1999, 2002, 2008 /)                 !last year of system

        ! load initial values of estimating parameters from external file
        call paramLoadStarting         
        ! print initial moments to file
        call SolveSimAndCalcMom(0, ' ', 1, 1, mompath//'simmom-initial.tex', len(mompath//'simmom-initial.tex'), objfc)
        ! run estimatin procedure and save final simulation of LC profiles and moments to files
        call QAestimation(simpath//'newsim.dta', len(simpath//'newsim.dta'), mompath//'simmom-final.tex', len(mompath//'simmom-final.tex'), objfc)

    end if


    ! -----------------------------------------------------------------------------------------------------
    ! Calculate variance of estimates

    write(*,*)
    write(*, '("Calculate variance of estimates? (Y=1/N=0) ")', ADVANCE='NO')
    read(*,*) proceed
    
    if (proceed==1) then
        sysN = 4
        if (allocated(sysD))  deallocate(sysD);   allocate(sysD(sysN));   sysD  = (/ 19950407, 19990407, 20020407, 20040407 /) !system dates
        if (allocated(sysY0)) deallocate(sysY0);  allocate(sysY0(sysN));  sysY0 = (/ 1991, 1997, 2000, 2003 /)                 !1st year of system
        if (allocated(sysY1)) deallocate(sysY1);  allocate(sysY1(sysN));  sysY1 = (/ 1996, 1999, 2002, 2008 /)                 !last year of system
        call paramLoadStarting
        call getVariance
    end if

        
    ! -----------------------------------------------------------------------------------------------------
    ! Simulate effects of reforms
    
    ! Reforms over the 1999-2002 period
    write(*,*)
    write(*, '("Simulate 1999-2002 reforms? (Y=1/N=0) ")', ADVANCE='NO')
    read(*,*) proceed
    
    if (proceed==1) then
        call paramLoadStarting
        call simReforms1999ST
        call simReforms1999LT
     end if
    
    
    ! Large reforms: remove tax credits and make eligibility conditional on personal income
    write(*,*)
    write(*, '("Simulate large reforms? (Y=1/N=0) ")', ADVANCE='NO')
    read(*,*) proceed
    
    if (proceed==1) then
        sysN = 1        
        call paramLoadStarting
        call simLargeReforms
    end if
     

    ! giveaway reforms using April 2002 tax-benefit system as baseline
    ! distribute 0.5% of total GDP by: reducing basic tax rate, increasing IS, increasing TC award
    write(*,*)
    write(*, '("Simulate giveaway policies? (Y=1/N=0) ")', ADVANCE='NO')
    read(*,*) proceed

    if (proceed==1) then    
        sysN = 1        
        call paramLoadStarting
        call fortax_loadsys_anydate(20020407)
        call GiveawayReforms
    end if


    ! Remove tuition fees and reduce loans accordingly
    write(*,*)
    write(*, '("Simulate model without tuition fees? (Y=1/N=0) ")', ADVANCE='NO')
    read(*,*) proceed

    if (proceed==1) then

        sysN = 4
        if (allocated(sysD))  deallocate(sysD);   allocate(sysD(sysN));   sysD  = (/ 19950407, 19990407, 20020407, 20040407 /) !system dates
        if (allocated(sysY0)) deallocate(sysY0);  allocate(sysY0(sysN));  sysY0 = (/ 1991, 1997, 2000, 2003 /)                 !1st year of system
        if (allocated(sysY1)) deallocate(sysY1);  allocate(sysY1(sysN));  sysY1 = (/ 1996, 1999, 2002, 2008 /)                 !last year of system
        
        ! re-calculate constant parameters and grids with no university fees
        dUfee = 0
        call CttParameters
        call DerivedCttParametersAndGrds

        ! get other parameters, solve and simulate
        call paramLoadStarting
        call SolveSimAndCalcMom(1, simpath//'sim-noUfees.dta', len(simpath//'sim-noUfees.dta'), 0, ' ', 1, objfc)

        ! reset university fees
        dUfee = 1

    end if
    

    ! -----------------------------------------------------------------------------------------------------
    ! Calculate sensitivity to risk under April 2002 tax-benefit system

    write(*,*)
    write(*, '("Calculate sensitivity to risk? (Y=1/N=0) ")', ADVANCE='NO')
    read(*,*) proceed
    
    if (proceed==1) then
        sysN = 1        
        call paramLoadStarting
        call fortax_loadsys_anydate(20020407)
        call sensitivityrisk
    end if


    ! -----------------------------------------------------------------------------------------------------
    ! compute labour supply elasticities

    write(*,*)
    write(*, '("Simulate LS elasticities? (Y=1/N=0) ")', ADVANCE='NO')
    read(*,*) proceed
    
    if (proceed==1) then
        sysN = 1        
        call paramLoadStarting
        call fortax_loadsys_anydate(19990407)
        call SolveSimFrisch
        call SolveSimMarshall
    end if

end program main



