! -----------------------------------------------------------------------------------------------------------
! -----------------------------------------------------------------------------------------------------------
! Calculates simulated moments and value of the objective function at given value of estimating parameters
! Then exports moments to text file
! -----------------------------------------------------------------------------------------------------------
! -----------------------------------------------------------------------------------------------------------



module momMod

use precision
use constants
use state
use nag_g_ib,   only : G02DAF
use stataMod
use IOpaths
implicit none

integer, parameter  :: defaultseedMom = 2762065
integer, parameter  :: nmom = 50            ! Total number of moments per section of Mstructure (upper bound)
type typeM
    integer         :: n                    ! Number of moments (excludes intercepts for wales and scotland)
    character(20)   :: name(nmom)           ! Moment names
    real(wp)        :: dta(nmom,nts)        ! Data moments
    real(wp)        :: dtaQ(nmom,nts)       ! Data quantile of the distribution: only to be used for quantile moments
    real(wp)        :: se(nmom,nts)         ! Data standard errors
    real(wp)        :: sim(nmom,nts)        ! Simulated moments
    real(wp)        :: sediff(nmom,nts)     ! no. se differences
    real(wp)        :: ssdiff               ! objective function: sum of squared se differences for set of moments
    integer         :: toUse                ! Whether moment is used in estimation (last 2 moments, if corresponding to wales and scotland, never used)
end type typeM

type structureM
    type(typeM)   :: Sdist            ! education distribution 
    type(typeM)   :: Sregr            ! coeffs from regression of education choice
    type(typeM)   :: E                ! mean employment rates during WL (<=50, england only)
    type(typeM)   :: PT               ! mean PT employment rates during WL (<=50, england only)
    type(typeM)   :: UE               ! mean transition rates (NW->E) during WL (<=50)
    type(typeM)   :: EU               ! mean transition rates (E->NW) during WL (<=50)
    type(typeM)   :: lW0              ! distribution of log wages at entrance in WL
    type(typeM)   :: dlWe             ! coeffs from dlog wage regression on years of work
    type(typeM)   :: lWregr           ! coeffs from log wage regression during WL
    type(typeM)   :: lWa              ! coeffs from log wage regression on age
    type(typeM)   :: dlWy             ! coeffs from dlog wage regression on past working status
    type(typeM)   :: lWdist           ! distribution of log wages during WL
    type(typeM)   :: CC               ! whether has positive CC costs while working if mother of young children
    type(typeM)   :: aux              ! auxilliary structure to run regressions using routine OLS when needed 
end type structureM

! matrix with data weights to compute simulated moments consistent with data distribution of family types
real(wp), parameter, dimension(nts,-1:maxmaxakd,0:domales) :: famwgh = (/ 0.13151383_wp, 0.19597771_wp, 0.22211947_wp, &  ! m=0, no kid, s=1,2,3
                                                                          0.01084425_wp, 0.00470138_wp, 0.00184945_wp, &  ! m=0, akd=0,  s=1,2,3
                                                                          0.01135371_wp, 0.00513669_wp, 0.00221935_wp, &  ! m=0, akd=1,  s=1,2,3
                                                                          0.01098981_wp, 0.00705206_wp, 0.00240429_wp, &  ! m=0, akd=2,  s=1,2,3
                                                                          0.01055313_wp, 0.00618144_wp, 0.00258924_wp, &  ! m=0, akd=3,  s=1,2,3
                                                                          0.00946143_wp, 0.00705206_wp, 0.00314407_wp, &  ! m=0, akd=4,  s=1,2,3
                                                                          0.00880640_wp, 0.00600731_wp, 0.00240429_wp, &  ! m=0, akd=5,  s=1,2,3
                                                                          0.00858806_wp, 0.00644263_wp, 0.00240429_wp, &  ! m=0, akd=6,  s=1,2,3
                                                                          0.00778748_wp, 0.00531081_wp, 0.00277418_wp, &  ! m=0, akd=7,  s=1,2,3
                                                                          0.00742358_wp, 0.00592025_wp, 0.00295913_wp, &  ! m=0, akd=8,  s=1,2,3
                                                                          0.00720524_wp, 0.00531081_wp, 0.00203440_wp, &  ! m=0, akd=9,  s=1,2,3
                                                                          0.00756914_wp, 0.00513669_wp, 0.00203440_wp, &  ! m=0, akd=10, s=1,2,3
                                                                          0.00596798_wp, 0.00496256_wp, 0.00258924_wp, &  ! m=0, akd=11, s=1,2,3
                                                                          0.00567686_wp, 0.00383075_wp, 0.00240429_wp, &  ! m=0, akd=12, s=1,2,3
                                                                          0.00531295_wp, 0.00348250_wp, 0.00314407_wp, &  ! m=0, akd=13, s=1,2,3
                                                                          0.00531295_wp, 0.00339544_wp, 0.00240429_wp, &  ! m=0, akd=14, s=1,2,3
                                                                          0.00458515_wp, 0.00287306_wp, 0.00277418_wp, &  ! m=0, akd=15, s=1,2,3
                                                                          0.00196507_wp, 0.00139300_wp, 0.00184945_wp, &  ! m=0, akd=16, s=1,2,3
                                                                          0.00087336_wp, 0.00095769_wp, 0.00129462_wp, &  ! m=0, akd=17, s=1,2,3
                                                                          0.00036390_wp, 0.00052238_wp, 0.00092473_wp, &  ! m=0, akd=18, s=1,2,3
                                                                          0.24788937_wp, 0.28216960_wp, 0.32513409_wp, &  ! m=1, no kid, s=1,2,3
                                                                          0.05596798_wp, 0.05972488_wp, 0.07194378_wp, &  ! m=1, akd=0,  s=1,2,3
                                                                          0.05240175_wp, 0.05397876_wp, 0.06084705_wp, &  ! m=1, akd=1,  s=1,2,3
                                                                          0.04446870_wp, 0.04327007_wp, 0.04475680_wp, &  ! m=1, akd=2,  s=1,2,3
                                                                          0.03617176_wp, 0.03578269_wp, 0.03014611_wp, &  ! m=1, akd=3,  s=1,2,3
                                                                          0.03384279_wp, 0.02942713_wp, 0.02570742_wp, &  ! m=1, akd=4,  s=1,2,3
                                                                          0.02874818_wp, 0.02568344_wp, 0.02293323_wp, &  ! m=1, akd=5,  s=1,2,3
                                                                          0.02569141_wp, 0.02350688_wp, 0.02126873_wp, &  ! m=1, akd=6,  s=1,2,3
                                                                          0.02561863_wp, 0.02211388_wp, 0.01904938_wp, &  ! m=1, akd=7,  s=1,2,3
                                                                          0.02474527_wp, 0.01976319_wp, 0.01609025_wp, &  ! m=1, akd=8,  s=1,2,3
                                                                          0.02227074_wp, 0.01863138_wp, 0.01535047_wp, &  ! m=1, akd=9,  s=1,2,3
                                                                          0.02161572_wp, 0.01819606_wp, 0.01294618_wp, &  ! m=1, akd=10, s=1,2,3
                                                                          0.02074236_wp, 0.01706425_wp, 0.01220640_wp, &  ! m=1, akd=11, s=1,2,3
                                                                          0.02059680_wp, 0.01567125_wp, 0.01109673_wp, &  ! m=1, akd=12, s=1,2,3
                                                                          0.02161572_wp, 0.01488769_wp, 0.01146662_wp, &  ! m=1, akd=13, s=1,2,3
                                                                          0.02001456_wp, 0.01349469_wp, 0.01035694_wp, &  ! m=1, akd=14, s=1,2,3
                                                                          0.01965066_wp, 0.01262406_wp, 0.00961716_wp, &  ! m=1, akd=15, s=1,2,3
                                                                          0.00880640_wp, 0.00618144_wp, 0.00591825_wp, &  ! m=1, akd=16, s=1,2,3
                                                                          0.00436681_wp, 0.00391781_wp, 0.00388385_wp, &  ! m=1, akd=17, s=1,2,3
                                                                          0.00262009_wp, 0.00226363_wp, 0.00295913_wp /)  ! m=1, akd=18, s=1,2,3

! Penalty of 50 standard deviations if fail to calculate moment
real(wp), parameter, private :: penalty = 50.0_wp

! working life moments
integer,  private, parameter            :: nXwl = 25, nXs = 8, nper=(50-19+1), n=nind*nrsim*nper
real(wp), private, target               :: X(n,nXwl)                                             ! explanatory variables
real(wp), private, dimension(n), target :: dE, dPT, dU, lW, dlW, ydlW, pveCC                     ! dependent variables
real(wp), private                       :: insample_ydlW(n), insampleWL(n), insampleWL_1(n), &
                                           insampleWLt1(n), insampleWLcum(n), dE_1(n), Ed(n,nts) ! weights for whether in estimating sample
integer, private                        :: couple(n), agekid(n)    

! education choice moments
real(wp), private, dimension(nind*nrsim, nXs), target :: Xs                     ! explanatory variables
real(wp), private, dimension(nts,nind*nrsim),  target :: smtx                   ! dependent variables
real(wp), private, dimension(nind*nrsim)              :: insampleS              ! weights for whether in estimating sample

! information for regressions
real(wp),dimension(n),private  :: wgt                                           ! weights in regression (determine observations to be included)
real(wp),private,pointer       :: py(:)                                         ! pointer to dependent variable
real(wp),private,pointer       :: pX(:,:)                                       ! pointer to the matrix of regressors 
real(wp),dimension(n),private  :: rvME                                          ! measurement error in log wages

! quantiles of wages distribution
real(wp), private :: QFT(5,nts), QPT(5,nts), QWL(3), Q0(3,nts)                  ! matrices containing data quantiles

! inverse of variance covariance of data moments
real(wp), allocatable :: Omega(:,:)

contains


    !--------------------------------------------------------------------------------------------------
    ! Import data moments and inverse of variance-covariance matrix

    subroutine selectMom(moments,totMom)
        use nag_f_ib,   only : f01adf
        use parameters
        implicit none
        type(structureM), intent(inout) :: moments
        integer, intent(out)            :: totMom
        integer                         :: unit, i, n, ifail, j
        character(10)                   :: var
        real(wp), allocatable           :: V(:,:)
        character(*), parameter         :: invVfile = mompath//'invV.dta'


        ! Read data moments from text files
        call LoadDataMoments(mompath//'moments-Sdist.txt',        len(mompath//'moments-Sdist.txt'),        moments%Sdist);  moments%Sdist%touse  = 1
        call LoadDataMoments(mompath//'moments-Sregr.txt',        len(mompath//'moments-Sregr.txt'),        moments%Sregr);  moments%Sregr%touse  = 1
        call LoadDataMoments(mompath//'moments-Emean.txt',        len(mompath//'moments-Emean.txt'),        moments%E);      moments%E%touse      = 1
        call LoadDataMoments(mompath//'moments-PTmean.txt',       len(mompath//'moments-PTmean.txt'),       moments%PT);     moments%PT%touse     = 1
        call LoadDataMoments(mompath//'moments-transUE.txt',      len(mompath//'moments-transUE.txt'),      moments%UE);     moments%UE%touse     = 1
        call LoadDataMoments(mompath//'moments-transEU.txt',      len(mompath//'moments-transEU.txt'),      moments%EU);     moments%EU%touse     = 1
        call LoadDataMoments(mompath//'moments-W0.txt',           len(mompath//'moments-W0.txt'),           moments%lW0);    moments%lW0%touse    = 1
        call LoadDataMoments(mompath//'moments-WregrdWLe.txt',    len(mompath//'moments-WregrdWLe.txt'),    moments%dlWe);   moments%dlWe%touse   = 1
        call LoadDataMoments(mompath//'moments-WregrWL.txt',      len(mompath//'moments-WregrWL.txt'),      moments%lWregr); moments%lWregr%touse = 1
        call LoadDataMoments(mompath//'moments-WregrWLslope.txt', len(mompath//'moments-WregrWLslope.txt'), moments%lWa);    moments%lWa%touse    = 1
        call LoadDataMoments(mompath//'moments-WregrdWLy.txt',    len(mompath//'moments-WregrdWLy.txt'),    moments%dlWy);   moments%dlWy%touse   = 1
        call LoadDataMoments(mompath//'moments-WdistWL.txt',      len(mompath//'moments-WdistWL.txt'),      moments%lWdist); moments%lWdist%touse = 1
        call LoadDataMoments(mompath//'moments-CC.txt',           len(mompath//'moments-CC.txt'),           moments%CC);     moments%CC%touse     = 1

        
        ! Now read data quantiles
        call LoadDataQ(mompath)
        
        
        ! Import inverse of var-cov matrix
        totMom = ( moments%Sdist%n + moments%Sregr%n ) * 2 + &
                 ( moments%E%n     + moments%PT%n     + moments%UE%n  + moments%EU%n   + moments%lW0%n    + &
                   moments%dlWe%n  + moments%lWregr%n + moments%lWa%n + moments%dlWy%n + moments%lWdist%n + moments%CC%n ) * 3
        
        if (allocated(Omega)) deallocate(Omega);  allocate (Omega(totMom,totMom))
        unit = 688
        call openStata(invVfile, unit)
        do i=1,totMom
            if (i<9.5) then
                n = 5
                write(var(1:n), '(A4,I1)') 'invV', i
            else if (i>9.5 .and. i<99.5) then
                n = 6
                write(var(1:n), '(A4,I2)') 'invV', i
            else if (i>99.5) then
                n = 7
                write(var(1:n), '(A4,I3)') 'invV', i
            end if
            call readStata(Omega(:,i), var(1:n))
        end do
        call closeOpenStata() 
        
    end subroutine selectMom



    !--------------------------------------------------------------------------------------------------
    ! Calculate moments and store

    subroutine getMomForEst(PrintScreen, PrintFile, momfile, momchar, Qform)
        implicit none
        integer,intent(in)            :: PrintScreen, PrintFile, momchar
        character(momchar),intent(in) :: momfile
        real(wp), intent(out)         :: Qform
        type(structureM)              :: moments
        real(wp), allocatable         :: Dvec(:,:), Q(:,:)
        integer                       :: totMom, mi, s, j, unit

        
        ! calculate moments
        call CalcSimMoments(Qform, totMom, moments)

        ! print moments: summary to screen
        if (PrintScreen==1) then
            write(*,'(A40, F18.6, I2)') 'Education distribution           ', moments%Sdist%ssdiff,  moments%Sdist%touse
            write(*,'(A40, F18.6, I2)') 'Education regressions            ', moments%Sregr%ssdiff,  moments%Sregr%touse
            write(*,'(A40, F18.6, I2)') 'Mean employment rates WL         ', moments%E%ssdiff,      moments%E%touse
            write(*,'(A40, F18.6, I2)') 'Mean PT employment rates WL      ', moments%PT%ssdiff,     moments%PT%touse
            write(*,'(A40, F18.6, I2)') 'mean transition rates U to E     ', moments%UE%ssdiff,     moments%UE%touse
            write(*,'(A40, F18.6, I2)') 'mean transition rates E to U     ', moments%EU%ssdiff,     moments%EU%touse
            write(*,'(A40, F18.6, I2)') 'Distrib log wages entrance WL    ', moments%lW0%ssdiff,    moments%lW0%touse
            write(*,'(A40, F18.6, I2)') '1st diffs log wages during WL    ', moments%dlWe%ssdiff,   moments%dlWe%touse
            write(*,'(A40, F18.6, I2)') 'Regr log wages during WL         ', moments%lWregr%ssdiff, moments%lWregr%touse
            write(*,'(A40, F18.6, I2)') 'Regr log wages during WL: slopes ', moments%lWa%ssdiff,    moments%lWa%touse
            write(*,'(A40, F18.6, I2)') 'Yearly change log wages          ', moments%dlWy%ssdiff,   moments%dlWy%touse
            write(*,'(A40, F18.6, I2)') 'Distrib log wages during WL      ', moments%lWdist%ssdiff, moments%lWdist%touse
            write(*,'(A40, F18.6, I2)') 'pve CC costs among wrk mothers   ', moments%CC%ssdiff,     moments%CC%touse
            write(*,'(A40, F18.6, I2)') 'Q form                           ', Qform
        end if

        ! save moments to log file        
        if (PrintFile==1) then

            unit = 8787
            open (unit, file=momfile)
            
            ! detailed log
            call PrintMoments(moments%Sdist,  unit, 'Education distribution          ')
            call PrintMoments(moments%Sregr,  unit, 'Education regressions           ')
            call PrintMoments(moments%E,      unit, 'Mean employment during WL       ')
            call PrintMoments(moments%PT,     unit, 'Mean PT employment during WL    ')
            call PrintMoments(moments%UE,     unit, 'mean transition rates U to E    ')
            call PrintMoments(moments%EU,     unit, 'mean transition rates E to U    ')
            call PrintMoments(moments%lW0,    unit, 'Distrib log wages entrance WL   ')
            call PrintMoments(moments%dlWe,   unit, 'dlW in WL by working yrs        ')
            call PrintMoments(moments%lWregr, unit, 'Regr lW in WL                   ')
            call PrintMoments(moments%lWa,    unit, 'Regr dlW in WL: slope by age    ')
            call PrintMoments(moments%dlWy,   unit, 'Yrly dlW by last working status ')
            call PrintMoments(moments%lWdist, unit, 'Distrib lW in WL                ')
            call PrintMoments(moments%CC,     unit, 'pve CC costs among wrk mothers  ')
            
            close (unit)

        end if

    end subroutine getMomForEst



    !--------------------------------------------------------------------------------------------------
    ! Main routine: calculate all moments and value of objective function

    subroutine CalcSimMoments(Qform, totMom, moments)
        use nag_g_ib,  only : g05kbf, g05laf
        implicit none
        integer,intent(out)           :: totMom
        type(structureM), intent(out) :: moments
        real(wp), intent(out)         :: Qform
        integer                       :: inX(nXwl), nb, s, j, igen, iseed(4), ifail, nm, b
        real(wp)                      :: se, ar1, x1, x2
        real(wp), allocatable         :: Dvec(:,:), Q(:,:)
        integer                       :: mi

        
        ! Import data moments and prepare matrix of regressors
        ! ----------------------------------------------------

        ! Read moments and var-cov matrix from files
        call selectMom(moments,totMom)

        ! Simulate measurement error in female log wages
        ifail = 0
        igen  = 0
        iseed = 0
        iseed(1) = defaultseedMom
        call g05kbf(igen,iseed)
        call g05laf (0.0_wp, 1.0_wp, n, rvME, igen, iseed, ifail)
        
        ! Get X matrices for regressions
        call getRegrMtrx

        
        ! Calculate moments
        ! -----------------
        
        ! Education choices: means (table 20)
        do s=1,3
            nm  = 1
            x1 = sum( merge( 1.0_wp , 0.0_wp , sim(:,:)%sampleS==1 .and. sim(:,:)%wales==0 .and. sim(:,:)%scotland==0 .and. sim(:,:)%s==s ) )
            x2 = sum( merge( 1.0_wp , 0.0_wp , sim(:,:)%sampleS==1 .and. sim(:,:)%wales==0 .and. sim(:,:)%scotland==0                     ) )
            call getRatio(x1, x2, moments%Sdist%dta(nm,s), moments%Sdist%se(nm,s), moments%Sdist%sim(nm,s), moments%Sdist%sediff(nm,s))
            
            do b=1,2
                nm = nm+1
                x1 = sum( merge( 1.0_wp , 0.0_wp , sim(:,:)%sampleS==1 .and. sim(:,:)%wales==0 .and. sim(:,:)%scotland==0 .and. sim(:,:)%f1i==b .and. sim(:,:)%s==s ) )
                x2 = sum( merge( 1.0_wp , 0.0_wp , sim(:,:)%sampleS==1 .and. sim(:,:)%wales==0 .and. sim(:,:)%scotland==0 .and. sim(:,:)%f1i==b                     ) )
                call getRatio(x1, x2, moments%Sdist%dta(nm,s), moments%Sdist%se(nm,s), moments%Sdist%sim(nm,s), moments%Sdist%sediff(nm,s))
            end do

            do b=1,2
                nm = nm+1
                x1 = sum( merge( 1.0_wp , 0.0_wp , sim(:,:)%sampleS==1 .and. sim(:,:)%wales==0 .and. sim(:,:)%scotland==0 .and. sim(:,:)%f2i==b .and. sim(:,:)%s==s ) )
                x2 = sum( merge( 1.0_wp , 0.0_wp , sim(:,:)%sampleS==1 .and. sim(:,:)%wales==0 .and. sim(:,:)%scotland==0 .and. sim(:,:)%f2i==b                     ) )
                call getRatio(x1, x2, moments%Sdist%dta(nm,s), moments%Sdist%se(nm,s), moments%Sdist%sim(nm,s), moments%Sdist%sediff(nm,s))
            end do
        end do
        moments%Sdist%ssdiff = sum( moments%Sdist%sediff(1:moments%Sdist%n,2:3)**2 )

        
        ! Education choice: linear regressions (table 21)
        nb  = moments%Sregr%n + 2
        inX = 0; inX(1:8) = 1
        pX  => Xs
        wgt(1:nind*nrsim) = insampleS
        do s=2,3
            py  => smtx(s,:)
            call OLS(s, nXs, inX(1:nb), nind*nrsim, 1, nb, moments%Sregr%n, moments%Sregr, 1, 0, se, ar1, ifail )
        end do
        moments%Sregr%ssdiff = sum( moments%Sregr%sediff(1:moments%Sregr%n,2:3)**2 )
        

        ! Employment and PT employment: means (table 22)
        do s=1,3
            py => dE
            call meanE( s, moments%E%dta(:,s), moments%E%se(:,s), moments%E%sim(:,s), moments%E%sediff(:,s) )
            py => dPT
            call meanE( s, moments%PT%dta(:,s), moments%PT%se(:,s), moments%PT%sim(:,s), moments%PT%sediff(:,s) )
        end do
        moments%E%ssdiff  = sum( moments%E%sediff(1:moments%E%n,:)**2 )
        moments%PT%ssdiff = sum( moments%PT%sediff(1:moments%PT%n,:)**2 )


        ! Transitions in and out of employment (tables 23 and 24)
        do s=1,3
            py => dE
            call meanUE( s, moments%UE%dta(:,s), moments%UE%se(:,s), moments%UE%sim(:,s), moments%UE%sediff(:,s) )
            py => dU
            call meanEU( s, moments%EU%dta(:,s), moments%EU%se(:,s), moments%EU%sim(:,s), moments%EU%sediff(:,s) )
        end do
        moments%UE%ssdiff = sum( moments%UE%sediff(1:moments%UE%n,:)**2 )
        moments%EU%ssdiff = sum( moments%EU%sediff(1:moments%EU%n,:)**2 )
        

        ! Log wages at the start of WL: descriptives (table 25)
        do s=1,3
        
            ! mean
            nm = 1
            x2 = sum( insampleWLt1 * (1.0_wp-X(:,23)-X(:,24)) * Ed(:,s) * dE )
            x1 = sum( insampleWLt1 * (1.0_wp-X(:,23)-X(:,24)) * Ed(:,s) * dE * lW )
            call getRatio(x1, x2, moments%lW0%dta(nm,s), moments%lW0%se(nm,s), moments%lW0%sim(nm,s), moments%lW0%sediff(nm,s))
            
            ! SE
            nm  = nm+1
            nb  = 3
            inX = 0; inX(23:24)=1
            pX  => X
            wgt = insampleWLt1*Ed(:,s)*dE
            py  => lW
            call OLS(s, nXwl, inX, n, 1, nb, 0, moments%aux, 0, 1, moments%lW0%sim(nm,s), ar1, ifail)
            if (ifail==0) then
                moments%lW0%sediff(nm,s) = abs(moments%lW0%sim(nm,s)-moments%lW0%dta(nm,s))/moments%lW0%se(nm,s)
            else
                moments%lW0%sediff(nm,s) = penalty
            end if

            ! means for high f1 / f2
            do b=1,2
                nm = nm+1
                x2 = sum( insampleWLt1 * (1.0_wp-X(:,23)-X(:,24)) * Ed(:,s) * dE * X(:,b) )
                x1 = sum( insampleWLt1 * (1.0_wp-X(:,23)-X(:,24)) * Ed(:,s) * dE * X(:,b) * lW )
                call getRatio(x1, x2, moments%lW0%dta(nm,s), moments%lW0%se(nm,s), moments%lW0%sim(nm,s), moments%lW0%sediff(nm,s))
            end do
            
            ! quantiles: p(lW<Q)
            x2 = sum( insampleWLt1 * (1.0_wp-X(:,23)-X(:,24)) * Ed(:,s) * dE )
            do j=1,3
                nm=nm+1
                x1 = sum( insampleWLt1 * (1.0_wp-X(:,23)-X(:,24)) * Ed(:,s) * dE * merge(1.0_wp,0.0_wp,lW<=Q0(j,s)) )
                call getRatio(x1, x2, moments%lW0%dta(nm,s), moments%lW0%se(nm,s), moments%lW0%sim(nm,s), moments%lW0%sediff(nm,s))
            end do

        end do
        moments%lW0%ssdiff = sum( moments%lW0%sediff(1:moments%lW0%n,:)**2 )


        ! Log wages: linear regressions in levels (table 26)
        py  => lW
        pX  => X        
        inX = 0; inX(1:2)=1; inX(12:14) = 1; inX(23:24)=1; inX(25)=1
        nb  = moments%lWregr%n + 3
        do s=1,3
            wgt = insampleWL*insampleWL_1*Ed(:,s)*dE*dE_1
            call OLS(s, nXwl, inX, n, 1, nb, moments%lWregr%n, moments%lWregr, 1, 1, moments%lWdist%sim(1,s), moments%lWdist%sim(2,s), ifail)
            if (ifail==0) then
                moments%lWdist%sediff(1,s) = abs(moments%lWdist%sim(1,s)-moments%lWdist%dta(1,s))/moments%lWdist%se(1,s)
                moments%lWdist%sediff(2,s) = abs(moments%lWdist%sim(2,s)-moments%lWdist%dta(2,s))/moments%lWdist%se(2,s)
            else
                moments%lWdist%sediff(1,s) = penalty
                moments%lWdist%sediff(2,s) = penalty
            end if
        end do


        ! Log wages: linear regressions on age (table 27)
        py  => lW
        pX  => X        
        inX = 0; inX(1:2)=1; inX(22) = 1; inX(23:24)=1
        nb  = moments%lWa%n + 2
        do s=1,3
            wgt = insampleWL*Ed(:,s)*dE
            call OLS(s, nXwl, inX, n, 1, nb, moments%lWa%n, moments%lWa, 1, 0, se, ar1, ifail)
        end do


        ! Log wages: distribution 
        do s=1,3

            ! mean wages in FT (table 28)
            nm = 3
            x1 = sum( insampleWL * (1.0_wp-X(:,23)-X(:,24)) * Ed(:,s) * dE * (1.0_wp-dPT) * lW )
            x2 = sum( insampleWL  * (1.0_wp-X(:,23)-X(:,24))* Ed(:,s) * dE * (1.0_wp-dPT) )
            call getRatio(x1, x2, moments%lWdist%dta(nm,s), moments%lWdist%se(nm,s), moments%lWdist%sim(nm,s), moments%lWdist%sediff(nm,s))

            ! mean wages in PT (table 28)
            nm = nm+1
            x1 = sum( insampleWL * (1.0_wp-X(:,23)-X(:,24)) * Ed(:,s) * dPT * lW )
            x2 = sum( insampleWL * (1.0_wp-X(:,23)-X(:,24)) * Ed(:,s) * dPT )
            call getRatio(x1, x2, moments%lWdist%dta(nm,s), moments%lWdist%se(nm,s), moments%lWdist%sim(nm,s), moments%lWdist%sediff(nm,s))

            ! quantiles in FT (table 28)
            do j=1,5
                nm = nm+1
                x1 = sum( insampleWL * (1.0_wp-X(:,23)-X(:,24)) * Ed(:,s) * dE * (1.0_wp-dPT) * merge(1.0_wp,0.0_wp,lW<=QFT(j,s)) )
                x2 = sum( insampleWL * (1.0_wp-X(:,23)-X(:,24)) * Ed(:,s) * dE * (1.0_wp-dPT) )
                call getRatio(x1, x2, moments%lWdist%dta(nm,s), moments%lWdist%se(nm,s), moments%lWdist%sim(nm,s), moments%lWdist%sediff(nm,s))
            end do

            ! quantiles in PT (table 28)
            do j=1,5
                nm = nm+1
                x1 = sum( insampleWL * (1.0_wp-X(:,23)-X(:,24)) * Ed(:,s) * dPT * merge(1.0_wp,0.0_wp,lW<=QPT(j,s)) )
                x2 = sum( insampleWL * (1.0_wp-X(:,23)-X(:,24)) * Ed(:,s) * dPT )
                call getRatio(x1, x2, moments%lWdist%dta(nm,s), moments%lWdist%se(nm,s), moments%lWdist%sim(nm,s), moments%lWdist%sediff(nm,s))
            end do

            ! mean wages by background ! (table 29)
            do b=1,2
                nm = nm+1
                x1 = sum( insampleWL * (1.0_wp-X(:,23)-X(:,24)) * Ed(:,s) * dE * X(:,b) * lW )
                x2 = sum( insampleWL * (1.0_wp-X(:,23)-X(:,24)) * Ed(:,s) * dE * X(:,b) )
                call getRatio(x1, x2, moments%lWdist%dta(nm,s), moments%lWdist%se(nm,s), moments%lWdist%sim(nm,s), moments%lWdist%sediff(nm,s))
            end do
           
        end do
        moments%lWdist%ssdiff = sum( moments%lWdist%sediff(1:moments%lWdist%n,:)**2 )

        
        ! Log wages: 1st difference regressions on log years worked so far (table 29)
        py  => dlW
        pX  => X        
        inX = 0; inX(15) = 1; inX(23:24)=1
        nb  = moments%dlWe%n + 2
        do s=1,3
            wgt = insampleWL*insampleWL_1*Ed(:,s)*dE*dE_1
            call OLS(s, nXwl, inX, n, 0, nb, moments%dlWe%n, moments%dlWe, 1, 0, se, ar1, ifail)
        end do


        ! Log wages: 1st difference regressions on working hours last period (table 29)
        py  => ydlW
        pX  => X        
        inX = 0; inX(16:18) = 1; inX(23:24)=1
        nb  = moments%dlWy%n + 2
        do s=1,3
            wgt = insample_ydlW*Ed(:,s)
            call OLS(s, nXwl, inX, n, 0, nb, moments%dlWy%n, moments%dlWy, 1, 0, se, ar1, ifail)
        end do


        ! Use of formal CC (table 30)
        do s=1,3
            x1 = sum( insampleWL * (1.0_wp-X(:,23)-X(:,24)) * Ed(:,s) * (X(:,7)+X(:,8)+X(:,9)) * dE * pveCC )
            x2 = sum( insampleWL * (1.0_wp-X(:,23)-X(:,24)) * Ed(:,s) * (X(:,7)+X(:,8)+X(:,9)) * dE )
            call getRatio(x1, x2, moments%CC%dta(1,s), moments%CC%se(1,s), moments%CC%sim(1,s), moments%CC%sediff(1,s))
        end do
        moments%CC%ssdiff = sum( moments%CC%sediff(1,:)**2 )


        ! Finally compute objective function
        !-----------------------------------
        
        ! set dimension of Dvec (to hold differences between simulated and data moments)
        if (allocated(Dvec)) deallocate(Dvec);  allocate (Dvec (totMom,1))
        if (allocated(Q)) deallocate(Q);        allocate (Q (1,1))
        
        ! store differences in order: education moments first (only for s=2,3)
        mi = 0
        do s=2,nts
            forall (j = 1:moments%Sdist%n) Dvec(mi+j,1) = moments%Sdist%dta(j,s)-moments%Sdist%sim(j,s) ;   mi = mi+moments%Sdist%n
            forall (j = 1:moments%Sregr%n) Dvec(mi+j,1) = moments%Sregr%dta(j,s)-moments%Sregr%sim(j,s) ;   mi = mi+moments%Sregr%n
        end do

        ! store diffs in order: WL moments
        do s=1,nts
            forall (j = 1 : moments%E%n)      Dvec(mi+j,1) = moments%E%dta(j,s)-moments%E%sim(j,s) ;            mi = mi+moments%E%n
            forall (j = 1 : moments%PT%n)     Dvec(mi+j,1) = moments%PT%dta(j,s)-moments%PT%sim(j,s) ;          mi = mi+moments%PT%n
            forall (j = 1 : moments%UE%n)     Dvec(mi+j,1) = moments%UE%dta(j,s)-moments%UE%sim(j,s) ;          mi = mi+moments%UE%n
            forall (j = 1 : moments%EU%n)     Dvec(mi+j,1) = moments%EU%dta(j,s)-moments%EU%sim(j,s) ;          mi = mi+moments%EU%n
            forall (j = 1 : moments%lW0%n)    Dvec(mi+j,1) = moments%lW0%dta(j,s)-moments%lW0%sim(j,s) ;        mi = mi+moments%lW0%n
            forall (j = 1 : moments%dlWe%n)   Dvec(mi+j,1) = moments%dlWe%dta(j,s)-moments%dlWe%sim(j,s) ;      mi = mi+moments%dlWe%n
            forall (j = 1 : moments%lWregr%n) Dvec(mi+j,1) = moments%lWregr%dta(j,s)-moments%lWregr%sim(j,s) ;  mi = mi+moments%lWregr%n
            forall (j = 1 : moments%lWa%n)    Dvec(mi+j,1) = moments%lWa%dta(j,s)-moments%lWa%sim(j,s) ;        mi = mi+moments%lWa%n
            forall (j = 1 : moments%dlWy%n)   Dvec(mi+j,1) = moments%dlWy%dta(j,s)-moments%dlWy%sim(j,s) ;      mi = mi+moments%dlWy%n
            forall (j = 1 : moments%lWdist%n) Dvec(mi+j,1) = moments%lWdist%dta(j,s)-moments%lWdist%sim(j,s) ;  mi = mi+moments%lWdist%n
            forall (j = 1 : moments%CC%n)     Dvec(mi+j,1) = moments%CC%dta(j,s)-moments%CC%sim(j,s) ;          mi = mi+moments%CC%n
        end do

        ! calculate value of objective function
        Q     = matmul( matmul(transpose(Dvec),Omega) , Dvec )
        Qform = Q(1,1)
    
    end subroutine CalcSimMoments

    

    !--------------------------------------------------------------------------------------------------
    ! Auxilliary routines
   
    ! Calculate SE differences 
    ! ------------------------
    subroutine getRatio(x1,x2,dtaM,seM,simM,sediff)
        implicit none
        real(wp), intent(in)  :: x1, x2, dtaM, seM
        real(wp), intent(out) :: simM,sediff

        if (x2>tol) then
            simM   = x1 / x2
            sediff = (abs(simM-dtaM)/seM)
        else
            simM = penalty
            sediff = penalty
        end if

    end subroutine getRatio

    

    ! OLS routine
    ! -----------
    subroutine OLS(s, nX, inX, nobs, intercept, nb, nbinc, mom, sestat, resstat, seR, ar1, ifail)
        implicit none
        integer,intent(in)   :: nX, inX(nX), &         ! nr cols of X (all possible regressors except intercept) & which regressors (cols of X) to use
                                nobs, &                ! nr of observations (lines in X)
                                intercept, &           ! whether to include an intercept in the regression model
                                nb, nbinc, &           ! nr of coeffs to estimate and to include as moments
                                sestat, resstat, &     ! whether to compute SEdiffs and SE and autocorr of residuals
                                s                      ! education
        integer, intent(out)      :: ifail
        type(typeM),intent(inout) :: mom
        real(wp),intent(out)      :: seR, ar1
        integer      :: df, rank
        real(wp)     :: rss, se(nb), cov(nb*(nb+1)/2), H(nobs), Q(nobs,nb+1), P(2*nb+nb*nb), wk(2*(nb-1)+nb*nb), res(nobs)
        logical      :: svd
        character(1) :: mean
        

        res   = 0.0_wp
        ifail = -1

        if (sum(wgt(1:nobs)) > tol .and. sum(abs(wgt(1:nobs)*py(1:nobs))) > tol) then
        
            mean = 'M'
            if (intercept==0) mean = 'Z'
            call G02DAF(mean=mean,       weight='W', n=nobs,            X=pX,     ldX=nobs,       &
                        m=nX,            isX=inX,    ip=nb,             y=py,     wt=wgt(1:nobs), &
                        rss=rss,         idf=df,     b=mom%sim(1:nb,s), se=se,    cov=cov,        &
                        res=res(1:nobs), H=H,        Q=Q,               ldQ=nobs, svd=svd,        &
                        irank=rank,      P=P,        tol=tol,           wk=wk,    ifail=ifail     )

            if (ifail==0) then

                ! consider only first nbinc regressors: exc scotland and wales and sometimes t00to08
                if (sestat==1) mom%sediff(1:nbinc,s) = abs(mom%sim(1:nbinc,s)-mom%dta(1:nbinc,s)) / mom%se(1:nbinc,s)
                
                if (resstat==1) then
                    seR = sum( res(1:nobs) * res(1:nobs) * wgt(1:nobs) ) / sum( wgt(1:nobs) )
                    if (sum(wgt(2:nobs)*wgt(1:nobs-1))>tol) then
                        ar1 = sum( res(2:nobs) * res(1:nobs-1) * wgt(2:nobs) * wgt(1:nobs-1) ) / sum( wgt(2:nobs) * wgt(1:nobs-1) )
                    end if
                end if

            else

                mom%sediff(1:nbinc,s) = penalty
                seR = 0.0_wp
                ar1 = 0.0_wp

            end if

        else

            if (sestat==1) mom%sediff(1:nbinc,s) = penalty
            seR = 0.0_wp
            ar1 = 0.0_wp

        end if

        if (sestat==1) mom%ssdiff = mom%ssdiff + sum( mom%sediff(1:nbinc,s)**2 )
       
    end subroutine OLS
    


    ! calculate employment moments
    ! ----------------------------
    subroutine meanE(s, Mdta, SEdta, Msim, SEdiff)
        implicit none
        integer, intent(in)   :: s
        real(wp), intent(in)  :: Mdta(nmom), SEdta(nmom)
        real(wp), intent(out) :: Msim(nmom), SEdiff(nmom)
        integer               :: mi, i, p, ak, m, nb, ifail, inX(nXwl)
        real(wp)              :: x1, x2, in(n), mE(-1:maxmaxakd,0:domales), mEpw(-1:maxmaxakd), mEbk(-1:maxmaxakd,0:domales,2), se, ar1
        type(typeM)           :: aux

        
        ! calculate conditional means (by family demographics)
        
        in = insampleWL * (1.0_wp-X(:,23)-X(:,24)) * Ed(:,s)

        do ak = -1,maxmaxakd
            
            ! by family demographics
            do m = 0,domales
                x1 = sum( in * merge(1.0_wp, 0.0_wp, couple==m .and. agekid==ak) * py )
                x2 = sum( in * merge(1.0_wp, 0.0_wp, couple==m .and. agekid==ak)      )
                if (x2>tol) then
                    mE(ak,m) = x1/x2
                else
                    mE(ak,m) = penalty
                end if
            end do
            
            ! working partner and child's age
            x1 = sum( in * merge(1.0_wp, 0.0_wp, agekid==ak) * X(:,11) * py )
            x2 = sum( in * merge(1.0_wp, 0.0_wp, agekid==ak) * X(:,11)      )
            if (x2>tol) then
                mEpw(ak) = x1/x2
            else
                mEpw(ak) = penalty
            end if

            ! by background and family demographics
            do i  = 1,2
            do m = 0,domales
                x1 = sum( in * merge(1.0_wp, 0.0_wp, couple==m .and. agekid==ak .and. X(:,i)==1) * py )
                x2 = sum( in * merge(1.0_wp, 0.0_wp, couple==m .and. agekid==ak .and. X(:,i)==1)      )
                if (x2>tol) then
                    mEbk(ak,m,i) = x1/x2
                else
                    mEbk(ak,m,i) = penalty
                end if
            end do
            end do
            
        end do


        ! calculate and store weighted means
        
        mi         = 1
        Msim(mi)   = sum(mE(:,:)*famwgh(s,:,:))/sum(famwgh(s,:,:))               ! all women
        SEdiff(mi) = (abs(Msim(mi)-Mdta(mi))/SEdta(mi))

        mi         = mi+1        
        Msim(mi)   = mE(-1,0)                                                    ! single women with no children
        SEdiff(mi) = (abs(Msim(mi)-Mdta(mi))/SEdta(mi))

        mi         = mi+1        
        Msim(mi)   = mE(-1,1)                                                    ! married women with no children
        SEdiff(mi) = (abs(Msim(mi)-Mdta(mi))/SEdta(mi))

        mi         = mi+1        
        Msim(mi)   = sum(mE(0:18,0)*famwgh(s,0:18,0))/sum(famwgh(s,0:18,0))      ! single mothers
        SEdiff(mi) = (abs(Msim(mi)-Mdta(mi))/SEdta(mi))

        mi         = mi+1        
        Msim(mi)   = sum(mE(0:18,1)*famwgh(s,0:18,1))/sum(famwgh(s,0:18,1))      ! mothers in couples
        SEdiff(mi) = (abs(Msim(mi)-Mdta(mi))/SEdta(mi))

        mi         = mi+1        
        Msim(mi)   = sum(mEpw(-1:18)*famwgh(s,-1:18,1))/sum(famwgh(s,-1:18,1))   ! women with a working spouse
        SEdiff(mi) = (abs(Msim(mi)-Mdta(mi))/SEdta(mi))

        mi         = mi+1        
        Msim(mi)   = sum(mE(0:2,:)*famwgh(s,0:2,:))/sum(famwgh(s,0:2,:))         ! mothers: kids aged 0-2
        SEdiff(mi) = (abs(Msim(mi)-Mdta(mi))/SEdta(mi))
        
        mi         = mi+1        
        Msim(mi)   = sum(mE(3:5,:)*famwgh(s,3:5,:))/sum(famwgh(s,3:5,:))         ! mothers: kids aged 3-5
        SEdiff(mi) = (abs(Msim(mi)-Mdta(mi))/SEdta(mi))

        mi         = mi+1        
        Msim(mi)   = sum(mE(6:10,:)*famwgh(s,6:10,:))/sum(famwgh(s,6:10,:))      ! mothers: kids aged 6-10
        SEdiff(mi) = (abs(Msim(mi)-Mdta(mi))/SEdta(mi))

        mi         = mi+1        
        Msim(mi)   = sum(mE(11:18,:)*famwgh(s,11:18,:))/sum(famwgh(s,11:18,:))   ! mothers: kids aged 11-18
        SEdiff(mi) = (abs(Msim(mi)-Mdta(mi))/SEdta(mi))

        do i=1,2                                                                 ! by family background
            mi = mi+1
            x1 = sum( insampleWL * (1.0_wp-X(:,23)-X(:,24)) * Ed(:,s) * X(:,i) * py )
            x2 = sum( insampleWL * (1.0_wp-X(:,23)-X(:,24)) * Ed(:,s) * X(:,i) )
            call getRatio(x1, x2, Mdta(mi), SEdta(mi), Msim(mi), SEdiff(mi))
        end do
        

        ! before-after difference (around 1999)
        
        mi  = mi+1
        pX  => X
        inX = 0; inX(25) = 1; inX(23:24)=1
        nb  = 4
        wgt = insampleWL*Ed(:,s)
        call OLS(s, nXwl, inX, n, 1, nb, nb, aux, 0, 0, se, ar1, ifail)
        if (ifail==0) then
            Msim(mi)   = aux%sim(4,s)
            SEdiff(mi) = abs(Msim(mi)-Mdta(mi))/SEdta(mi)
        else 
            SEdiff(mi) = penalty
        end if
                
    end subroutine meanE



    ! calculate transitions out of employment
    ! ---------------------------------------
    subroutine meanEU(s, Mdta, SEdta, Msim, SEdiff)
        implicit none
        integer, intent(in)   :: s
        real(wp), intent(in)  :: Mdta(nmom), SEdta(nmom)
        real(wp), intent(out) :: Msim(nmom), SEdiff(nmom)
        integer  :: i,mi
        real(wp) :: x1, x2

        
        ! all women 
        mi = 1
        x1 = sum( insampleWL * insampleWL_1 * (1.0_wp-X(:,23)-X(:,24)) * dE_1 * Ed(:,s) * py )
        x2 = sum( insampleWL * insampleWL_1 * (1.0_wp-X(:,23)-X(:,24)) * dE_1 * Ed(:,s) )
        call getRatio(x1, x2, Mdta(mi), SEdta(mi), Msim(mi), SEdiff(mi))
        
        ! women without children
        mi = mi+1
        x1 = sum( insampleWL * insampleWL_1 * (1.0_wp-X(:,23)-X(:,24)) * dE_1 * Ed(:,s) * (X(:,3)+X(:,4)) * py )
        x2 = sum( insampleWL * insampleWL_1 * (1.0_wp-X(:,23)-X(:,24)) * dE_1 * Ed(:,s) * (X(:,3)+X(:,4)) )
        call getRatio(x1, x2, Mdta(mi), SEdta(mi), Msim(mi), SEdiff(mi))

        ! single mothers, married mothers
        do i=5,6
            mi = mi+1
            x1 = sum( insampleWL * insampleWL_1 * (1.0_wp-X(:,23)-X(:,24)) * dE_1 * Ed(:,s) * X(:,i) * py )
            x2 = sum( insampleWL * insampleWL_1 * (1.0_wp-X(:,23)-X(:,24)) * dE_1 * Ed(:,s) * X(:,i) )
            call getRatio(x1, x2, Mdta(mi), SEdta(mi), Msim(mi), SEdiff(mi))
        end do
        
        ! ln(w) below p10, p50, p90
        do i=19,21
            mi = mi+1
            x1 = sum( insampleWL * insampleWL_1 * (1.0_wp-X(:,23)-X(:,24)) * dE_1 * Ed(:,s) * X(:,i) * py )
            x2 = sum( insampleWL * insampleWL_1 * (1.0_wp-X(:,23)-X(:,24)) * dE_1 * Ed(:,s) * X(:,i) )
            call getRatio(x1, x2, Mdta(mi), SEdta(mi), Msim(mi), SEdiff(mi))
        end do

    end subroutine meanEU



    ! calculate transitions to employment
    ! -----------------------------------    
    subroutine meanUE(s, Mdta, SEdta, Msim, SEdiff)
        implicit none
        integer, intent(in)   :: s
        real(wp), intent(in)  :: Mdta(nmom), SEdta(nmom)
        real(wp), intent(out) :: Msim(nmom), SEdiff(nmom)
        integer  :: i,mi
        real(wp) :: x1, x2

        ! all women 
        mi = 1
        x1 = sum( insampleWL * insampleWL_1 * (1.0_wp-X(:,23)-X(:,24)) * Ed(:,s) * (1.0_wp-dE_1) * py )
        x2 = sum( insampleWL * insampleWL_1 * (1.0_wp-X(:,23)-X(:,24)) * Ed(:,s) * (1.0_wp-dE_1) )
        call getRatio(x1, x2, Mdta(mi), SEdta(mi), Msim(mi), SEdiff(mi))
        
        ! women without children
        mi = mi+1
        x1 = sum( insampleWL * insampleWL_1 * (1.0_wp-X(:,23)-X(:,24)) * Ed(:,s) * (1.0_wp-dE_1) * (X(:,3)+X(:,4)) * py )
        x2 = sum( insampleWL * insampleWL_1 * (1.0_wp-X(:,23)-X(:,24)) * Ed(:,s) * (1.0_wp-dE_1) * (X(:,3)+X(:,4)) )
        call getRatio(x1, x2, Mdta(mi), SEdta(mi), Msim(mi), SEdiff(mi))

        ! single mothers, married mothers
        do i=5,6
            mi = mi+1
            x1 = sum( insampleWL * insampleWL_1 * (1.0_wp-X(:,23)-X(:,24)) * Ed(:,s) * (1.0_wp-dE_1) * X(:,i) * py )
            x2 = sum( insampleWL * insampleWL_1 * (1.0_wp-X(:,23)-X(:,24)) * Ed(:,s) * (1.0_wp-dE_1) * X(:,i) )
            call getRatio(x1, x2, Mdta(mi), SEdta(mi), Msim(mi), SEdiff(mi))
        end do

    end subroutine meanUE



    ! Print to file, latex format
    !----------------------------
    subroutine PrintMoments(mom,unit,title)
        implicit none
        type(typeM),intent(in)   :: mom
        integer,intent(in)       :: unit
        character(32),intent(in) :: title
        integer                  :: i, s

        write(unit,*)
        write(unit, '( "\begin{table}[htbp]\centering"                            )')
        write(unit, '( "\caption{", A30, "}"                                      )') title
        write(unit, '( "\begin{tabular}{l*{4}{r}}"                                )')
        write(unit, '( "\hline\hline"                                             )')
        write(unit, '( A25, " & ", A13, " & ", A13, " & ", A13, " & ", A13, " \\" )') "Moment", "Data", "Simulated", "SE data", "No. SE diff"
        write(unit, '( "\hline\hline"                                             )')
        do s=1, nts
            write(unit,'( "\multicolumn{5}{l}{s = ", I1, "} \\" )') s
            do i=1, mom%n
                write(unit,'(A25, " & ", F13.3, " & ", F13.3, " & ", F13.3, " & ", F13.3, " \\" )' ) mom%name(i), mom%dta(i,s), mom%sim(i,s), mom%se(i,s), mom%sediff(i,s)
            end do
            write(unit, '( "\hline" )')
        end do
        write(unit, '( "\hline"         )')
        write(unit, '( "\end{tabular}"  )')
        write(unit, '( "\end{table}"    )')
        write(unit,*)

    end subroutine PrintMoments
    
    
        
    ! set simulated data ready for estimation
    !----------------------------------------
    subroutine getRegrMtrx
        use parameters, only : seME
        implicit none
        integer  :: lo19, lo, hi, is, aN, rep, i, durU, s, a, ni, age25

        
        X             = 0.0_wp
        Xs            = 0.0_wp
        dE            = 0.0_wp
        dPT           = 0.0_wp
        dU            = 0.0_wp
        lW            = 0.0_wp
        dlW           = 0.0_wp
        ydlW          = 0.0_wp
        pveCC         = 0.0_wp
        smtx          = 0.0_wp
        insample_ydlW = 0.0_wp
        insampleWL    = 0.0_wp
        insampleWL_1  = 0.0_wp
        insample_ydlW = 0.0_wp
        insampleWLt1  = 0.0_wp
        dE_1          = 0.0_wp
        insampleS     = 0.0_wp

        lo = 0
        hi = 0
        is = 0
        aN = 50

        do rep = 1,nrsim
        do i   = 1,nind

            s  = sim(i,rep)%s 
            lo19 = hi+1
            lo   = lo19+merge(3,0,s==3)	   ! index for 1st WL observation	
            hi   = hi+nper                 ! index for last WL observation
            is   = is+1                    ! index for education moments
            

            ! WL dependent variables
            ! ----------------------        
            dE(lo:hi)     = merge(0.0_wp, 1.0_wp, sim(i,rep)%aa(as(s):aN)%Ch%l==1)         ! employment
            dPT(lo:hi)    = merge(1.0_wp, 0.0_wp, sim(i,rep)%aa(as(s):aN)%Ch%l==2)         ! PT employment
            dU(lo:hi)     = 1.0_wp-dE(lo:hi)                                               ! not working
            lW(lo:hi)     = log(sim(i,rep)%aa(as(s):aN)%Oh%w) + seME(s)*rvME(lo:hi)        ! log wages with ME
            dlW(lo+1:hi)  = lW(lo+1:hi)-lW(lo:hi-1)                                        ! 1st diff log wages
            pveCC(lo:hi)  = merge(1.0_wp, 0.0_wp, sim(i,rep)%aa(as(s):aN)%Oh%cc>0.0_wp)    ! +ve CC costs

            durU    = 0                    ! [ log(w) - log(w when last worked) ] / dur out-of-work spell
            do a=as(s)+1,aN

                ni = lo+a-as(s)
                    
                if (sim(i,rep)%aa(a-1)%Ch%l .ne. 1) then
                    durU = 1
                else if (durU .ne. 0) then
                    durU = durU+1
                end if
                    
                if (durU>0 .and. sim(i,rep)%aa(a)%Ch%l .ne. 1) then
                    ydlw(ni)          = ( lW(ni) - lW(ni-durU) ) / real(durU,wp)
                    insample_ydlw(ni) = sim(i,rep)%aa(a)%Oh%insample * sim(i,rep)%aa(a-durU)%Oh%insample
                end if

            end do


            ! WL regressors
            ! -------------         
            X(lo:hi,1) = real( sim(i,rep)%f1i-1 , wp )       ! parental background high in 1st factor
            X(lo:hi,2) = real( sim(i,rep)%f2i-1 , wp )       ! parental background high in 2nd factor
            
            X(lo:hi,3) = real( (1-sim(i,rep)%aa(as(s):aN)%Oh%m) * (1-sim(i,rep)%aa(as(s):aN)%Oh%kd) , wp )  ! single with no children
            X(lo:hi,4) = real( sim(i,rep)%aa(as(s):aN)%Oh%m     * (1-sim(i,rep)%aa(as(s):aN)%Oh%kd) , wp )  ! couple with no children
            X(lo:hi,5) = real( (1-sim(i,rep)%aa(as(s):aN)%Oh%m) * sim(i,rep)%aa(as(s):aN)%Oh%kd , wp )      ! lone mother
            X(lo:hi,6) = real( sim(i,rep)%aa(as(s):aN)%Oh%m     * sim(i,rep)%aa(as(s):aN)%Oh%kd , wp )      ! mother in couple
           
            X(lo:hi,7)  = merge(1.0_wp, 0.0_wp, sim(i,rep)%aa(as(s):aN)%Oh%akd>=0  .and. sim(i,rep)%aa(as(s):aN)%Oh%akd<=2)   ! kid aged 0-2
            X(lo:hi,8)  = merge(1.0_wp, 0.0_wp, sim(i,rep)%aa(as(s):aN)%Oh%akd>=3  .and. sim(i,rep)%aa(as(s):aN)%Oh%akd<=5)   ! kid aged 3-5
            X(lo:hi,9)  = merge(1.0_wp, 0.0_wp, sim(i,rep)%aa(as(s):aN)%Oh%akd>=6  .and. sim(i,rep)%aa(as(s):aN)%Oh%akd<=10)  ! kid aged 6-10
            X(lo:hi,10) = merge(1.0_wp, 0.0_wp, sim(i,rep)%aa(as(s):aN)%Oh%akd>=11 .and. sim(i,rep)%aa(as(s):aN)%Oh%akd<=18)  ! kid aged 11-18

            X(lo:hi,11) = real( sim(i,rep)%aa(as(s):aN)%Oh%lm-1 , wp )                     ! partner working
                
            X(lo+1:hi,12) = lW(lo:hi-1)                                                    ! lagged log wage rates
            X(lo:hi,13)   = log( real( sim(i,rep)%aa(as(s):aN)%Oh%nDl + 1, wp) )           ! log of years worked so far
            X(lo+1:hi,14) = X(lo:hi-1,13)                                                  ! lagged log of years worked so far
            X(lo+1:hi,15) = X(lo+1:hi,13)-X(lo+1:hi,14)                                    ! 1st diffs log of years worked so far

            X(lo+1:hi,16) = merge(1.0_wp, 0.0_wp, sim(i,rep)%aa(as(s):aN-1)%Ch%l==3)       ! lagged working status: FT
            X(lo+1:hi,17) = merge(1.0_wp, 0.0_wp, sim(i,rep)%aa(as(s):aN-1)%Ch%l==2)       ! lagged working status: PT
            X(lo+1:hi,18) = merge(1.0_wp, 0.0_wp, sim(i,rep)%aa(as(s):aN-1)%Ch%l==1)       ! lagged working status: NW

            X(lo+1:hi,19) = merge(1.0_wp, 0.0_wp, sim(i,rep)%aa(as(s):aN-1)%Ch%l>=2 .and. lW(lo:hi-1)<=QWL(1))  ! wage rate<c10 last year
            X(lo+1:hi,20) = merge(1.0_wp, 0.0_wp, sim(i,rep)%aa(as(s):aN-1)%Ch%l>=2 .and. lW(lo:hi-1)<=QWL(2))  ! wage rate<c50 last year
            X(lo+1:hi,21) = merge(1.0_wp, 0.0_wp, sim(i,rep)%aa(as(s):aN-1)%Ch%l>=2 .and. lW(lo:hi-1)<=QWL(3))  ! wage rate<c90 last year

            forall (a=19:50) X(lo19+a-19,22) = real(a,wp)/10.0_wp                          ! age/10
            
            X(lo:hi,23) = real( sim(i,rep)%scotland, wp )                                  ! lives in scotland
            X(lo:hi,24) = real( sim(i,rep)%wales, wp )                                     ! lives in wales

            X(lo:hi,25) = merge(1.0_wp, 0.0_wp, sim(i,rep)%aa(as(s):aN)%Oh%year>=2000 .and. sim(i,rep)%aa(as(s):aN)%Oh%year<=2008)  ! 2000-08
            
 
            ! WL sample selection
            ! -------------------
            insampleWL(lo:hi)     = real( sim(i,rep)%aa(as(s):aN)%Oh%insample, wp )
            insampleWL_1(lo+1:hi) = insampleWL(lo:hi-1 )
            dE_1(lo+1:hi)         = dE(lo:hi-1)
            Ed(lo:hi,1)           = merge( 1.0_wp, 0.0_wp, s==1 ) 
            Ed(lo:hi,2)           = merge( 1.0_wp, 0.0_wp, s==2 ) 
            Ed(lo:hi,3)           = merge( 1.0_wp, 0.0_wp, s==3 ) 
            couple(lo:hi)         = sim(i,rep)%aa(as(s):aN)%Oh%m
            agekid(lo:hi)         = sim(i,rep)%aa(as(s):aN)%Oh%akd

            insampleWLcum(lo) = insampleWL(lo)
            do ni=lo+1,hi
                insampleWLcum(ni) = insampleWLcum(ni-1)+insampleWL(ni)
            end do
            insampleWLcum(lo:hi) = insampleWLcum(lo:hi)*insampleWL(lo:hi)

            age25 = lo+3+merge(3,0,s<3)
            insampleWLt1(lo:age25) = real (sim(i,rep)%student) * merge( 1.0_wp, 0.0_wp, abs(insampleWLcum(lo:age25)-1.0_wp)<=0.01_wp )


            ! Regressors: education choice
            Xs(is,1) = merge(1.0_wp, 0.0_wp, sim(i,rep)%g80==1)        ! dummy for born 82+
            Xs(is,2) = sim(i,rep)%f1                                   ! background factors
            Xs(is,3) = sim(i,rep)%f2
            Xs(is,4) = Xs(is,1)*Xs(is,2)
            Xs(is,5) = Xs(is,1)*Xs(is,3)
            Xs(is,6) = sim(i,rep)%parentYres                           ! residual, log parental income
            Xs(is,7) = merge(1.0_wp, 0.0_wp, sim(i,rep)%scotland==1)   ! indicator: scotland
            Xs(is,8) = merge(1.0_wp, 0.0_wp, sim(i,rep)%wales==1)      ! indicator: wales

            ! education choice dependent variables
            smtx(1,is) = merge( 1.0_wp, 0.0_wp, s==1)
            smtx(2,is) = merge( 1.0_wp, 0.0_wp, s==2)
            smtx(3,is) = merge( 1.0_wp, 0.0_wp, s==3)

            ! education choice sample selection
            insampleS(is) = real(sim(i,rep)%sampleS, wp)
            
        end do
        end do
        
    end subroutine getRegrMtrx



    ! Load data moments from file
    !----------------------------
    subroutine LoadDataMoments(filename, nchar, mom)
        use precision, only : wp
        implicit none
        integer, intent(in)         :: nchar
        character(nchar),intent(in) :: filename
        type(typeM),intent(out)     :: mom
        integer                     :: unit, ios, s

        ! Initialise mom
        mom%n        = 1
        mom%name     = ''
        mom%dta      = 0.0_wp
        mom%dtaQ     = 0.0_wp
        mom%se       = 0.0_wp
        mom%sim      = 0.0_wp
        mom%sediff   = 0.0_wp
        mom%ssdiff   = 0.0_wp

        ! open file containing moments info
        unit = 4
        open(unit, file=filename, action='read')
        
        ! Read lines
        do
            read(unit, 234, iostat=ios) mom%name(mom%n), mom%dta(mom%n,1), mom%se(mom%n,1), &
                                        mom%dta(mom%n,2), mom%se(mom%n,2), mom%dta(mom%n,3), mom%se(mom%n,3)

            ! Exit do loop if end of file
            if (ios < 0) then
                mom%n = mom%n - 1
                exit
            end if

            ! If not blank lines/comments...
            if ((mom%name(mom%n)(1:1) .ne. " ") .and. (mom%name(mom%n)(1:1) .ne. "!")) then
                mom%n = mom%n + 1
            end if

        end do

        close (unit)
        234 format (A12,6F13.8)

    end subroutine LoadDataMoments



    ! Load data quantiles from file
    !------------------------------
    subroutine LoadDataQ(path)
        use precision, only : wp
        implicit none
        character(*),intent(in) :: path
        character(12)           :: row
        integer                 :: unit, ios, mi

        ! Initialise Q matrices
        QFT = 0.0_wp
        QPT = 0.0_wp
        QWL = 0.0_wp
        Q0  = 0.0_wp
        
        ! quantiles for FT workers, by education
        unit = 3857
        open(unit, file=path//'QFTs.txt', action='read')
        do mi=1,5
            read(unit, 232, iostat=ios) row, QFT(mi,1), QFT(mi,2), QFT(mi,3)
        end do
        close (unit)
        
        ! quantiles for PT workers, by education
        unit = 524
        open(unit, file=path//'QPTs.txt', action='read')
        do mi = 1,5
            read(unit, 232, iostat=ios) row, QPT(mi,1), QPT(mi,2), QPT(mi,3)
        end do
        close (unit)
        
        ! quantiles for WL, all workers, all education groups
        unit = 612
        open(unit, file=path//'QWL.txt', action='read')
        read(unit, 232, iostat=ios) row, QWL(1), QWL(2), QWL(3)
        close (unit)

        ! quantiles for workers at entrance in WL, by education
        unit = 524
        open(unit, file=path//'Qentry.txt', action='read')
        do mi=1,3
            read(unit, 232, iostat=ios) row, Q0(mi,1), Q0(mi,2), Q0(mi,3)
        end do
        close (unit)
        
        232 format (A12,3F13.8)

    end subroutine LoadDataQ

end module momMod
