! -----------------------------------------------------------------------------------------------------------
! -----------------------------------------------------------------------------------------------------------
! Routines to I/O the values of the parameters (both constant and to estimate)
!   and to fill structures of original and derived parameters
! -----------------------------------------------------------------------------------------------------------
! -----------------------------------------------------------------------------------------------------------

module parametersfcs

use precision,  only : wp
use constants
use parameters
use gridvars,   only : ngpl, ngplm, ngppm, minKfloor
use IOpaths


contains

   
    ! -------------------------------------------------------------------------------------------------------
    ! Load constant parameters
    
    subroutine CttParameters
        implicit none 
        
        R    = 1.015_wp            ! interest rate
        beta = 1.0_wp/1.02_wp      ! diiscount rate
        mu0  = -0.56_wp            ! curvature of utility fc: consumption

        ! Men: wages (by sm)
        mktwm  = (/7.02_wp, 7.92_wp, 7.77_wp/)    ! market wage values (2008 wage levels)
        alpham = (/0.09_wp, 0.18_wp, 0.35_wp/)    ! Coeff on log female age
        rhom   = 0.99_wp                          ! Autocorrelation coefficient
        sepmas = (/0.37_wp, 0.36_wp, 0.39_wp/)    ! SE initial productivity
        seqm   = (/0.12_wp, 0.10_wp, 0.10_wp/)    ! SE productivity shock

        ! Men: employment probabilities (by sm)
        prlmnew = (/ 0.74_wp, 0.87_wp, 0.83_wp /)  ! probability man in new couple is working
        dlta0   = (/ 0.05_wp, 0.36_wp, 0.58_wp /)  ! probability man in ongoing couple is working if not working last period
        dlta1   = (/ 1.52_wp, 1.39_wp, 1.27_wp /)  ! probability man in ongoing couple is working if working last period

        ! Cost of childcare: hourly rate (2008 wage levels)
        ccPerHr = 2.60_wp     

        ! university fees (1000*3yrs, 2008 wage levels) and max loans (fee+2,000)
        Ufee      = merge(3000.0_wp, 0.0_wp, dUfee==1)/(52.0_wp*p99) 
        minKfloor = (/ 0.0_wp, 0.0_wp, merge(-5000.0_wp, -2000.0_wp, dUfee==1)/(52.0_wp*p99) /)

    end subroutine CttParameters

    

    ! -------------------------------------------------------------------------------------------------------
    ! calculate parameters marked as "derived" in module parameters under "constant parameters"
    ! and grids that remain unchanged with estimation

    subroutine DerivedCttParametersAndGrds
        use gridfcs, only : popge, popgk, popkdvec, popgpm
        implicit none
        integer a,s,sm
        real(wp) gSN(ngppm)

        !discount rates to compute present value of income variables
        forall (a=mina:maxa) disc(a) = R**(mina-a)  

        ! utility parameters
        omu0      = 1.0_wp/mu0                          ! reciprocal of mu0
        omu0l1    = 1.0_wp/(mu0-1.0_wp)                 ! power for approx inverse
        mu0omu0l1 = mu0/(mu0-1.0_wp)                    ! power in Euler equation
        phi       = (beta**(1.0_wp/(1.0_wp-mu0)))*(R**(mu0/(1.0_wp-mu0)))   ! EE ratio of C for retirement periods

        ! Grids for e, k and kids
        call popge              ! experience
        call popgk              ! assets: IS needed (run fortax before)
        call popkdvec           ! bounds on age of present kids

        ! Standard error for pm by (a,sm)
        do sm=1,ntsm
            sepm(sm,:) = sepmas(sm)
            do a=mina+1, maxa
                sepm(sm,a) = sqrt(rhom**2.0_wp * sepm(sm,a-1)**2.0_wp  + (seqm(sm)**2.0_wp))
            end do
        end do
        
        ! grid for spouse's productivity shock (pm) and transition matrix
        call popgpm

        ! Family related parameters
        call popCcCost          ! CC cost
        call popfamtransprobs   ! family transition matrix

    end subroutine DerivedCttParametersAndGrds



    ! -------------------------------------------------------------------------------------------------------
    ! Parameters marked as "derived" in module parameters under "parameters to estimate"
    ! need to be re-calculated in each iteration of estimation procedure

    subroutine DerivedParameters(prisk)
        implicit none
        real(wp), intent(in) :: prisk
        integer a,s,sm,th

        !preference parameters: th=2
        do s=1,nts
            Epas(s,2) = -ptheta1*Epas(s,1)/(1.0_wp-ptheta1)
        end do

        do s=1,nts
            sep(s,mina:as(s)) = sepas(s) * sqrt(prisk)
            do th=1,ntth
                Ep(s,th,mina:as(s)) = Epas(s,th) + 0.5_wp*(1.0_wp-prisk)*(sepas(s)**2)
            end do
            do a=as(s)+1, maxa
                sep(s,a) = sqrt((rho(s)**2.0_wp) * (sep(s,a-1)**2.0_wp)  + (seq(s)**2.0_wp))
                do th=1,ntth
                    Ep(s,th,a) = rho(s)*Ep(s,th,a-1) + Eq(s)
                end do
            end do
        end do

        ! Utility parameters
        call popuparams

    end subroutine DerivedParameters



    ! -------------------------------------------------------------------------------------------------------
    ! Matrix of work-related utility scalers

    subroutine popuparams
        use gridvars, only : ngpl, ngplm
        implicit none
        integer  m, kd, akd, l2, th, lm, l, s, f1i, f2i
        real(wp) lambda, rH1, rH2, rdm, rmw, rdk


        do th=1, ntth

            ! case l=1 - not working
            mu1(1,th)%b    = theta(1,th)
            expmu1(1,th)%b = exp(mu1(1,th)%b)

            lambda = exp(theta(1,th)/(mu0-1.0_wp))
            do akd=-1, maxmaxakd               ! age of kid (case akd=-1: no kids)
            do m=0,domales                     ! whether partner present
                kd = merge(0,1,akd==-1)
                EEpar(1,th)%b(:,m,akd,:,:,:) = ((beta*R)**omu0l1) * (eqsc(m,kd)**mu0omu0l1) / lambda
            end do
            end do


            ! case l=2/3 - working PT or FT
            mu1(2,th)%b = 0.0_wp
            mu1(3,th)%b = 0.0_wp

            do l=2, ngpl

                l2 = merge(1.0_wp,0.0_wp,l==2) ! whether PT

                do f2i=1,ntb2                  ! factor 2
                do f1i=1,ntb1                  ! factor 1
                do s=1,nts                     ! education
                do akd=-1, maxmaxakd           ! age of kid (case akd=-1: no kids)
                do m=0,domales                 ! whether partner present
                do lm=1,1+m*(ngplm-1)          ! working status of parter

                    rH1 = real(f1i-1,wp)
                    rH2 = real(f2i-1,wp)
                    rdm = real(m,wp)
                    rmw = merge(1.0_wp,0.0_wp,lm>1)
                    rdk = merge(1.0_wp,0.0_wp,akd>-1)
                    
                    mu1(l,th)%b(lm,m,akd,s,f1i,f2i) = theta(l,th) + rH1*(Ub1+l2*Ub1PT) + rH2*(Ub2+l2*Ub2PT) + rdm*rmw*(Umwk+l2*UmwkPT) + &
                                                           (1.0_wp-rdk) * ( (1.0_wp-rdm)*(U(s)+l2*UPT(s))   + rdm*(Um(s)+l2*UmPT(s)) )       + &
                                                           rdk          * ( (1.0_wp-rdm)*(Uk(s)+l2*UkPT(s)) + rdm*(Umk(s)+l2*UmkPT(s))       + &
                                                                            (Uk02+l2*Uk02PT  ) * merge(1.0_wp,0.0_wp,akd<=2)                 + &
                                                                            (Uk35+l2*Uk35PT  ) * merge(1.0_wp,0.0_wp,akd>2 .and. akd<=5)     + &
                                                                            (Uk610+l2*Uk610PT) * merge(1.0_wp,0.0_wp,akd>5 .and. akd<=10) )

                    kd    = merge(0,1,akd==-1)
                    lambda = exp(mu1(l,th)%b(lm,m,akd,s,f1i,f2i)/(mu0-1.0_wp))
                    EEpar(l,th)%b(lm,m,akd,s,f1i,f2i) = ((beta*R)**omu0l1) * (eqsc(m,kd)**mu0omu0l1) / lambda

                 end do
                 end do
                 end do
                 end do
                 end do
                 end do

                 expmu1(l,th)%b = exp(mu1(l,th)%b)

             end do
        end do

    end subroutine popuparams
    


    ! -------------------------------------------------------------------------------------------------------
    ! childcare costs by family composition

    subroutine popCcCost
        use gridvars, only : ngpl, gl
        implicit none
        integer l, m

        ccCost = 0.0_wp
        
        if (dokids==1) then
            do l  = 2,ngpl
            do m  = 0,domales
                ccCost(l,m,:,0:aSchSt)       = gl(l)*ccPerHr                         !younger child (up to 5)
                ccCost(l,m,:,aSchSt+1:aCcHi) = max(gl(l)-schHrs,0.0_wp) * ccPerHr    !child aged 6-10
                if (m==1) then
                    ccCost(l,m,1,:) = 0.0_wp
                end if
            end do
            end do
        end if

    end subroutine popCcCost



    ! -------------------------------------------------------------------------------------------------------
    ! Populate family transition matrices: child and male arrival and male departure
    ! and transition in male characteristics: (sm, lm, pm) 

    subroutine popfamtransprobs
        use statamod
        use nag_g_ib,  only : g01eaf
        use gridvars,  only : ngppm, gpm
        use gridfcs,   only : trprob
        implicit none
        integer  :: m_1, s, sm, a, kd_1, akd_1, pmi, pmi_1, lm, lm_1, ifail
        integer  :: stataUnit, nobs
        real(wp) :: correct_pr0m(nts), transitionpm(ngppm)

        
        ! Read in family transition probabilities
        
        stataUnit = 3
        call openStata(parampath//"familytransitions.dta",stataUnit)

        nobs = nobsStata()
        
        ! Kid arrival
        call readStata(prkd(:,1,0,-1), 'kdArrPoly_s1_dm0_nokd')
        call readStata(prkd(:,2,0,-1), 'kdArrPoly_s2_dm0_nokd')
        call readStata(prkd(:,3,0,-1), 'kdArrPoly_s3_dm0_nokd')
        call readStata(prkd(:,1,1,-1), 'kdArrPoly_s1_dm1_nokd')
        call readStata(prkd(:,2,1,-1), 'kdArrPoly_s2_dm1_nokd')
        call readStata(prkd(:,3,1,-1), 'kdArrPoly_s3_dm1_nokd')

        ! Kid arrival: kid at home aged 0
        call readStata(prkd(:,1,0,0), 'kdArrPoly_s1_dm0_akd0')
        call readStata(prkd(:,2,0,0), 'kdArrPoly_s2_dm0_akd0')
        call readStata(prkd(:,3,0,0), 'kdArrPoly_s3_dm0_akd0')
        call readStata(prkd(:,1,1,0), 'kdArrPoly_s1_dm1_akd0')
        call readStata(prkd(:,2,1,0), 'kdArrPoly_s2_dm1_akd0')
        call readStata(prkd(:,3,1,0), 'kdArrPoly_s3_dm1_akd0')

        ! Kid arrival: kid at home aged 1
        call readStata(prkd(:,1,0,1), 'kdArrPoly_s1_dm0_akd1')
        call readStata(prkd(:,2,0,1), 'kdArrPoly_s2_dm0_akd1')
        call readStata(prkd(:,3,0,1), 'kdArrPoly_s3_dm0_akd1')
        call readStata(prkd(:,1,1,1), 'kdArrPoly_s1_dm1_akd1')
        call readStata(prkd(:,2,1,1), 'kdArrPoly_s2_dm1_akd1')
        call readStata(prkd(:,3,1,1), 'kdArrPoly_s3_dm1_akd1')

        ! Kid arrival: kid at home aged 2
        call readStata(prkd(:,1,0,2), 'kdArrPoly_s1_dm0_akd2')
        call readStata(prkd(:,2,0,2), 'kdArrPoly_s2_dm0_akd2')
        call readStata(prkd(:,3,0,2), 'kdArrPoly_s3_dm0_akd2')
        call readStata(prkd(:,1,1,2), 'kdArrPoly_s1_dm1_akd2')
        call readStata(prkd(:,2,1,2), 'kdArrPoly_s2_dm1_akd2')
        call readStata(prkd(:,3,1,2), 'kdArrPoly_s3_dm1_akd2')

        ! Kid arrival: kid at home aged 3
        call readStata(prkd(:,1,0,3), 'kdArrPoly_s1_dm0_akd3')
        call readStata(prkd(:,2,0,3), 'kdArrPoly_s2_dm0_akd3')
        call readStata(prkd(:,3,0,3), 'kdArrPoly_s3_dm0_akd3')
        call readStata(prkd(:,1,1,3), 'kdArrPoly_s1_dm1_akd3')
        call readStata(prkd(:,2,1,3), 'kdArrPoly_s2_dm1_akd3')
        call readStata(prkd(:,3,1,3), 'kdArrPoly_s3_dm1_akd3')

        ! Kid arrival: kid at home aged 4
        call readStata(prkd(:,1,0,4), 'kdArrPoly_s1_dm0_akd4')
        call readStata(prkd(:,2,0,4), 'kdArrPoly_s2_dm0_akd4')
        call readStata(prkd(:,3,0,4), 'kdArrPoly_s3_dm0_akd4')
        call readStata(prkd(:,1,1,4), 'kdArrPoly_s1_dm1_akd4')
        call readStata(prkd(:,2,1,4), 'kdArrPoly_s2_dm1_akd4')
        call readStata(prkd(:,3,1,4), 'kdArrPoly_s3_dm1_akd4')

        ! Kid arrival: kid at home aged 5
        call readStata(prkd(:,1,0,5), 'kdArrPoly_s1_dm0_akd5')
        call readStata(prkd(:,2,0,5), 'kdArrPoly_s2_dm0_akd5')
        call readStata(prkd(:,3,0,5), 'kdArrPoly_s3_dm0_akd5')
        call readStata(prkd(:,1,1,5), 'kdArrPoly_s1_dm1_akd5')
        call readStata(prkd(:,2,1,5), 'kdArrPoly_s2_dm1_akd5')
        call readStata(prkd(:,3,1,5), 'kdArrPoly_s3_dm1_akd5')

        ! Kid arrival: kid at home aged 6
        call readStata(prkd(:,1,0,6), 'kdArrPoly_s1_dm0_akd6')
        call readStata(prkd(:,2,0,6), 'kdArrPoly_s2_dm0_akd6')
        call readStata(prkd(:,3,0,6), 'kdArrPoly_s3_dm0_akd6')
        call readStata(prkd(:,1,1,6), 'kdArrPoly_s1_dm1_akd6')
        call readStata(prkd(:,2,1,6), 'kdArrPoly_s2_dm1_akd6')
        call readStata(prkd(:,3,1,6), 'kdArrPoly_s3_dm1_akd6')

        ! Kid arrival: kid at home aged 7
        call readStata(prkd(:,1,0,7), 'kdArrPoly_s1_dm0_akd7')
        call readStata(prkd(:,2,0,7), 'kdArrPoly_s2_dm0_akd7')
        call readStata(prkd(:,3,0,7), 'kdArrPoly_s3_dm0_akd7')
        call readStata(prkd(:,1,1,7), 'kdArrPoly_s1_dm1_akd7')
        call readStata(prkd(:,2,1,7), 'kdArrPoly_s2_dm1_akd7')
        call readStata(prkd(:,3,1,7), 'kdArrPoly_s3_dm1_akd7')

        ! Kid arrival: kid at home aged 8
        call readStata(prkd(:,1,0,8), 'kdArrPoly_s1_dm0_akd8')
        call readStata(prkd(:,2,0,8), 'kdArrPoly_s2_dm0_akd8')
        call readStata(prkd(:,3,0,8), 'kdArrPoly_s3_dm0_akd8')
        call readStata(prkd(:,1,1,8), 'kdArrPoly_s1_dm1_akd8')
        call readStata(prkd(:,2,1,8), 'kdArrPoly_s2_dm1_akd8')
        call readStata(prkd(:,3,1,8), 'kdArrPoly_s3_dm1_akd8')

        ! Kid arrival: kid at home aged 9
        call readStata(prkd(:,1,0,9), 'kdArrPoly_s1_dm0_akd9')
        call readStata(prkd(:,2,0,9), 'kdArrPoly_s2_dm0_akd9')
        call readStata(prkd(:,3,0,9), 'kdArrPoly_s3_dm0_akd9')
        call readStata(prkd(:,1,1,9), 'kdArrPoly_s1_dm1_akd9')
        call readStata(prkd(:,2,1,9), 'kdArrPoly_s2_dm1_akd9')
        call readStata(prkd(:,3,1,9), 'kdArrPoly_s3_dm1_akd9')

        ! Kid arrival: kid at home aged 10
        call readStata(prkd(:,1,0,10), 'kdArrPoly_s1_dm0_akd10')
        call readStata(prkd(:,2,0,10), 'kdArrPoly_s2_dm0_akd10')
        call readStata(prkd(:,3,0,10), 'kdArrPoly_s3_dm0_akd10')
        call readStata(prkd(:,1,1,10), 'kdArrPoly_s1_dm1_akd10')
        call readStata(prkd(:,2,1,10), 'kdArrPoly_s2_dm1_akd10')
        call readStata(prkd(:,3,1,10), 'kdArrPoly_s3_dm1_akd10')

        ! Kid arrival: kid at home aged 11
        call readStata(prkd(:,1,0,11), 'kdArrPoly_s1_dm0_akd11')
        call readStata(prkd(:,2,0,11), 'kdArrPoly_s2_dm0_akd11')
        call readStata(prkd(:,3,0,11), 'kdArrPoly_s3_dm0_akd11')
        call readStata(prkd(:,1,1,11), 'kdArrPoly_s1_dm1_akd11')
        call readStata(prkd(:,2,1,11), 'kdArrPoly_s2_dm1_akd11')
        call readStata(prkd(:,3,1,11), 'kdArrPoly_s3_dm1_akd11')

        ! Kid arrival: kid at home aged 12
        call readStata(prkd(:,1,0,12), 'kdArrPoly_s1_dm0_akd12')
        call readStata(prkd(:,2,0,12), 'kdArrPoly_s2_dm0_akd12')
        call readStata(prkd(:,3,0,12), 'kdArrPoly_s3_dm0_akd12')
        call readStata(prkd(:,1,1,12), 'kdArrPoly_s1_dm1_akd12')
        call readStata(prkd(:,2,1,12), 'kdArrPoly_s2_dm1_akd12')
        call readStata(prkd(:,3,1,12), 'kdArrPoly_s3_dm1_akd12')

        ! Kid arrival: kid at home aged 13
        call readStata(prkd(:,1,0,13), 'kdArrPoly_s1_dm0_akd13')
        call readStata(prkd(:,2,0,13), 'kdArrPoly_s2_dm0_akd13')
        call readStata(prkd(:,3,0,13), 'kdArrPoly_s3_dm0_akd13')
        call readStata(prkd(:,1,1,13), 'kdArrPoly_s1_dm1_akd13')
        call readStata(prkd(:,2,1,13), 'kdArrPoly_s2_dm1_akd13')
        call readStata(prkd(:,3,1,13), 'kdArrPoly_s3_dm1_akd13')

        ! Kid arrival: kid at home aged 14
        call readStata(prkd(:,1,0,14), 'kdArrPoly_s1_dm0_akd14')
        call readStata(prkd(:,2,0,14), 'kdArrPoly_s2_dm0_akd14')
        call readStata(prkd(:,3,0,14), 'kdArrPoly_s3_dm0_akd14')
        call readStata(prkd(:,1,1,14), 'kdArrPoly_s1_dm1_akd14')
        call readStata(prkd(:,2,1,14), 'kdArrPoly_s2_dm1_akd14')
        call readStata(prkd(:,3,1,14), 'kdArrPoly_s3_dm1_akd14')

        ! Kid arrival: kid at home aged 15
        call readStata(prkd(:,1,0,15), 'kdArrPoly_s1_dm0_akd15')
        call readStata(prkd(:,2,0,15), 'kdArrPoly_s2_dm0_akd15')
        call readStata(prkd(:,3,0,15), 'kdArrPoly_s3_dm0_akd15')
        call readStata(prkd(:,1,1,15), 'kdArrPoly_s1_dm1_akd15')
        call readStata(prkd(:,2,1,15), 'kdArrPoly_s2_dm1_akd15')
        call readStata(prkd(:,3,1,15), 'kdArrPoly_s3_dm1_akd15')

        ! Kid arrival: kid at home aged 16
        call readStata(prkd(:,1,0,16), 'kdArrPoly_s1_dm0_akd16')
        call readStata(prkd(:,2,0,16), 'kdArrPoly_s2_dm0_akd16')
        call readStata(prkd(:,3,0,16), 'kdArrPoly_s3_dm0_akd16')
        call readStata(prkd(:,1,1,16), 'kdArrPoly_s1_dm1_akd16')
        call readStata(prkd(:,2,1,16), 'kdArrPoly_s2_dm1_akd16')
        call readStata(prkd(:,3,1,16), 'kdArrPoly_s3_dm1_akd16')

        ! Kid arrival: kid at home aged 17
        call readStata(prkd(:,1,0,17), 'kdArrPoly_s1_dm0_akd17')
        call readStata(prkd(:,2,0,17), 'kdArrPoly_s2_dm0_akd17')
        call readStata(prkd(:,3,0,17), 'kdArrPoly_s3_dm0_akd17')
        call readStata(prkd(:,1,1,17), 'kdArrPoly_s1_dm1_akd17')
        call readStata(prkd(:,2,1,17), 'kdArrPoly_s2_dm1_akd17')
        call readStata(prkd(:,3,1,17), 'kdArrPoly_s3_dm1_akd17')

        ! Kid arrival: kid at home aged 18
        call readStata(prkd(:,1,0,18), 'kdArrPoly_s1_dm0_akd18')
        call readStata(prkd(:,2,0,18), 'kdArrPoly_s2_dm0_akd18')
        call readStata(prkd(:,3,0,18), 'kdArrPoly_s3_dm0_akd18')
        call readStata(prkd(:,1,1,18), 'kdArrPoly_s1_dm1_akd18')
        call readStata(prkd(:,2,1,18), 'kdArrPoly_s2_dm1_akd18')
        call readStata(prkd(:,3,1,18), 'kdArrPoly_s3_dm1_akd18')

        ! Men: probability of present spouse today conditional on his presence last period
        call readStata(pr0m(1,:,1), 'sm1ArrPoly_s1')        ! new marriage, by (sm,mina:enda,s)
        call readStata(pr0m(2,:,1), 'sm2ArrPoly_s1')
        call readStata(pr0m(3,:,1), 'sm3ArrPoly_s1')
        call readStata(pr0m(1,:,2), 'sm1ArrPoly_s2')
        call readStata(pr0m(2,:,2), 'sm2ArrPoly_s2')
        call readStata(pr0m(3,:,2), 'sm3ArrPoly_s2')
        call readStata(pr0m(1,:,3), 'sm1ArrPoly_s3')
        call readStata(pr0m(2,:,3), 'sm2ArrPoly_s3')
        call readStata(pr0m(3,:,3), 'sm3ArrPoly_s3')

        call readStata(trCS(1,0,:), 'sm1GoPoly_dk0')        ! ongoing marriage, by (sm_1,kd_1,mina:enda)
        call readStata(trCS(2,0,:), 'sm2GoPoly_dk0')
        call readStata(trCS(3,0,:), 'sm3GoPoly_dk0')
        call readStata(trCS(1,1,:), 'sm1GoPoly_dk1')
        call readStata(trCS(2,1,:), 'sm2GoPoly_dk1')
        call readStata(trCS(3,1,:), 'sm3GoPoly_dk1')

        call closeOpenStata() 


        ! new sopuse sm at a(s) for woman s
        do s=1,3
            do sm=1,3
                correct_pr0m(sm) =  (initfam(s,1,0)+initfam(s,1,1)) * pr0m(sm,as(s),s) / sum(pr0m(:,as(s),s))
            end do
            pr0m(:,as(s),s) = correct_pr0m
        end do


        ! new spouse (sm,lm,pmi) when he was not present in previous period by (a,s)
        forall (s=1:nts, a=mina:maxa) trSC(a,s)%m = 0.0_wp
        do s = 1,nts
        do a = as(s),maxa
            trSS(a,s) = 1.0_wp - sum(pr0m(:,a,s))                                              ! no new spouse
            forall (pmi=1:ngppm)
                trSC(a,s)%m(:,2,pmi) = pr0m(:,a,s) * prlmnew / real(ngppm,wp)                  ! new spouse working
                trSC(a,s)%m(:,1,pmi) = pr0m(:,a,s) * (1.0_wp-prlmnew(:)) / real(ngppm,wp)      ! new spouse not working
            end forall
        end do
        end do

    
        ! ongoing couples at a
        ! probability that spouse (sm,lm_1) is employedtoday
        trlm = 0.0_wp
        ifail = 0
        do sm=1, ntsm
        do lm_1=1, ngplm
            trlm(lm_1,sm) = g01eaf('L', dlta0(sm) + dlta1(sm)*real(lm_1-1,wp), ifail)
        end do
        end do
        

        ! ongoing couples at a
        ! transition in spouse's characteristics in ongoing couples
        forall (sm=1:ntsm, lm=1:ngplm, pmi=1:ngppm) trCC(:,:,:,:)%m(sm,lm,pmi) = 0.0_wp     

        do a     = mina+1, maxa     ! age today
        do sm    = 1, ntsm          ! if in couple yesterday the education of partner does not change
        do pmi_1 = 1, ngppm         ! partner's productivity yesterday

            ! transition in productivity
            call trprob(gpm(pmi_1,sm,a-1), rhom, 0.0_wp, seqm(sm), ngppm, gpm(:,sm,a), transitionpm(:))
            
            forall (kd_1=0:dokids, lm_1=1:2)
                trCC(pmi_1,lm_1,kd_1,a)%m(sm,2,:) = transitionpm(:) * ( 1.0_wp-trCS(sm,kd_1,a) ) * trlm(lm_1,sm)
                trCC(pmi_1,lm_1,kd_1,a)%m(sm,1,:) = transitionpm(:) * ( 1.0_wp-trCS(sm,kd_1,a) ) * (1.0_wp-trlm(lm_1,sm))
            end forall

        end do
        end do
        end do            


        ! Transition matrix for child age trkd(akd,akd_1,a,s,m_1)
        if (dokids==1) then

            trkd = 0.0_wp

            trkd(0,-1,:,:,:)  = prkd(:,:,:,-1)                      !childbirth for women with no older children
            trkd(-1,-1,:,:,:) = 1.0_wp-prkd(:,:,:,-1)

            trkd(0,maxmaxakd,:,:,:)  = prkd(:,:,:,maxmaxakd)        !transition when youngest kid is 18 at a-1
            trkd(-1,maxmaxakd,:,:,:) = 1.0_wp-prkd(:,:,:,maxmaxakd)

            do s=1,nts                                              !transition when youngest kid is 0-17
            do a=minabirth(s)+1,maxabirth+maxmaxakd
                forall (akd_1=minakd(a-1,s):maxakd(a,s)-1, m_1=0:domales) trkd(0,akd_1,a,s,m_1) = prkd(a,s,m_1,akd_1)
                forall (akd_1=minakd(a-1,s):maxakd(a,s)-1, m_1=0:domales) trkd(akd_1+1,akd_1,a,s,m_1) = 1.0_wp-prkd(a,s,m_1,akd_1)
            end do
            end do

        else
        
            trkd(-1,:,:,:,:) = 1.0_wp
        
        end if

    end subroutine popfamtransprobs
    
    

    ! -------------------------------------------------------------------------------------------------------
    ! Load starting parameters from text file

    subroutine paramLoadStarting
        implicit none
        character(*), parameter         :: paramFile = parampath//'initParam.txt'
        integer                         :: unit, iParam, ios
        character(160)                  :: buffer

        ! Get unit and open parameter file
        unit = 903
        open (unit, file=paramFile, action='read')
        
        ! Read parameters
        iParam = 0
        do

            read(unit, '(A160)', iostat=ios) buffer
            if (ios < 0) exit

            if (.not.((buffer == "") .or. (buffer(1:1) == "!"))) then
                iParam = iParam + 1
                read(buffer, '(A12, T13, F18.14, T34, F5.2, T41, F5.2, T48, A50)') &
                    param%name(iParam), param%val(iParam), param%lbval(iParam), param%ubval(iParam), param%text(iParam)
            end if

        end do
        param%n = iParam

        ! Close file
        close (unit)

    end subroutine paramLoadStarting



    ! -------------------------------------------------------------------------------------------------------
    ! Save all parameters to text file suitable for reading in next time

    subroutine paramSaveAll(param,paramFile,headerText)
        implicit none
        type(param_t),  intent(in)  :: param
        character(*), intent(in)    :: paramFile
        character(*), intent(in)    :: headerText
        integer                     :: paramUnit, iParam


        ! Get unit, open it and write header
        paramUnit = 333
        open (paramUnit, file=paramFile, action='write')
        write(paramUnit, '("! ",A)') headerText


        ! Write parameters
        write(paramUnit, '(/,A)') '! Wage rates, by (s,f1,f2)'
        do iParam=1, 9
            write(paramUnit, 316) param%name(iParam), param%val(iParam), param%lbval(iParam), param%ubval(iParam), param%text(iParam)
        end do

        write(paramUnit, '(/,A)') '! Returns to experience, by (s,f1,f2)'
        do iParam=10, 18
            write(paramUnit, 316) param%name(iParam), param%val(iParam), param%lbval(iParam), param%ubval(iParam), param%text(iParam)
        end do

        write(paramUnit, '(/,A)') '! Persistence in productivity shock'
        do iParam=19, 21
            write(paramUnit, 316) param%name(iParam), param%val(iParam), param%lbval(iParam), param%ubval(iParam), param%text(iParam)
        end do

        write(paramUnit, '(/,A)') '! SE innovation in productivity'
        do iParam=22, 24
            write(paramUnit, 316) param%name(iParam), param%val(iParam), param%lbval(iParam), param%ubval(iParam), param%text(iParam)
        end do

        write(paramUnit, '(/,A)') '! Mean initial prod for theta1: Epas'
        do iParam=25, 27
            write(paramUnit, 316) param%name(iParam), param%val(iParam), param%lbval(iParam), param%ubval(iParam), param%text(iParam)
        end do

        write(paramUnit, '(/,A)') '! Dispersion in initial wage rates'
        do iParam=28, 30
            write(paramUnit, 316) param%name(iParam), param%val(iParam), param%lbval(iParam), param%ubval(iParam), param%text(iParam)
        end do

        write(paramUnit, '(/,A)') '! Preferences for work'
        do iParam=31, 48
            write(paramUnit, 316) param%name(iParam), param%val(iParam), param%lbval(iParam), param%ubval(iParam), param%text(iParam)
        end do

        write(paramUnit, '(/,A)') '! Preferences for PT work'
        do iParam=49, 66
            write(paramUnit, 316) param%name(iParam), param%val(iParam), param%lbval(iParam), param%ubval(iParam), param%text(iParam)
        end do

        write(paramUnit, '(/,A)') '! Unobserved heterogeneity (l x theta)'
        do iParam=67, 69
            write(paramUnit, 316) param%name(iParam), param%val(iParam), param%lbval(iParam), param%ubval(iParam), param%text(iParam)
        end do

        write(paramUnit, '(/,A)') '! Accumulation of HC if PTE: delta0'
        do iParam=70, 72
            write(paramUnit, 316) param%name(iParam), param%val(iParam), param%lbval(iParam), param%ubval(iParam), param%text(iParam)
        end do

        write(paramUnit, '(/,A)') '! Depreciation rate (all lms): delta1'
        do iParam=73, 75
            write(paramUnit, 316) param%name(iParam), param%val(iParam), param%lbval(iParam), param%ubval(iParam), param%text(iParam)
        end do

        write(paramUnit, '(/,A)') '! P(positive CCC if working): pposCC'
        iParam=76
        write(paramUnit, 316) param%name(iParam), param%val(iParam), param%lbval(iParam), param%ubval(iParam), param%text(iParam)

        write(paramUnit, '(/,A)') '! Preferences for educ: vS and seS'
        do iParam=77, 86
            write(paramUnit, 316) param%name(iParam), param%val(iParam), param%lbval(iParam), param%ubval(iParam), param%text(iParam)
        end do

        write(paramUnit, '(/,A)') '! Measurement error: seME'
        do iParam=  87, 89
            write(paramUnit, 316) param%name(iParam), param%val(iParam), param%lbval(iParam), param%ubval(iParam), param%text(iParam)
        end do

        ! Close file
        close (paramUnit)
        
        316 format (A12, T13, F18.14, T34, F5.2, T41, F5.2, T48, A50)
        
    end subroutine paramSaveAll


    ! -------------------------------------------------------------------------------------------------------
    ! Copy parameters into relevant arrays (and populate remainder of those arrays)

    subroutine paramCopy(param,prisk)
        type(param_t),  intent(in)  :: param
        real(wp), intent(in)        :: prisk
        integer :: f1i, f2i, s

        ! Wage rates (mktw)
        i=1
        do s=1,nts
            mktw(s,1,1) = param%val(i)
            mktw(s,2,1) = param%val(i)+param%val(i+nts)
            mktw(s,1,2) = param%val(i)+param%val(i+nts+nts)
            mktw(s,2,2) = param%val(i)+param%val(i+nts)+param%val(i+nts+nts)
            i=i+1
        end do

        ! Returns to experience (alpha)
        i = 10
        do s=1,nts
            alpha(s,1,1) = param%val(i)
            alpha(s,2,1) = param%val(i)+param%val(i+nts)
            alpha(s,1,2) = param%val(i)+param%val(i+nts+nts)
            alpha(s,2,2) = param%val(i)+param%val(i+nts)+param%val(i+nts+nts)
            i=i+1
        end do
        
        ! persistence in productivity shock (rho)
        i=19; rho(1) = param%val(i)
        i=20; rho(2) = param%val(i)
        i=21; rho(3) = param%val(i)

        ! SE innovation in productivity (se q)
        i=22;  seq(1) = param%val(i)*sqrt(prisk);  Eq(1) = 0.0_wp + 0.5_wp*(1.0_wp-prisk)*(param%val(i)**2)
        i=23;  seq(2) = param%val(i)*sqrt(prisk);  Eq(2) = 0.0_wp + 0.5_wp*(1.0_wp-prisk)*(param%val(i)**2)
        i=24;  seq(3) = param%val(i)*sqrt(prisk);  Eq(3) = 0.0_wp + 0.5_wp*(1.0_wp-prisk)*(param%val(i)**2)

        ! Mean of initial productivity shock by type (Epas(s,theta))
        Epas  = 0.0_wp
        i=25; Epas(1,1) = param%val(i)
        i=26; Epas(2,1) = param%val(i)
        i=27; Epas(3,1) = param%val(i)

        ! dispersion of initial wage rates (sepas)
        i=28; sepas(1) = param%val(i)
        i=29; sepas(2) = param%val(i)
        i=30; sepas(3) = param%val(i)

        ! Preference for work (U)
        i=31
        do s=1,nts
            U(s)  = param%val(i)
            Uk(s) = param%val(i+nts)
            i = i+1
        end do
        i=37; Uk02   = param%val(i)
        i=38; Uk35   = param%val(i)
        i=39; Uk610  = param%val(i)
        i=40
        do s=1,nts
            Um(s)  = param%val(i)
            Umk(s) = param%val(i+nts)
            i = i+1
        end do
        i=46; Umwk   = param%val(i)
        i=47; Ub1    = param%val(i)
        i=48; Ub2    = param%val(i)
        
        ! Preference for PT work (UPT)
        i = 49
        do s=1,nts
            UPT(s)  = param%val(i)
            UkPT(s) = param%val(i+nts)
            i = i+1
        end do
        i=55; Uk02PT   = param%val(i)
        i=56; Uk35PT   = param%val(i)
        i=57; Uk610PT  = param%val(i)
        i = 58
        do s=1,nts
            UmPT(s)  = param%val(i)
            UmkPT(s) = param%val(i+nts)
            i = i+1
        end do
        i=64; UmwkPT   = param%val(i)
        i=65; Ub1PT    = param%val(i)
        i=66; Ub2PT    = param%val(i)

        ! Unobserved heterogeneity (theta by l)
        theta = 0.0_wp
        i=67; theta(2,1) = param%val(i)
        i=68; theta(3,1) = param%val(i)
        i=69; ptheta1    = param%val(i)
        theta(2,2) = -ptheta1 * theta(2,1) / (1.0_wp-ptheta1)
        theta(3,2) = -ptheta1 * theta(3,1) / (1.0_wp-ptheta1)

        ! Accumulation of HC (delta0(l,s))
        delta0      = 0.0_wp
        delta1      = 0.0_wp        
        delta0(3,:) = 1.0_wp
        i=70; delta0(2,1) = param%val(i)
        i=71; delta0(2,2) = param%val(i)
        i=72; delta0(2,3) = param%val(i)

        ! Depreciation rate (delta1(l,s))
        i=73; delta1(1) = param%val(i)
        i=74; delta1(2) = param%val(i)
        i=75; delta1(3) = param%val(i)

        ! P of positive CCC if working (pposCC)
        i=76; pposCC = param%val(i)

        ! preferences for education (vS and seS)
        vS(1,:) = 0.0_wp
        do j=1,6
            i=76+j; vS(2,j) = param%val(i)
            i=80+j; vS(3,j) = param%val(i)
        end do
        seS(1) = 0.0_wp
        i = 85; seS(2) = param%val(i)
        i = 86; seS(3) = param%val(i)
        
        ! se measurement error in log wages (seME)
        i=87; seME(1) = param%val(i)
        i=88; seME(2) = param%val(i)
        i=89; seME(3) = param%val(i)

    end subroutine paramCopy



    ! -------------------------------------------------------------------------------------------------------
    ! Calculate power-of-10 scaling factor so that val is in a range determined by offset

    subroutine scaleToRange(val,scale,tval)
        implicit none
        real(wp), intent(in)  :: val
        real(wp), intent(out) :: scale,tval

        scale = floor( log10( max( abs(val), 0.000001_wp ) ) )
        tval  = val / (10.d0**scale)

    end subroutine scaleToRange


end module parametersfcs

