! -----------------------------------------------------------------------------------------------------------
! -----------------------------------------------------------------------------------------------------------
! This file contains the routines to simulate the life-cycle profiles of women observed in BHPS
! -----------------------------------------------------------------------------------------------------------
! -----------------------------------------------------------------------------------------------------------


module simMod

use precision, only : wp, infty, tol
use lEmatrices
use state
use constants
use parameters
use gridvars
use SimConditions
use fortax_data
use slnMod,        only : solve
use generalfns

integer,  parameter                 :: totn  = nind*(maxa-mina+1)
real(wp), parameter, dimension(nts) :: prent = (/ 0.318_wp, 0.108_wp, 0.027_wp /)     ! % living in rented accommodation: BHPS data, 2006, aged 30-50

! random shocks 
real(wp), private, dimension(totn,nrsim) :: rvp,rvsm,rvlm,rvpm,rvnkd                  ! shocks: female prod, male educ, ls and prod, presence of kd
real(wp), private                        :: rvdrent(nind,nrsim), rvuS(nts,nind,nrsim) ! shocks: whether in rented accommodation, pref for education

! pointers to simulated variables
real(wp),pointer, private :: pk(:), pe(:), pw(:), pwm(:), pc(:), py(:), pym(:), pinc(:), ptax(:), pcc(:), pp(:), ppm(:), pV(:)
integer, pointer, private :: pdrent, ps, pnDl(:), pdm(:), psm(:), plmi(:), pnkd(:), pakd(:), pli(:) !, pDli(:)  

! pre-determined variables in simulations: use vectors instead of pointers
real(wp), private :: irvuS(nts), irvdrent, irvp(maxa-mina+1), irvsm(maxa-mina+1), irvlm(maxa-mina+1), irvpm(maxa-mina+1), irvnkd(maxa-mina+1)
integer,  private :: ith, idCC
integer,  private :: idbk1, idbk2
real(wp), private :: ibk1, ibk2, iparentYres

! MC simulations to integrate value at start of life
integer, parameter,private :: MCrep = 10000
real(wp),private           :: MCuS(nts,MCrep)


contains


    !------------------------------------------------------------------------------------------------
    ! simulation wrappers: simulate LC decisions of woman i


    ! simulate choices of woman i from age0 to age1, under a single tax-benefits system
    ! ---------------------------------------------------------------------------------
    subroutine sim_a0toa1 (i, rep, age0, age1, chooseS, dstartlife)
        implicit none
        integer, intent(in) :: i, rep, age0, age1, chooseS, dstartlife
        integer a, ai
        
        ! initiate lifecycle if age0=mina
        if (dstartlife==1 .and. age0==mina) call startlife(i, rep, chooseS)

        ! simulate choices and state space from age0 to age1        
        do a = max(as(ps),age0), age1
            ai  = a-mina+1                          ! pointers index starts always at 1
            call getopt(a,ai)                       ! obtain optimum (l,k1) and store (w,y,inc,c,e1) as well
            if (a<maxa) call state_a1(a,ai)         ! fill state space for tomorrow
        end do
        
    end subroutine sim_a0toa1 
    

    ! simulate woman i during entire life under a single system conditional on education
    ! ----------------------------------------------------------------------------------
    subroutine sim_bys (i, rep)
        implicit none
        integer, intent(in) :: i, rep
        integer s, a, ai
        
        do s=1,nts
            sim(i,rep)%s=s
            call startlife(i, rep, 0)                                   ! initiate lifecycle: no education choice but get other starting values
            do a = max(as(ps),mina), maxa
                ai  = a-mina+1                                          ! pointers index starts always at 1
                call getopt(a,ai)                                       ! obtain optimum (l,k1) and store (w,y,inc,c,e1) as well
                if (a<maxa) call state_a1(a,ai)                         ! fill state space for tomorrow
            end do
            sim(i,rep)%aa(:)%Oh%incbyS(s) = sim(i,rep)%aa(:)%Oh%inc     ! store income by education level
        end do
        
    end subroutine sim_bys
        
        
   
    !------------------------------------------------------------------------------------------------
    ! Simulation of WL decisions, from age a1, for all women and stores endogenous variables
    ! To calculate elasticities and income effects

    ! For Marshallian elasticities and income effects
    subroutine sim_forElast(a1)
        implicit none
        integer, intent(in) :: a1
        integer             :: a, ai, dCC, rep, i
        
        do rep=1,nrsim
        do i=1,nind        
            call ilocate(i,rep)                
            do a=a1,maxa
                ai = a-mina+1                          !pointers index starts always at 1
                dCC = merge(idCC,1,a<maxabirth+aCcHi)  ! if a>=maxabirth+aCcHi(=10), read continuation values from part of matrix where dCC=1
                call getopt(a,ai)                      !obtain optimum (l,k1) and store (w,y,inc,c,e1) as well
            end do
        end do
        end do

    end subroutine sim_forElast


    ! For Frisch elasticities
    ! Compensation (lump-sum transfer at start of WL) calculated to keep the expected 
    !    marginal utility of consumption constant 
    subroutine sim_forFrisch
        implicit none
        integer  :: akd_1, dCC, th, s, ix, f1i, f2i, rep, i, a1, ai, a, ubk, lbk
        real(wp) :: lEdu_as(nts,ntth,1-doCC:1,ntb1,ntb2,ngpk)


        ! store matrix of expected marginal utility of consumption at entrance in WL
        akd_1 = -1
        do dCC = 1-doCC,1
        do s   = 1,nts
            ix = iEmtx%ix(akd_1, as(s), dCC, s)
            forall (th=1:ntth, f1i=1:ntb1, f2i=1:ntb2) lEdu_as(s,th,dCC,f1i,f2i,:) = lEdu0(ix,th,f1i,f2i)%m0(:,1,1)
        end do        
        end do        

        
        ! calculate compensation and simulate individual decisions under new conditions
        do rep=1,nrsim
        do i=1,nind
        
            call ilocate(i,rep)          ! identify individual observations in sim matrices
            a1 = as(ps)                  ! age for compensation and start simulation
            
            ! compute new initial k that reaches baseline E marginal utility at entrance in WL 
            ubk = 2                                    
            do
                if (gk(a1,ubk,ps) > pk(a1-mina+1) .or. ubk==ngpk) exit
                ubk = ubk+1
            end do
            lbk           = ubk-1
            pk(a1-mina+1) = LinInterp1D( sim(i,rep)%lEdu_as, lEdu_as(ps,ith,idCC,idbk1,idbk2,lbk), lEdu_as(ps,ith,idCC,idbk1,idbk2,ubk), &
                                         gk(a1,lbk,ps), gk(a1,ubk,ps) ) 

            ! simulate LC decisions in new scenario with new initial k
            do a=a1,maxa
                ai = a-mina+1                          ! pointers index starts always at 1
                dCC = merge(idCC,1,a<maxabirth+aCcHi)  ! if a>=maxabirth+aCcHi(=10), read continuation values from part of matrix where dCC=1
                call getopt(a,ai)                      ! obtain optimum (l,k1) and store (w,y,inc,c,e1) as well
            end do

        end do
        end do

    end subroutine sim_forFrisch



    !------------------------------------------------------------------------------------------------
    ! Prepare to start WL of individual i
    !    simulate optimal education choice and complete simulation structure where possible
    ! This routine is only called under the system the individual faces at age 19
    
    subroutine startlife(i, rep, chooseS)
        use slnMod,     only : solveEE
        use incomefns,  only : wm
        implicit none
        integer, intent(in)  :: i, rep, chooseS
        integer  :: a1, ai, a1i, s, sOpt(1), j, ix, akd_1, lbk, ubk
        real(wp) :: gamma0 , gamma1, EVs16(nts), V16(nts), EVs19(nts), eulerArr(ngpk)


        gamma0 = (beta*R)**(1.0_wp/(mu0-1.0_wp))
        gamma1 = 1.0_wp + R*gamma0 + ((R*gamma0)**2)
        fee    = Ufee

        ! Education choice at age mina
        call getS(s, EVs19, EVs16, V16)       
        if (chooseS==1) sim(i,rep)%s=s
        sim(i,rep)%EVpreS  = sum( max(EVs16(1), EVs16(2)+MCuS(2,:), EVs16(3)+MCuS(3,:)) ) / real(MCrep,wp)
        sim(i,rep)%EVpostS = EVs19
        sim(i,rep)%V16     = V16
        sim(i,rep)%EV16    = EVs16
        
        ! work out state at entrance in working life
        ai    = 1             ! pointers index starts at 1 for mina
        a1    = as(ps)        ! age start WL
        a1i   = a1-mina+1     ! pointers index at a1 (mina=1)
        akd_1 = -1            ! no childbirth before entering WL
        
        ! solve for initial assets (k(a1)) if s=3
        if (ps==3) then
            ix = iEmtx%ix(akd_1, a1, idCC, ps)
            eulerArr = EEpar(ngpl,ith)%b(plmi(ai),pdm(ai),pakd(ai),ps,idbk1,idbk2) * lEDU0(ix,ith,idbk1,idbk2)%m0(:,1,1) - & 
                       ((R**(a1-mina))*(pk(ai)-fee) - gk(a1,:,ps))/gamma1
            call solveEE(a1, ps, eulerArr, lbk, ubk, pk(a1i))
            pc(ai+2) = ((R**(a1-mina))*(pk(ai)-fee)-pk(a1i))/gamma1
            pc(ai+1) = gamma0*pc(ai+2)
            pc(ai)   = (gamma0**2)*pc(ai+2)
        end if

        ! productivity shock at a1
        pp(a1i) = irvp(a1i)*sep(ps,a1) + Epas(ps,ith)

        ! determine family composition at a1
        pnkd(a1i) = 1
        pdm(a1i)  = 1
        if (irvnkd(a1i)<initfam(ps,0,0)) then
            pnkd(a1i) = 0
            pdm(a1i)  = 0
        else if (irvnkd(a1i)<initfam(ps,0,0)+initfam(ps,1,0)) then
            pnkd(a1i) = 0
        else if (irvnkd(a1i)<initfam(ps,0,0)+initfam(ps,1,0)+initfam(ps,0,1)) then
            pdm(a1i)  = 0
        end if
        pakd(a1i) = pnkd(a1i)-1

        ! fill male characteristics at a1
        psm(a1i)  = merge(2, 1, irvsm(a1i) < sum(pr0m(2:ntsm,a1,ps))/sum(pr0m(:,a1,ps)) .and. pdm(a1i)==1)
        psm(a1i)  = merge(3, psm(a1i), irvsm(a1i) < pr0m(3,a1,ps)/sum(pr0m(:,a1,ps)) .and. pdm(a1i)==1)
        plmi(a1i) = 1 + pdm(a1i)*merge(1, 0, irvlm(a1i)<prlmnew(psm(a1i)))                          !labour supply
        ppm(a1i)  = real(pdm(a1i),wp) * irvpm(a1i) * sepm(psm(a1i),a1)                              !productivity
        pwm(a1i)  = merge(1.0_wp, 0.0_wp, pdm(a1i)==1 .and. plmi(a1i)>1) * wm(psm(a1i),a1,ppm(a1i)) !hourly wage rate
        pym(a1i)  = glm(plmi(a1i)) * pwm(a1i)                                                       !gross weekly earnings
        
        ! determined whether is a renter
        pdrent = 0
        if (irvdrent<prent(ps)) pdrent = 1

    end subroutine startlife


    ! optimal education choice at age mina
    ! ------------------------------------
    subroutine getS(s, EVs19, EVs16, V16)
        use preffns, only : valueS, calcEVinvinv
        implicit none
        integer,  intent(out) :: s
        real(wp), intent(out) :: EVs19(nts), EVs16(nts), V16(nts)
        integer               :: akd_1, ix, sOpt(1)

        akd_1 = -1                                                            ! no childbirth before entering WL        
        do s=1, nts
            ix       = iEmtx%ix(akd_1, mina, idCC, s)                         ! recover index in EV/EDU - function of (akd_1,a,dCC, s)
            EVs19(s) = calcEVinvinv(lEV0(ix,ith,idbk1,idbk2)%m0(1,1,1))       ! expected value of education s after preferences but before productivity are realised
            EVs16(s) = valueS (s, EVs19(s), ibk1, ibk2, iparentYres, 0.0_wp)  ! expected net (of observed utility costs) value of education
            V16(s)   = EVs16(s)+irvuS(s)                                      ! expected net (of all utility costs) value of education
        end do
        
        sOpt = maxloc(V16)                                                    ! optimal education choice
        s    = sOpt(1)
    
    end subroutine getS



    !------------------------------------------------------------------------------------------------
    ! Prepare to simulate age a+1, setting the state space
    ! note: e1 and k1 determined already in getopt routine

    subroutine state_a1(a,ai)
        use incomefns, only : wm
        implicit none
        integer, intent(in)    :: a,ai
        integer  a1, a1i, newkd


        !locate age and index for pointers
        a1  = a+1          !age next period
        a1i = a1-mina+1    !pointers index (mina=1)
        
        ! keep track of working periods and productivity   
        pnDL(a1i) = pnDL(ai) + merge(1,0,pli(ai)>1)            ! # periods worked so far
        pp(a1i) = rho(ps)*pp(ai) + irvp(a1i)*seq(ps) + Eq(ps)  ! female productivity

        !child arrival/departure and age
        if (pnkd(ai)==1) then
            newkd     = merge(1, 0, irvnkd(a1i)<prkd(a1,ps,pdm(ai),pakd(ai)))
            pnkd(a1i) = merge(1, 0, newkd==1 .or. pakd(ai)<maxmaxakd)
            pakd(a1i) = -1
            if (pnkd(a1i)==1) pakd(a1i) = merge(0, pakd(ai)+1, newkd==1)
        else
            pnkd(a1i) = merge(1, 0, irvnkd(a1i)<prkd(a1,ps,pdm(ai),-1))
            pakd(a1i) = pnkd(a1i)-1
        end if

        !spouse's arrival/departure and characteristics
        if (pdm(ai)==1) then
            pdm(a1i)  = merge(1, 0, irvsm(a1i) > trCS(psm(ai),pnkd(ai),a1))             !continue married
            psm(a1i)  = merge(psm(ai), 1, pdm(a1i)==1)                                  !spouse's education
            plmi(a1i) = 1 + pdm(a1i)*merge(1, 0, irvlm(a1i)<trlm(plmi(ai),psm(ai)))     !spouse's labour supply
            ppm(a1i)  = real(pdm(a1i),wp) * (rhom*ppm(ai) + irvpm(a1i)*seqm(psm(ai)))   !spouse's productivity
        else
            pdm(a1i)  = merge(1, 0, irvsm(a1i) < sum(pr0m(:,a1,ps)))                    !new marriage
            psm(a1i)  = merge(2, 1, irvsm(a1i) < sum(pr0m(2:ntsm,a1,ps)))               !spouse's education
            psm(a1i)  = merge(3, psm(a1i), irvsm(a1i) < pr0m(3,a1,ps))                  !
            plmi(a1i) = 1 + pdm(a1i)*merge(1, 0, irvlm(a1i)<prlmnew(psm(a1i)))          !spouse's labour supply
            ppm(a1i)  = real(pdm(a1i),wp) * irvpm(a1i) * sepm(psm(a1i),a1)              !spouse's productivity
        end if
        pwm(a1i) = merge(1.0_wp, 0.0_wp, pdm(a1i)==1 .and. plmi(a1i)>1) * wm(psm(a1i),a1,ppm(a1i)) !spouse's gross hourly wage rate 
        pym(a1i) = glm(plmi(a1i)) * pwm(a1i)                                                       !spouse's gross weekly earnings
        
    end subroutine state_a1



    !------------------------------------------------------------------------------------------------
    ! Simulate optimal choices age a given state space: labour supply and savings
    
    ! Simulate optimal choice at age a and store solution
    ! ---------------------------------------------------
    subroutine getopt(a,ai)
        use incomefns, only : w
        implicit none
        integer,intent(in)  :: a, ai
        real(wp),dimension(ngpl) :: v, inc, c, k1, e1val
        integer act(1)


        call SimValue (a, ai, v, inc, c, k1, e1val)     ! simulate value of the labour supply choice
        act      = maxloc(v)                            ! optimal labour supply
        pli(ai)  = act(1)
        pw(ai)   = w(ps,pe(ai),pp(ai),idbk1,idbk2)      ! wage rate
        py(ai)   = gl(act(1)) * pw(ai)                  ! gross weekly earnings
        pinc(ai) = inc(act(1))                          ! weekly disposable income (family)
        ptax(ai) = py(ai) + pym(ai) - pinc(ai)          ! family tax liability (net of benefits)
        pc(ai)   = c(act(1))                            ! optimal consumption
        if (a<maxa) then
            pk(ai+1) = k1(act(1))                       ! next period assets (k)
            pe(ai+1) = e1val(act(1))                    ! next period experience (e)
        end if
        pV(ai)  = v(act(1))                             ! value of optimal choice

        pcc(ai) = real(idCC,wp)*ccCost(pli(ai),pdm(ai),plmi(ai),pakd(ai))    ! childcare cost

    end subroutine getopt


    ! main simulation routine
    ! calculate value of each labour supply at the optimal consumption choice
    ! -----------------------------------------------------------------------
    subroutine SimValue (a, ai, v, inc, c, k1, e1val)
        use incomefns,  only : netincome, Ttax
        use motionfns,  only : nxte
        use preffns,    only : util, calcEVinvinv
        use slnMod,     only : solveEE
        implicit none
        integer,intent(in)                   :: a, ai
        real(wp),intent(out),dimension(ngpl) :: v, inc, c, k1, e1val
        real(wp),pointer                     :: plEV0(:,:,:), plEDU0(:,:,:), plEV1(:,:,:,:), plEDU1(:,:,:,:)
        real(wp)                             :: lbpval, ubpval, mininc, lbe1val, ube1val, lbpmval, ubpmval, eulerArr(ngpk), lEVlbk, lEVubk, p, pm
        integer                              :: lbp, ubp, l, lbe1, ube1, lbk, ubk, lbpm, ubpm, ix


        ! get productivity and bounds in grids
        p  = pp(ai)
        pm = ppm(ai)
        call bounds(p, gp(:,a,ps,ith), ngpp, lbp, ubp)
        lbpval = gp(lbp,a,ps,ith)
        ubpval = gp(ubp,a,ps,ith)
        
        ! calculate minimum income that makes minimum consumption affordable
        mininc = mink(a+1,ps)+minc-R*pk(ai)
        
        ! initialise value to -infty
        v  = -infty

        ! locate relevant part of value function next period
        if (a<maxa) then
	        ix     =  iEmtx%ix(pakd(ai), a+1, idCC, ps)
            if (pdm(ai)==0) then
                ! dimensions of pointers are (k(a+1),e(a+1),p(a))
                plEV0  => lEV0(ix,ith,idbk1,idbk2)%m0(:,1:ngpe(a+1,ps),:)
                plEDU0 => lEDU0(ix,ith,idbk1,idbk2)%m0(:,1:ngpe(a+1,ps),:)
            else
                ! dimensions of pointers are (k(a+1),e(a+1),p(a),pm(a))
                plEV1  => lEV1(ix,ith,idbk1,idbk2)%m1(:,1:ngpe(a+1,ps),:,psm(ai),plmi(ai),:)   
                plEDU1 => lEDU1(ix,ith,idbk1,idbk2)%m1(:,1:ngpe(a+1,ps),:,psm(ai),plmi(ai),:)
                ! bounds in pm (spouse's productivity)
                call bounds(pm, gpm(:,psm(ai),a), ngppm, lbpm, ubpm)
                lbpmval  = gpm(lbpm,psm(ai),a)
                ubpmval  = gpm(ubpm,psm(ai),a)
            end if
        end if


        ! compute value of each labour supply option
        do l=1,ngpl

            inc(l)   = netincome(l, ps, pe(ai), p, pdm(ai), plmi(ai), psm(ai), a, pm, pakd(ai), idCC, ith, idbk1, idbk2)
            e1val(l) = nxte(pe(ai), l, ps)

            if (inc(l)>=mininc-tol) then

                if (a==maxa) then

                    eulerArr  = EEpar(l,ith)%b(plmi(ai),pdm(ai),pakd(ai),ps,idbk1,idbk2) * lEDUr(ps,pakd(ai),pdm(ai),psm(ai),:) - &
                                (R*pk(ai) + inc(l) - gk(maxa+1,:,ps))
                    call solveEE(maxa+1, ps, eulerArr, lbk, ubk, k1(l))
                    c(l) = R*pk(ai)+inc(l)-k1(l)
                    v(l) = util(c(l), l, pdm(ai), plmi(ai), pnkd(ai), pakd(ai), ith, ps, idbk1, idbk2) + &
                            beta*calcEVinvinv(LinInterp1d (k1(l), gk(maxa+1,lbk,ps), gk(maxa+1,ubk,ps), &
                                                        lEVr(ps,pakd(ai),pdm(ai),psm(ai),lbk), lEVr(ps,pakd(ai),pdm(ai),psm(ai),ubk)))
            
                else

                    ! bounds on next e
                    call bounds(e1val(l), ge(a+1,1:ngpe(a+1,ps),ps), ngpe(a+1,ps), lbe1, ube1)
                    lbe1val = ge(a+1, lbe1, ps)
                    ube1val = ge(a+1, ube1, ps)

                    ! optimal consumption and V conditional on labour supply: singles
                    if (pdm(ai)==0) then

                        eulerArr = LinInterp2D(e1val(l), lbe1val, ube1val, p, lbpval, ubpval, &
                                               plEDU0(:,lbe1,lbp), plEDU0(:,lbe1,ubp), plEDU0(:,ube1,lbp), plEDU0(:,ube1,ubp))
                        eulerArr = EEpar(l,ith)%b(plmi(ai),pdm(ai),pakd(ai),ps,idbk1,idbk2)*eulerArr - (R * pk(ai) + inc(l) - gk(a+1,:,ps))
                        call solveEE(a+1, ps, eulerArr, lbk, ubk, k1(l))
                        c(l) = R*pk(ai)+inc(l)-k1(l)
                        v(l) = util(c(l), l, pdm(ai), plmi(ai), pnkd(ai), pakd(ai), ith, ps, idbk1, idbk2) + &
                                beta*calcEVinvinv( LinInterp3d (k1(l), gk(a+1,lbk,ps), gk(a+1,ubk,ps), e1val(l), lbe1val, ube1val, p, lbpval, ubpval, &  
                                                plEV0(lbk,lbe1,lbp), plEV0(ubk,lbe1,lbp), plEV0(lbk,ube1,lbp), plEV0(ubk,ube1,lbp), &
                                                plEV0(lbk,lbe1,ubp), plEV0(ubk,lbe1,ubp), plEV0(lbk,ube1,ubp), plEV0(ubk,ube1,ubp)) )

                    ! optimal consumption and V conditional on labour supply: couples
                    else

                        eulerArr = LinInterp3D(e1val(l), lbe1val, ube1val, p, lbpval, ubpval, pm, lbpmval, ubpmval, &
                                      plEDU1(:,lbe1,lbp,lbpm), plEDU1(:,ube1,lbp,lbpm), plEDU1(:,lbe1,ubp,lbpm), plEDU1(:,ube1,ubp,lbpm), &
                                      plEDU1(:,lbe1,lbp,ubpm), plEDU1(:,ube1,lbp,ubpm), plEDU1(:,lbe1,ubp,ubpm), plEDU1(:,ube1,ubp,ubpm))
                        eulerArr = EEpar(l,ith)%b(plmi(ai),pdm(ai),pakd(ai),ps,idbk1,idbk2)*eulerArr - (R * pk(ai) + inc(l) - gk(a+1,:,ps))
                        call solveEE(a+1, ps, eulerArr, lbk, ubk, k1(l))
                        lEVlbk = LinInterp3d (e1val(l), lbe1val, ube1val, p, lbpval, ubpval, pm, lbpmval, ubpmval, &  
                                      plEV1(lbk,lbe1,lbp,lbpm), plEV1(lbk,ube1,lbp,lbpm), plEV1(lbk,lbe1,ubp,lbpm), plEV1(lbk,ube1,ubp,lbpm), &
                                      plEV1(lbk,lbe1,lbp,ubpm), plEV1(lbk,ube1,lbp,ubpm), plEV1(lbk,lbe1,ubp,ubpm), plEV1(lbk,ube1,ubp,ubpm))
                        lEVubk = LinInterp3d (e1val(l), lbe1val, ube1val, p, lbpval, ubpval, pm, lbpmval, ubpmval, &  
                                      plEV1(ubk,lbe1,lbp,lbpm), plEV1(ubk,ube1,lbp,lbpm), plEV1(ubk,lbe1,ubp,lbpm), plEV1(ubk,ube1,ubp,lbpm), &
                                      plEV1(ubk,lbe1,lbp,ubpm), plEV1(ubk,ube1,lbp,ubpm), plEV1(ubk,lbe1,ubp,ubpm), plEV1(ubk,ube1,ubp,ubpm))
                        c(l) = R*pk(ai)+inc(l)-k1(l)
                        v(l) = util(c(l), l, pdm(ai), plmi(ai), pnkd(ai), pakd(ai), ith, ps, idbk1, idbk2) + &
                                beta*calcEVinvinv( LinInterp1d (k1(l), gk(a+1,lbk,ps), gk(a+1,ubk,ps), lEVlbk, lEVubk) )

                   end if
                end if
            end if
            
        end do

    end subroutine SimValue


    !------------------------------------------------------------------------------------------------
    ! compute bounds in grid for a variable x

    subroutine bounds(x,gx,ngpx,lb,ub)
        implicit none
        integer,intent(in)  :: ngpx
        real(wp),intent(in) :: x,gx(ngpx)
        integer,intent(out) :: lb,ub
        integer i

        do i=1,ngpx
            if (abs(x-gx(i))<tol) then
                lb = i
                ub = i
                exit
            else if (x<gx(i) .or. i==ngpx) then
                lb = max(i-1,1)
                ub = lb+1
                exit
            end if
        end do
        
        return    
    end subroutine bounds



    !------------------------------------------------------------------------------------------------
    ! direct pointers to relevant parts of state and choice matrices to read and write
    
    subroutine ilocate(i,rep)
        implicit none
        integer,intent(in) :: i, rep
        integer lb,ub
        
        !get index limits
        lb    = (i-1)*(maxa-mina+1)+1
        ub    = i*(maxa-mina+1)
        
        !select relevant parts of Oh (state space)
        ps     => sim(i,rep)%s
        pk     => sim(i,rep)%aa(:)%Oh%k
        pe     => sim(i,rep)%aa(:)%Oh%e
        pp     => sim(i,rep)%aa(:)%Oh%p
        pnDl   => sim(i,rep)%aa(:)%Oh%nDl
        pdm    => sim(i,rep)%aa(:)%Oh%m
        psm    => sim(i,rep)%aa(:)%Oh%sm
        plmi   => sim(i,rep)%aa(:)%Oh%lm
        ppm    => sim(i,rep)%aa(:)%Oh%pm
        pwm    => sim(i,rep)%aa(:)%Oh%wm
        pym    => sim(i,rep)%aa(:)%Oh%ym
        pnkd   => sim(i,rep)%aa(:)%Oh%kd
        pakd   => sim(i,rep)%aa(:)%Oh%akd
        pw     => sim(i,rep)%aa(:)%Oh%w
        py     => sim(i,rep)%aa(:)%Oh%y
        pinc   => sim(i,rep)%aa(:)%Oh%inc
        ptax   => sim(i,rep)%aa(:)%Oh%tax
        pcc    => sim(i,rep)%aa(:)%Oh%cc
        pV     => sim(i,rep)%aa(:)%Oh%V
        pdrent => sim(i,rep)%drent

        !select relevant parts of Ch (choice)
        pli  => sim(i,rep)%aa(:)%Ch%l
        pc   => sim(i,rep)%aa(:)%Ch%c
        
        ! pre-determined variables: not to be changed during simulation
        ith         = sim(i,rep)%th
        idCC        = sim(i,rep)%pveCC
        idbk1       = sim(i,rep)%f1i
        idbk2       = sim(i,rep)%f2i
        ibk1        = sim(i,rep)%f1
        ibk2        = sim(i,rep)%f2
        iparentYres = sim(i,rep)%parentYres

        ! select relevant parts of random vectors
        irvuS    = rvuS(:,i,rep)
        irvdrent = rvdrent(i,rep)
        irvp     = rvp(lb:ub,rep)
        irvsm    = rvsm(lb:ub,rep) 
        irvlm    = rvlm(lb:ub,rep) 
        irvpm    = rvpm(lb:ub,rep) 
        irvnkd   = rvnkd(lb:ub,rep) 

    end subroutine ilocate



    !------------------------------------------------------------------------------------------------
    ! Read initial conditions from Stata
    !    may be at mina or during LC if want to keep early decisions fixed

    subroutine simReadInit
        use stataMod
        use IOpaths,  only : BHPSpath
        implicit none
        integer                                 :: unit, nobs1, rep, i, j
        integer,  dimension(nind*(maxa-mina+1)) :: idWL, age, year, insample
        integer,  dimension(nind*nrsim)         :: id, yob, f1i, f2i, g80, sampleS, student, scotland, wales
        real(wp), dimension(nind*nrsim)         :: f1, f2, parentYres
        character(200)                          :: BHPSinitialcond, BHPSwindow


        ! I/O files
        BHPSinitialcond = BHPSpath//'InitialConditions.dta'
        BHPSwindow      = BHPSpath//'ObsWindow.dta'

        ! Read initial conditions
        unit = 623
        call openStata(BHPSinitialcond, unit)
        
        nobs1 = nobsStata()
        if (nobs1 .ne. nind*nrsim) then
            print *, 'Wrong number of observations in initial conditions file: ', nobs1, nind*nrsim
        end if

        call readStata(id,         'pid'        )
        call readStata(yob,        'yob'        )
        call readStata(student,    'student'    )
        call readStata(parentYres, 'parentYres' )
        call readStata(f1,         'f1'         )
        call readStata(f2,         'f2'         )
        call readStata(f1i,        'H1'         )
        call readStata(f2i,        'H2'         )
        call readStata(g80,        'g80'        )
        call readStata(scotland,   'scotland'   )
        call readStata(wales,      'wales'      )
        call readStata(sampleS,    'sampleS'    )
        call closeOpenStata() 
        
        ! Fill structure
        j = 1
        do i=1,nind
        do rep=1,nrsim
            sim(i,rep)%id         = id(j)
            sim(i,rep)%yob        = yob(j)
            sim(i,rep)%scotland   = scotland(j)
            sim(i,rep)%wales      = wales(j)
            sim(i,rep)%student    = student(j)
            sim(i,rep)%parentYres = parentYres(j)
            sim(i,rep)%f1         = f1(j)
            sim(i,rep)%f2         = f2(j)
            sim(i,rep)%f1i        = merge(f1i(j),1,ntb1>1)
            sim(i,rep)%f2i        = merge(f2i(j),1,ntb2>1)
            sim(i,rep)%g80        = g80(j)
            sim(i,rep)%sampleS    = sampleS(j)
            j=j+1
        end do
        end do

        ! Read observation window
        unit = 376
        call openStata(BHPSwindow, unit)
        
        nobs1 = nobsStata()
        if (nobs1 .ne. nind*(maxa-mina+1)) then
            print *, 'Wrong number of observations in insample file:', nobs1, nind*(maxa-mina+1)
        end if
        
        call readStata(idWL,      'pid'        )
        call readStata(age,       'age'        )
        call readStata(year,      'year'       )
        call readStata(insample,  'insampleWL' )
        call closeOpenStata() 

        ! Fill structure
        do i=1,nind
            forall (rep=1:nrsim) sim(i,rep)%aa(:)%Oh%id       = idWL     ( (maxa-mina+1)*(i-1)+1 : (maxa-mina+1)*i)
            forall (rep=1:nrsim) sim(i,rep)%aa(:)%Oh%a        = age      ( (maxa-mina+1)*(i-1)+1 : (maxa-mina+1)*i)
            forall (rep=1:nrsim) sim(i,rep)%aa(:)%Oh%year     = year     ( (maxa-mina+1)*(i-1)+1 : (maxa-mina+1)*i)
            forall (rep=1:nrsim) sim(i,rep)%aa(:)%Oh%insample = insample ( (maxa-mina+1)*(i-1)+1 : (maxa-mina+1)*i)
            if ( sim(i,1)%aa(mina)%Oh%id .ne. sim(i,1)%id ) then
                write(*,*) 'error in simReadInit: non-matching id'
                stop
            end if
        end do

    end subroutine simReadInit



    !------------------------------------------------------------------------------------------------
    ! Cleans simulation structure except for initial conditions
    ! Fills periods under each system

    subroutine simClean
        implicit none
        integer :: i, system, a0, aN, rep
        
        ! Store periods under each tax-benefit system
        if (sysN==1) then
            sim(:,:)%sysA0(1) = mina
            sim(:,:)%sysA1(1) = maxa
            forall (i=1:nind, rep=1:nrsim) sim(i,rep)%aa(:)%Oh%system = 1
        else
            do i=1,nind
                sim(i,:)%sysA0(1)    = min(mina, sysY0(1)-sim(i,1)%yob)                        !if older than 19 in 91 => starting age of sys1 is 19
                sim(i,:)%sysA1(sysN) = max(maxa, sysY1(sysN)-sim(i,1)%yob)                     !if younger than 59 in 08 => ending age of sysN is 59
                forall(system=1:sysN-1) sim(i,:)%sysA1(system) = sysY1(system) - sim(i,1)%yob  !age last experienced each system
                forall(system=2:sysN)   sim(i,:)%sysA0(system) = sim(i,:)%sysA1(system-1) + 1  !age first experienced system
                do system = 1,sysN
                    a0 = max( sim(i,1)%sysA0(system), mina )
                    aN = min( sim(i,1)%sysA1(system), maxa )
                    if (aN>=a0) forall(rep=1:nrsim) sim(i,rep)%aa(a0:aN)%Oh%system = system
                end do
            end do
        end if
        forall(system=sysN+1:maxsysN) sim(:,:)%sysA0(system) = 0
        forall(system=sysN+1:maxsysN) sim(:,:)%sysA1(system) = 0
        
        ! Clean other parts of the sim matrices
        forall (i=1:nind, rep=1:nrsim)
            sim(i,rep)%aa(:)%Oh%k   = 0.0_wp
            sim(i,rep)%aa(:)%Oh%e   = 0.0_wp
            sim(i,rep)%aa(:)%Oh%p   = 0.0_wp
            sim(i,rep)%aa(:)%Oh%nDl = 0
            sim(i,rep)%aa(:)%Oh%m   = 0
            sim(i,rep)%aa(:)%Oh%sm  = 1
            sim(i,rep)%aa(:)%Oh%lm  = 1
            sim(i,rep)%aa(:)%Oh%pm  = 0.0_wp
            sim(i,rep)%aa(:)%Oh%wm  = 0.0_wp
            sim(i,rep)%aa(:)%Oh%ym  = 0.0_wp
            sim(i,rep)%aa(:)%Oh%kd  = 0
            sim(i,rep)%aa(:)%Oh%akd = -1
            sim(i,rep)%aa(:)%Oh%w   = 0.0_wp
            sim(i,rep)%aa(:)%Oh%y   = 0.0_wp
            sim(i,rep)%aa(:)%Oh%inc = 0.0_wp
            sim(i,rep)%aa(:)%Oh%tax = 0.0_wp
            sim(i,rep)%aa(:)%Oh%cc  = 0.0_wp
            sim(i,rep)%aa(:)%Oh%V   = 0.0_wp

            sim(i,rep)%aa(:)%Ch%l   = 1
            sim(i,rep)%aa(:)%Ch%c   = 0.0_wp
            
            sim(i,rep)%th           = 0
            sim(i,rep)%s            = 0
            sim(i,rep)%drent        = 0
            sim(i,rep)%EVpreS       = 0.0_wp
            sim(i,rep)%EVpostS      = 0.0_wp
            sim(i,rep)%lEdu_as      = 0.0_wp
        end forall

    end subroutine simClean



    !------------------------------------------------------------------------------------------------
    ! Draw random shocks

    ! Wrapper to simulate random shocks
    ! ---------------------------------
    subroutine getShocks(seed)
        use nag_g_ib,      only : g05kbf
        integer, intent(in)    :: seed
        integer                :: igen, iseed(4)

        igen     = 0
        iseed    = 0
        iseed(1) = seed
        call g05kbf(igen,iseed)
        call slDraw(iseed)
        call wlDraw(iseed)
        call MCdraws(iseed)
        
    end subroutine getShocks


    ! initial conditions
    ! preferences for schooling and working, whether pays for CC (uS, th, dCC)
    ! ------------------------------------------------------------------------
    subroutine slDraw(iseed)
        use nag_g_ib,  only : g05laf, g05lgf
        implicit none
        integer, intent(inout) :: iseed(4)
        integer isim, s, ifail, igen, rep
        real(wp) vec(nind,nrsim)


        ! uS: taste for schooling - normal(0,seS)
        rvuS(1,:,:) = 0.0_wp       !idiosyncratic preference for basic education
        do s=2, nts                !idiosyncratic preference for medium and high education
            igen  = 0
            ifail = 0
            call g05laf (0.0_wp, seS(s), nind*nrsim, rvuS(s,:,:), igen, iseed, ifail)
        end do

        ! theta: preference for working - uniform[0,1] for discrete variable
        igen  = 0
        ifail = 0
        call g05lgf(0.0_wp, 1.0_wp, nind*nrsim, vec, igen, iseed, ifail)
        sim(:,:)%th = 1
        where (vec >= ptheta1) sim%th = 2

        ! dCC: whether exposed to CC costs when working  - uniform[0,1] for discrete variable
        igen  = 0
        ifail = 0
        call g05lgf(0.0_wp, 1.0_wp, nind*nrsim, vec, igen, iseed, ifail)
        sim(:,:)%pveCC = doCC
        where (vec >  pposCC) sim(:,:)%pveCC = 0 

    end subroutine slDraw


    ! initial conditions
    ! uS for Monte-Carlo simulations of value of education
    ! ----------------------------------------------------
    subroutine MCdraws(iseed)
        use nag_g_ib,  only : g05laf
        implicit none
        integer, intent(inout) :: iseed(4)
        integer s, ifail, igen

        MCuS(1,:) = 0.0_wp       !idiosyncratic preference for basic education
        do s=2, nts              !idiosyncratic preference for medium and high education
            igen  = 0
            ifail = 0
            call g05laf (0.0_wp, seS(s), MCrep, MCuS(s,:), igen, iseed, ifail)
        end do

    end subroutine MCdraws


    ! WL shocks 
    ! productivity, marriage, spouse's education, employment and productivity, 
    !    childbirth and dummy for living in rented accomodation (p,m,sm,lm,pm,kd,drent)
    ! ---------------------------------------------------------------------------------
    subroutine wlDraw(iseed)
        use nag_g_ib,  only : g05laf, g05lgf    
        implicit none
        integer, intent(inout) :: iseed(4)
        real(wp) mu, se 
        integer  igen, ifail


        !female productivity
        mu    = 0.0_wp
        se    = 1.0_wp
        igen  = 0
        ifail = 0
        call g05laf (mu, se**2.0_wp, totn*nrsim, rvp(:,:), igen, iseed, ifail)

        !male presence and education
        igen  = 0
        ifail = 0
        call g05lgf(0.0_wp, 1.0_wp, totn*nrsim, rvsm(:,:), igen, iseed, ifail)

        !male labour supply
        igen  = 0
        ifail = 0
        call g05lgf(0.0_wp, 1.0_wp, totn*nrsim, rvlm(:,:), igen, iseed, ifail)

        !male productivity
        igen  = 0
        ifail = 0
        call g05laf (mu, se**2.0_wp, totn*nrsim, rvpm(:,:), igen, iseed, ifail)

        !childbirth
        igen  = 0
        ifail = 0
        call g05lgf(0.0_wp, 1.0_wp, totn*nrsim, rvnkd(:,:), igen, iseed, ifail)

        !whether a renter
        igen  = 0
        ifail = 0
        call g05lgf(0.0_wp, 1.0_wp, nind*nrsim, rvdrent(:,:), igen, iseed, ifail)

        return    
    end subroutine wlDraw



    !------------------------------------------------------------------------------------------------
    ! Save sim structure to Stata

    subroutine writesimstata(filename,nchar)
        use stataMod
        implicit none
        integer,intent(in)          :: nchar
        character(nchar),intent(in) :: filename
        integer, parameter          :: tota = maxa-mina+1
        integer                     :: i, a, rep, sys

        
        call saveStata(filename,3,nind*nrsim*tota)

        call writeStata((/ ( spread(rep,1,tota*nind),                          rep=1,nrsim) /), 'rep'        )
        call writeStata((/ (((sim(i,rep)%aa(:)%Oh%id),              i=1,nind), rep=1,nrsim) /), 'pid'        )
        call writeStata((/ (((a, a=mina,maxa),                      i=1,nind), rep=1,nrsim) /), 'a'          )
        call writeStata((/ ((spread(i,1,tota),                      i=1,nind), rep=1,nrsim) /), 'i'          )
        call writeStata((/ ((spread(sim(i,rep)%th,         1,tota), i=1,nind), rep=1,nrsim) /), 'th'         )
        call writeStata((/ ((spread(sim(i,rep)%pveCC,      1,tota), i=1,nind), rep=1,nrsim) /), 'pveCC'      )
        call writeStata((/ ((spread(sim(i,rep)%s,          1,tota), i=1,nind), rep=1,nrsim) /), 's'          )
        call writeStata((/ (((sim(i,rep)%aa(:)%Oh%k),               i=1,nind), rep=1,nrsim) /), 'k'          )
        call writeStata((/ (((sim(i,rep)%aa(:)%Oh%e),               i=1,nind), rep=1,nrsim) /), 'e'          )
        call writeStata((/ (((sim(i,rep)%aa(:)%Oh%w),               i=1,nind), rep=1,nrsim) /), 'w'          )
        call writeStata((/ (((sim(i,rep)%aa(:)%Oh%m),               i=1,nind), rep=1,nrsim) /), 'm'          )
        call writeStata((/ (((sim(i,rep)%aa(:)%Oh%sm),              i=1,nind), rep=1,nrsim) /), 'sm'         )
        call writeStata((/ (((sim(i,rep)%aa(:)%Oh%lm),              i=1,nind), rep=1,nrsim) /), 'lm'         )
        call writeStata((/ (((sim(i,rep)%aa(:)%Oh%wm),              i=1,nind), rep=1,nrsim) /), 'wm'         )
        call writeStata((/ (((sim(i,rep)%aa(:)%Oh%kd),              i=1,nind), rep=1,nrsim) /), 'kd'         )
        call writeStata((/ (((sim(i,rep)%aa(:)%Ch%l),               i=1,nind), rep=1,nrsim) /), 'l'          )
        call writeStata((/ (((sim(i,rep)%aa(:)%Ch%c),               i=1,nind), rep=1,nrsim) /), 'c'          )
        call writeStata((/ (((sim(i,rep)%aa(:)%Oh%inc),             i=1,nind), rep=1,nrsim) /), 'inc'        )
        call writeStata((/ (((sim(i,rep)%aa(:)%Oh%y),               i=1,nind), rep=1,nrsim) /), 'y'          )
        call writeStata((/ (((sim(i,rep)%aa(:)%Oh%ym),              i=1,nind), rep=1,nrsim) /), 'ym'         )
        call writeStata((/ (((sim(i,rep)%aa(:)%Oh%insample),        i=1,nind), rep=1,nrsim) /), 'insample'   )
        call writeStata((/ (((sim(i,rep)%aa(:)%Oh%year),            i=1,nind), rep=1,nrsim) /), 'year'       )
        call writeStata((/ (((sim(i,rep)%aa(:)%Oh%system),          i=1,nind), rep=1,nrsim) /), 'system'     )
        call writeStata((/ (((sim(i,rep)%aa(:)%Oh%nDl),             i=1,nind), rep=1,nrsim) /), 'nDl'        )
        call writeStata((/ (((sim(i,rep)%aa(:)%Oh%akd),             i=1,nind), rep=1,nrsim) /), 'akd'        )
        call writeStata((/ (((sim(i,rep)%aa(:)%Oh%V),               i=1,nind), rep=1,nrsim) /), 'EV'         )
        call writeStata((/ ((spread(sim(i,rep)%g80,        1,tota), i=1,nind), rep=1,nrsim) /), 'g80'        )
        call writeStata((/ ((spread(sim(i,rep)%sampleS,    1,tota), i=1,nind), rep=1,nrsim) /), 'sampleS'    )
        call writeStata((/ ((spread(sim(i,rep)%parentYres, 1,tota), i=1,nind), rep=1,nrsim) /), 'parentYres' )
        call writeStata((/ ((spread(sim(i,rep)%f1,         1,tota), i=1,nind), rep=1,nrsim) /), 'f1'         )
        call writeStata((/ ((spread(sim(i,rep)%f2,         1,tota), i=1,nind), rep=1,nrsim) /), 'f2'         )
        call writeStata((/ ((spread(sim(i,rep)%f1i,        1,tota), i=1,nind), rep=1,nrsim) /), 'f1i'        )
        call writeStata((/ ((spread(sim(i,rep)%f2i,        1,tota), i=1,nind), rep=1,nrsim) /), 'f2i'        )
        call writeStata((/ ((spread(sim(i,rep)%student,    1,tota), i=1,nind), rep=1,nrsim) /), 'student'    )
        call writeStata((/ ((spread(sim(i,rep)%scotland,   1,tota), i=1,nind), rep=1,nrsim) /), 'scotland'   )
        call writeStata((/ ((spread(sim(i,rep)%wales,      1,tota), i=1,nind), rep=1,nrsim) /), 'wales'      )
        call writeStata((/ ((spread(sim(i,rep)%EVpreS,     1,tota), i=1,nind), rep=1,nrsim) /), 'EVpreS'     )
        call writeStata((/ ((spread(sim(i,rep)%EVpostS(1), 1,tota), i=1,nind), rep=1,nrsim) /), 'EVpostSs1'  )
        call writeStata((/ ((spread(sim(i,rep)%EVpostS(2), 1,tota), i=1,nind), rep=1,nrsim) /), 'EVpostSs2'  )
        call writeStata((/ ((spread(sim(i,rep)%EVpostS(3), 1,tota), i=1,nind), rep=1,nrsim) /), 'EVpostSs3'  )
        call writeStata((/ ((spread(sim(i,rep)%V16(1),     1,tota), i=1,nind), rep=1,nrsim) /), 'V16s1'      )
        call writeStata((/ ((spread(sim(i,rep)%V16(2),     1,tota), i=1,nind), rep=1,nrsim) /), 'V16s2'      )
        call writeStata((/ ((spread(sim(i,rep)%V16(3),     1,tota), i=1,nind), rep=1,nrsim) /), 'V16s3'      )
        call writeStata((/ ((spread(sim(i,rep)%EV16(1),    1,tota), i=1,nind), rep=1,nrsim) /), 'EV16s1'     )
        call writeStata((/ ((spread(sim(i,rep)%EV16(2),    1,tota), i=1,nind), rep=1,nrsim) /), 'EV16s2'     )
        call writeStata((/ ((spread(sim(i,rep)%EV16(3),    1,tota), i=1,nind), rep=1,nrsim) /), 'EV16s3'     )

        call closeSaveStata

    end subroutine writesimstata

end module simMod


