! -----------------------------------------------------------------------------------------------------------
! -----------------------------------------------------------------------------------------------------------
! This file contains the routines that solve dynamic LC problem
! -----------------------------------------------------------------------------------------------------------
! -----------------------------------------------------------------------------------------------------------


module slnMod

use precision, only : wp, infty, tol
use constants
use gridvars
use parameters
use lEmatrices
use preffns,    only : util, dudc, calcEVinv, calcEdUinv, calcEVinvinv, calcEdUinvinv
use generalfns, only : LinInterp1d, LinInterp2d
use incomefns,  only : netincome
use omp_lib

! EV and EDU: discrete representations of expect value and marginal utility functions
!             conditional (prefix c) and unconditional (prefix u) on present family variables  
!             suffix 0/1 for whether man present today (case prefix c) or yesterday (case prefix u)
real(wp),dimension(ngpp,-1:maxmaxakd),private,target                  :: cEVdm0, cEDUdm0
real(wp),dimension(ngpp,-1:maxmaxakd),private                         :: uEVdm0, uEDUdm0
real(wp),dimension(ngpp,-1:maxmaxakd,ntsm,ngplm,ngppm),private,target :: cEVdm1, cEDUdm1
real(wp),dimension(ngpp,-1:maxmaxakd,ntsm,ngplm,ngppm),private        :: uEVdm1, uEDUdm1

! workings parts of value function and marginal utility matrices
real(wp),pointer,private :: plEDU(:,:,:), plEV(:,:,:), &      ! lEDU/lEV at a+1 by (k1,e1,p)
                            pcEV(:,:), pcEDU(:,:)             ! cEV and cEDU by (p_1,akd)

! future experience
real(wp),dimension(maxngpe,ngpl),private :: e1                ! next period experience
integer, dimension(maxngpe,ngpl),private :: e1lb,e1ub         ! bounds in grid

! parallel computing (comment out twice if using only 1 processor)
!OMP Directives
!The variables below are all module variables that must be made THREADPRIVATE so parallel version of code works
!$OMP THREADPRIVATE(plEDU, plEV, pcEDU, pcEV, cEVdm0, cEDUdm0, uEVdm0, uEDUdm0, cEVdm1, cEDUdm1, uEVdm1, uEDUdm1)


contains


    ! -----------------------------------------------------------------------------------------------------
    ! Wrapper for solution

    subroutine solve
        implicit none
        integer :: lasta, ix, th, f1i, f2i, s, a, dCC, ei, ki, ngpeOMP
        

	    ! Initialise matrices
        forall(ix=1:toti, th=1:ntth, f1i=1:ntb1, f2i=1:ntb2)
            lEV0(ix,th,f1i,f2i)%m0  = 0.0_wp
            lEDU0(ix,th,f1i,f2i)%m0 = 0.0_wp
            lEV1(ix,th,f1i,f2i)%m1  = 0.0_wp
            lEDU1(ix,th,f1i,f2i)%m1 = 0.0_wp
	    end forall
        

	    ! solution for retirement yrs
        do s=1,nts
            call slnRL(s)
        end do


	    ! solution for working life, a=as(s),...,maxa
        do f2i = 1,ntb2
        do f1i = 1,ntb1
        do th   = 1,ntth
	    do dCC  = 0,1

            lasta  = merge(maxa, maxabirth+aCcHi, dCC==0)

    	    do s = 1,nts
	        do a = lasta,as(s),-1
	        
                if (a<maxa) call nxtebds(a,s)
                ngpeOMP = ngpe(a,s)

                do ei=1,ngpeOMP
                    !$OMP PARALLEL DO DEFAULT(NONE) &
                    !$OMP SHARED(a, s, th, f1i, f2i, dCC, ei, ngpeOMP, lasta) &
                    !$OMP SHARED(e1, e1lb, e1ub) &
                    !$OMP PRIVATE (ki)
                    do ki=1,ngpk
                        call slnWL(th, f1i, f2i, dCC, a, s, ei, ki)
                    end do
                    !$OMP END PARALLEL DO
                end do

            end do
	        end do

    	end do
	    end do
	    end do
	    end do


        ! solution for education years if taking university education
        ! completes Ematrices at a=mina (19) for s=3
        call slnSL
        

    end subroutine solve



    ! -----------------------------------------------------------------------------------------------------
    ! Routines to solve woman's problem at each stage of life
    ! -----------------------------------------------------------------------------------------------------

    
    ! -----------------------------------------------------------------------------------------------------
    ! Solution to women's problem during retirement periods
    ! uncertainty:  only presence of partner
    ! exports approximate inverse of EV/EDU at a=maxa+1 conditional on male presence at a=maxa

    subroutine slnRL(s)
        implicit none
        integer, intent(in) :: s
        integer             :: th, f1i, f2i, l, a, ki, lbk, ubk, m, sm, lm, m_1, sm_1, kd, akd, kd_1, akd_1 
        real(wp)            :: p, pm, yr(-1:maxmaxakd,0:domales), c, v(-1:maxmaxakd,0:domales,ntsm), du(-1:maxmaxakd,0:domales,ntsm), k1, eulerArr(ngpk)


        ! initiate variables and grids
	    th  = 1                    ! solution indep of th since l=1 during retirement 
	    f1i = 1                    ! solution indep of background after retirement
	    f2i = 1
        l   = 1                    ! women are all retired
        p   = 0.0_wp               ! her productivity is irrelevant
        lm  = 1                    ! (present) men are all retired
        pm  = 0.0_wp               ! his productivity is irrelevant
        yr  = 0.0_wp               ! retirement income by age of kid and presence of male


        ! now solve problem during retirement years (ages maxa+1 to enda)
        do a=enda,maxa+1,-1
        do ki=1,ngpk
    
            v  = 0.0_wp
            du = 0.0_wp
            
            ! solution: last period of life
            if (a==enda) then
            
                do m  = 0,domales
                do kd = 0,maxnkdvec(a,s)
                do akd = (kd-1)+kd*minakd(a,s),(kd-1)+kd*maxakd(a,s)
                    c           = R*gk(a,ki,s)+yr(akd,m)
                    v(akd,m,:)  = util(c,l,m,lm,kd,akd,th,s,f1i,f2i)
                    du(akd,m,:) = dudc(c,l,m,lm,kd,akd,th,s,f1i,f2i)
                end do
                end do
                end do
            
            ! solution: other periods after retirement
            else

                do kd = 0,maxnkdvec(a,s)
                do akd = (kd-1)+kd*minakd(a,s),(kd-1)+kd*maxakd(a,s)
                do m  = 0,domales
                do sm  = 1,1+m*(ntsm-1)
                    eulerArr = EEpar(l,th)%b(lm,m,akd,s,f1i,f2i)*lEDUr(s,akd,m,sm,:) - (R*gk(a,ki,s) + yr(akd,m) - gk(a+1,:,s))
                    call solveEE(a+1, s, eulerArr, lbk, ubk, k1)
                    c            = R*gk(a,ki,s) + yr(akd,m) - k1
                    v(akd,m,sm)  = util(c,l,m,lm,kd,akd,th,s,f1i,f2i) + &
                                   beta*calcEVinvinv(LinInterp1d(k1,gk(a+1,lbk,s),gk(a+1,ubk,s),lEVr(s,akd,m,sm,lbk),lEVr(s,akd,m,sm,ubk)))
                    du(akd,m,sm) = dudc(c,l,m,lm,kd,akd,th,s,f1i,f2i)
                end do
                end do
                end do
                end do
                
            end if


            !integrate family composition conditional on m_1=0: (kd,akd,m)
            m_1 = 0
            do kd_1 = 0,maxnkdvec(a-1,s)
            do akd_1 = (kd_1-1)+kd_1*minakd(a-1,s),(kd_1-1)+kd_1*maxakd(a-1,s)
                lEVr(s,akd_1,m_1,:,ki)  = calcEVinv  (dot_product( trkd(:,akd_1,a,s,m_1) ,  &
                                                                   (1.0_wp-sum(pr0m(:,a,s)))*v(:,0,1) + pr0m(1,a,s)*v(:,1,1) + pr0m(2,a,s)*v(:,1,2) + pr0m(3,a,s)*v(:,1,3) ))
                lEDUr(s,akd_1,m_1,:,ki) = calcEdUinv (dot_product( trkd(:,akd_1,a,s,m_1) ,  &
                                                                   (1.0_wp-sum(pr0m(:,a,s)))*du(:,0,1) + pr0m(1,a,s)*du(:,1,1) + pr0m(2,a,s)*du(:,1,2) + pr0m(3,a,s)*du(:,1,3) ))
            end do
            end do


            !integrate family composition conditional on m_1=1: (kd,akd,m)
            m_1=1
            do sm_1  = 1, ntsm
            do kd_1 = 0,maxnkdvec(a-1,s)
            do akd_1 = (kd_1-1)+kd_1*minakd(a-1,s),(kd_1-1)+kd_1*maxakd(a-1,s)
                lEVr(s,akd_1,m_1,sm_1,ki)  = calcEVinv  ( dot_product( trkd(:,akd_1,a,s,m_1) , &
                                                                       trCS(sm_1,kd_1,a)*v(:,0,1) + (1.0_wp-trCS(sm_1,kd_1,a))*v(:,1,sm_1) ))
                lEDUr(s,akd_1,m_1,sm_1,ki) = calcEdUinv ( dot_product( trkd(:,akd_1,a,s,m_1) , &
                                                                       trCS(sm_1,kd_1,a)*du(:,0,1) + (1.0_wp-trCS(sm_1,kd_1,a))*du(:,1,sm_1) ))
            end do
            end do
            end do

        end do
        end do

    end subroutine slnRL


    
    ! -----------------------------------------------------------------------------------------------------
    ! Solution to women's problem during working life: ages as(s):maxa
    ! solves at point (a,th,s,dCC) for all (e,k,p,kd,akd,m,sm,lm)
    ! then integrates over p|p_1, and later (kd,akd,m,sm,lm)|(kd_1,akd_1,m_1,sm_1,lm_1)

    subroutine slnWL(th, f1i, f2i, dCC, a, s, ei, ki)
        implicit none
        integer, intent(in) :: th, f1i, f2i, dCC, s, a, ei, ki
        integer             :: m, sm, lm, pmi, kd, akd, kd_1, akd_1, akd0, akd1, ix, nNaN0, nNaN1
        real(wp)            :: e, k, pm, miny


        ! initiate variables and grids
	    e    = ge(a,ei,s)                      ! experience
	    k    = gk(a,ki,s)                      ! assets
	    miny = mink(a+1,s)+minc-R*k            ! min income making minc affordable for remaining life
	    cEVdm0  = 0.0_wp;  cEVdm1  = 0.0_wp    ! expected continuation value conditional on m=0
	    cEDUdm0 = 0.0_wp;  cEDUdm1 = 0.0_wp    ! expected continuation value conditional on m=1 and his characteristics


        !singles today (m=0): solve and integrate over p conditional on p_1
        m=0;   lm=1;   sm=1;   pm=0.0_wp                      ! male characteristics

        pcEDU => cEDUdm0(:,:)                                  ! to fill EDU/EV (p_1,akd)
        pcEV  => cEVdm0(:,:)

        do kd = 0, maxnkdvec(a,s)                              ! sln by presence and age of child today
            akd0 = merge(-1,minakd(a,s),kd==0)
            akd1 = merge(-1,maxakd(a,s),kd==0)
            do akd = akd0, akd1
                if (a<maxa) then
	                ix    =  iEmtx%ix(akd, a+1, dCC, s)
                    plEDU => lEDU0(ix,th,f1i,f2i)%m0(:,:,:)
                    plEV  => lEV0(ix,th,f1i,f2i)%m0(:,:,:)
                end if
                call get_cEmtr(th, s, a, k, ei, e, m, sm, lm, pm, kd, akd, dCC, miny, f1i, f2i)
            end do
        end do


        !women in couples today (m=1): solve and integrate over p conditional on p_1
        m = 1
        do sm  = 1,ntsm
        do lm  = 1,ngplm
        do pmi = 1,ngppm

            pm = gpm(pmi,sm,a)                            ! male productivity

            pcEDU => cEDUdm1(:,:,sm,lm,pmi)               ! to fill EDU/EV (p_1,akd)
            pcEV  => cEVdm1(:,:,sm,lm,pmi)

            do kd = 0,maxnkdvec(a,s)                      ! sln by presence and age of child today
                akd0 = merge(-1,minakd(a,s),kd==0)
                akd1 = merge(-1,maxakd(a,s),kd==0)
                do akd = akd0, akd1
                    if (a<maxa) then
          	            ix    =  iEmtx%ix(akd, a+1, dCC, s)
                        plEDU => lEDU1(ix,th,f1i,f2i)%m1(:,:,:,sm,lm,pmi)
                        plEV  => lEV1(ix,th,f1i,f2i)%m1(:,:,:,sm,lm,pmi)
                    end if
                    call get_cEmtr(th, s, a, k, ei, e, m, sm, lm, pm, kd, akd, dCC, miny, f1i, f2i)
                end do
            end do

        end do
        end do
        end do


        ! integrate family characteristics and invert E fns
        call integrate_family(a, s)
        call calcEinv(th, f1i, f2i, dCC, a, s, ki, ei)


        ! check that all points have feasible solutions
        do kd_1 = 0,maxnkdvec(a-1,s)
        do akd_1 = (kd_1-1)+kd_1*minakd(a-1,s),(kd_1-1)+kd_1*maxakd(a-1,s)
            ix    =  iEmtx%ix(akd_1, a, dCC, s)
            nNaN0 = count(isnan(lEV0(ix,th,f1i,f2i)%m0(ki,ei,:)))
            nNaN1 = count(isnan(lEV1(ix,th,f1i,f2i)%m1(ki,ei,:,:,:,:)))          
            if (nNaN0+nNaN1>0) then
                write(*,*) 'NaN', nNaN0, nNaN1
            end if
        end do
        end do
            
    end subroutine slnWL



    ! -----------------------------------------------------------------------------------------------------
    ! Solution to women's problem during university years

    subroutine slnSL
        use nag_g_ib,     only : g05laf, g05kbf
        use fortax_data,  only : sysdate
        implicit none
        integer,parameter  :: nMC = 10000
        real(wp)           :: gamma0, gamma1 
        real(wp),pointer   :: plEVk(:), plEDUk(:)
        real(wp)           :: k, k1, v, c19, c20, c21, eulerArr(ngpk), EVs(nts), EV0(nMC), uS(2:3,nMC)
        integer            :: s, s1, a, a1, m_1, lm_1, kd_1, akd_1, dCC, f1i, f2i, th, lbk, ubk, ix, ix1, ki
        integer            :: iseed(4), igen, ifail, i


        gamma0 = (beta*R)**(1.0_wp/(mu0-1.0_wp))           ! utility parameters
        gamma1 = 1.0_wp + R*gamma0 + ((R*gamma0)**2)       ! utility parameters
        fee    = Ufee                                      ! constant Ufee over period
        
        ! constants before working life
        s1    = nts
        a     = mina
        a1    = as(s1)
        m_1   = 0
        lm_1  = 1
        kd_1  = 0
        akd_1 = -1     ! no kids at as(s1)-1


        ! solve problem at all points in state space (th,f1,f2,k,CC)
        do dCC = 1-doCC,1
        do f2i = 1,ntb2
        do f1i = 1,ntb1
        do th   = 1,ntth

	        ix     =  iEmtx%ix(akd_1, a, dCC, s1)               ! index during university years
	        ix1    =  iEmtx%ix(akd_1, a1, dCC, s1)              ! index for continuation value at 22
            plEVk  => lEV0(ix1,th,f1i,f2i)%m0(:,1,1)            ! continuation value at 22
            plEDUk => lEDU0(ix1,th,f1i,f2i)%m0(:,1,1)

            do ki=1,ngpk
            
                k        = gk(a,ki,s1)
                eulerArr = EEpar(ngpl,th)%b(lm_1,m_1,akd_1,s1,f1i,f2i)*plEDUk - ((R**(a1-a))*(k-fee) - gk(a1,:,s1))/gamma1
                call solveEE(a1,s1,eulerArr,lbk,ubk,k1)

                ! optimal consumption and savings during university years
                c21 = ((R**(a1-a))*(k-fee)-k1)/gamma1
                c20 = gamma0*c21
                c19 = (gamma0**2)*c21
                v   = util(c19,ngpl,m_1,lm_1,kd_1,akd_1,th,s1,f1i,f2i) + &
                      beta*util(c20,ngpl,m_1,lm_1,kd_1,akd_1,th,s1,f1i,f2i) + &
                      (beta**2)*util(c21,ngpl,m_1,lm_1,kd_1,akd_1,th,s1,f1i,f2i) + &
                      (beta**3)*calcEVinvinv( LinInterp1d (k1, gk(a1,lbk,s1), gk(a1,ubk,s1), plEVk(lbk), plEVk(ubk)) )
                lEV0(ix,th,f1i,f2i)%m0(ki,:,:) = calcEVinv(v)

            end do

        end do
        end do
        end do
        end do
        
    end subroutine slnSL
    


    ! -----------------------------------------------------------------------------------------------------
    ! Solve woman's problem at age a during working life
    !   Compute optimal (c,l) at point (a,s,th,dCC,e,k,p,kd,akd,m,sm,lm,pm) 
    !   Then calculate V and dU at optimum and integrate over p|p_1

    subroutine get_cEmtr(th, s, a, k, ei, e, m, sm, lm, pm, kd, akd, dCC, miny, f1i, f2i)
        implicit none
        integer,  intent(in) :: th, s, a, ei, m, sm, lm, kd, akd, dCC, f1i, f2i
        real(wp), intent(in) :: k, e, pm, miny
        real(wp) y(ngpl), c(ngpl), v(ngpl), optV(ngpp), optDU(ngpp), eulerArr(ngpk)
        real(wp) resources, k1, e1lbval, e1ubval, e1val
        integer pi, pi_1, l, act(1), e1lbi, e1ubi, lbk, ubk


        y(1) = netincome(1,s,e,0.0_wp,m,lm,sm,a,pm,akd,dCC,th,f1i,f2i)            ! income if not working

        do pi=1,ngpp

            y(2) = netincome(2,s,e,gp(pi,a,s,th),m,lm,sm,a,pm,akd,dCC,th,f1i,f2i) ! present income if working PT
            y(3) = netincome(3,s,e,gp(pi,a,s,th),m,lm,sm,a,pm,akd,dCC,th,f1i,f2i) ! present income if working FT

            !optimal consumption conditional on labour supply
            v = -infty
            c = 0.0_wp
            
            do l=1,ngpl    
            if (y(l)>miny-tol) then

                resources = R*k+y(l)
                    
                if (a<maxa) then

                    e1val    = e1(ei,l)
                    e1lbi    = e1lb(ei,l)
                    e1ubi    = e1ub(ei,l)
                    e1lbval  = ge(a+1, e1lbi, s)
                    e1ubval  = ge(a+1, e1ubi, s)
                    eulerArr = LinInterp1D(e1val, e1lbval, e1ubval, plEDU(:,e1lbi,pi), plEDU(:,e1ubi,pi))
                    eulerArr = EEpar(l,th)%b(lm,m,akd,s,f1i,f2i)*eulerArr - (resources-gk(a+1,:,s))
                    call solveEE(a+1, s, eulerArr, lbk, ubk, k1)
                    c(l) = resources-k1
                    v(l) = util(c(l),l,m,lm,kd,akd,th,s,f1i,f2i) + &
                           beta*calcEVinvinv( LinInterp2d (k1, gk(a+1,lbk,s), gk(a+1,ubk,s), e1val, e1lbval, e1ubval, &
                                                           plEV(lbk,e1lbi,pi), plEV(lbk,e1ubi,pi), plEV(ubk,e1lbi,pi), plEV(ubk,e1ubi,pi)) )

                else

                    eulerArr = EEpar(l,th)%b(lm,m,akd,s,f1i,f2i)*lEDUr(s,akd,m,sm,:) - (resources-gk(maxa+1,:,s))
                    call solveEE(a+1, s, eulerArr, lbk, ubk, k1)
                    c(l) = resources - k1
                    v(l) = util(c(l),l,m,lm,kd,akd,th,s,f1i,f2i) + &
                           beta * calcEVinvinv(LinInterp1d (k1,gk(maxa+1,lbk,s),gk(maxa+1,ubk,s),lEVr(s,akd,m,sm,lbk),lEVr(s,akd,m,sm,ubk)))

                end if
                    
            end if
            end do

            ! optimum labour supply and value
	        act       = maxloc(v(:))
	        optV(pi)  = v(act(1))
	        optDU(pi) = dudc(c(act(1)),act(1),m,lm,kd,akd,th,s,f1i,f2i)
            
        end do

        ! integrate over female productivity conditional on past productivity
        do pi_1=1,ngpp
            pcEV(pi_1,akd+2)  = dot_product(optV,trp(:,pi_1,a,s,th))
            pcEDU(pi_1,akd+2) = dot_product(optDU,trp(:,pi_1,a,s,th))
        end do

    end subroutine get_cEmtr




    ! -----------------------------------------------------------------------------------------------------
    ! Mixed auxilliary routines
    ! -----------------------------------------------------------------------------------------------------

    ! -----------------------------------------------------------------------------------------------------
    ! solves Euler equation and outputs optimum k(a+1) and ub on grid for k(a+1)

    subroutine solveEE(a1,educ,eulerArr,lbk,ubk,k1)
        implicit none
        integer,intent(in)   :: a1, educ
        real(wp),intent(in)  :: eulerArr(ngpk)
        integer,intent(out)  :: lbk, ubk
        real(wp),intent(out) :: k1 
        real(wp) k1EE

        
        ! if credit constraint is binding
        if (eulerArr(1)>0.0_wp) then    
            lbk=1
            ubk=1
        
        ! if it is not
        else
            ubk = 2
            do
                if ((eulerArr(ubk) > 0.0_wp) .or. (ubk == ngpk)) exit
                ubk = ubk+1
            end do
            lbk=ubk-1
        
            do
                if (eulerArr(lbk) < eulerArr(ubk)) exit
                if (ubk==ngpk) then
                    write(*,*) 'error in EE'
                    exit
                else
                    ubk = ubk+1
                end if
            end do
        end if
            
        k1EE = LinInterp1d(0.0_wp, eulerArr(lbk), eulerArr(ubk), gk(a1,lbk,educ), gk(a1,ubk,educ))
        k1   = max( mink(a1,educ), k1EE)
        
    end subroutine solveEE



    ! -----------------------------------------------------------------------------------------------------
    ! integrate out family characteristics at age a, (kd, akd, m, sm, lm, pm), conditional on family 
    !    characteristics at age a-1, (kd_1, akd_1, m_1, sm_1, lm_1, pm_1)
    ! inputs:  matrices cEV and cEDU - EV and EDU conditional on (kd, akd, m, sm, lm, pm)
    ! outputs: matrices uEV and uEDU - EV and EDU as fns of (kd_1, akd_1, m_1, sm_1, lm_1, pm_1)

    subroutine integrate_family(a, s)
        implicit none
        integer,  intent(in)                               :: a, s
        real(wp), dimension(ngpp,-1:maxmaxakd)             :: EV0, EDU0
        real(wp), dimension(ngpp,-1:maxmaxakd,ngplm,ngppm) :: EV1, EDU1
        integer  :: lbakd, ubakd, lbakd_1, ubakd_1, minnkd, maxnkd, kd, akd, kd_1, akd_1, pi_1, m_1, sm_1, pmi_1, lm_1
        real(wp) :: pmleave


        uEVdm0  = 0.0_wp
        uEVdm1  = 0.0_wp
        uEDUdm0 = 0.0_wp
        uEDUdm1 = 0.0_wp


        ! single women at a-1
        ! -------------------
        m_1  = 0

        do kd_1 = 0,maxnkdvec(a-1,s)
                
            EV0  = 0.0_wp
            EDU0 = 0.0_wp        

            ! integrate over male characteristics today, by presence and age of child
            minnkd = merge(0, 1, kd_1==0 .or. maxakd(a-1,s)==18)
            maxnkd = merge(0, maxnkdvec(a,s), kd_1==0 .and. a>maxabirth)
            do kd = minnkd, maxnkd
                lbakd = merge(-1, minakd(a,s), kd==0)
                ubakd = merge(-1, kd_1*maxakd(a,s), kd==0)
                forall (akd=lbakd:ubakd, pi_1=1:ngpp)
                    EV0(pi_1,akd) =  trSS(a,s)*cEVdm0(pi_1,akd)  + sum( trSC(a,s)%m(:,:,:)*cEVdm1(pi_1,akd,:,:,:) )
                    EDU0(pi_1,akd) = trSS(a,s)*cEDUdm0(pi_1,akd) + sum( trSC(a,s)%m(:,:,:)*cEDUdm1(pi_1,akd,:,:,:) )
                end forall
            end do

            ! integrate over child characteristics today
            lbakd_1 = merge(-1, minakd(a-1,s), kd_1==0)
            ubakd_1 = merge(-1, maxakd(a-1,s), kd_1==0)
            forall (akd_1=lbakd_1:ubakd_1, pi_1=1:ngpp)
                uEVdm0(pi_1,akd_1)  = dot_product(trkd(:,akd_1,a,s,m_1),EV0(pi_1,:))
                uEDUdm0(pi_1,akd_1) = dot_product(trkd(:,akd_1,a,s,m_1),EDU0(pi_1,:))
            end forall

        end do
        

        ! women in couples at a-1
        ! -----------------------
        if (a>minam(s)) then

            m_1  = 1

            do sm_1  = 1,ntsm
            do kd_1 = 0,maxnkdvec(a-1,s)
            
                EV1      = 0.0_wp
                EDU1     = 0.0_wp        

                ! integrate over male characteristics today, by presence and age of child
                pmleave = trCS(sm_1,kd_1,a)
                minnkd  = merge(0, 1, kd_1==0 .or. maxakd(a-1,s)==18)
                maxnkd  = merge(0, maxnkdvec(a,s), kd_1==0 .and. a>maxabirth)
                
                do kd = minnkd, maxnkd
                    lbakd = merge(-1, minakd(a,s), kd==0)
                    ubakd = merge(-1, kd_1*maxakd(a,s), kd==0)
                    forall (pmi_1=1:ngppm, lm_1=1:ngplm, akd=lbakd:ubakd, pi_1=1:ngpp)
                        EV1(pi_1,akd,lm_1,pmi_1)  = pmleave * cEVdm0(pi_1,akd) + sum( trCC(pmi_1,lm_1,kd_1,a)%m(sm_1,:,:) * cEVdm1(pi_1,akd,sm_1,:,:) )
                        EDU1(pi_1,akd,lm_1,pmi_1) = pmleave * cEDUdm0(pi_1,akd) + sum( trCC(pmi_1,lm_1,kd_1,a)%m(sm_1,:,:) * cEDUdm1(pi_1,akd,sm_1,:,:) )
                    end forall
                end do

                ! integrate over child characteristics today
                lbakd_1 = merge(-1, minakd(a-1,s), kd_1==0)
                ubakd_1 = merge(-1, maxakd(a-1,s), kd_1==0)
                forall (pmi_1=1:ngppm, lm_1=1:ngplm, akd_1=lbakd_1:ubakd_1, pi_1=1:ngpp)
                    uEVdm1(pi_1,akd_1,sm_1,lm_1,pmi_1)  = dot_product(trkd(:,akd_1,a,s,m_1),EV1(pi_1,:,lm_1,pmi_1))
                    uEDUdm1(pi_1,akd_1,sm_1,lm_1,pmi_1) = dot_product(trkd(:,akd_1,a,s,m_1),EDU1(pi_1,:,lm_1,pmi_1))
                end forall
            
            end do
            end do
            
        end if

    end subroutine integrate_family
    


    ! -----------------------------------------------------------------------------------------------------
    ! calculate and store linearise versions of EV and EDU and store values

    subroutine calcEinv(th, f1i, f2i, dCC, a, s, ki, ei)
        implicit none
        integer, intent(in) :: th, f1i, f2i, dCC, a, s, ki, ei
        integer             :: ix, akd_1

        ! if no kids and no partner at a-1
        akd_1 = -1
        ix    = iEmtx%ix(akd_1, a, dCC, s)
        lEV0(ix,th,f1i,f2i)%m0(ki,ei,:)  = calcEVinv(uEVdm0(:,akd_1))
        lEDU0(ix,th,f1i,f2i)%m0(ki,ei,:) = calcEdUinv(uEDUdm0(:,akd_1))
        
        ! in the presence of kids but no partner at a-1
        do akd_1 = minakd(a-1,s),maxakd(a-1,s)
            ix = iEmtx%ix(akd_1, a, dCC, s)
            lEV0(ix,th,f1i,f2i)%m0(ki,ei,:)  = calcEVinv(uEVdm0(:,akd_1))
            lEDU0(ix,th,f1i,f2i)%m0(ki,ei,:) = calcEdUinv(uEDUdm0(:,akd_1))
        end do

        !for women in couples at a-1
        if (a>as(s)) then
        
            ! if no kids at a-1
            akd_1 = -1
            ix    = iEmtx%ix(akd_1, a, dCC, s)
            lEV1(ix,th,f1i,f2i)%m1(ki,ei,:,:,:,:)  = calcEVinv(uEVdm1(:,akd_1,:,:,:))
            lEDU1(ix,th,f1i,f2i)%m1(ki,ei,:,:,:,:) = calcEdUinv(uEDUdm1(:,akd_1,:,:,:))
        
            ! in the presence of kids at a-1
            do akd_1 = minakd(a-1,s),maxakd(a-1,s)
                ix = iEmtx%ix(akd_1, a, dCC, s)
                lEV1(ix,th,f1i,f2i)%m1(ki,ei,:,:,:,:)  = calcEVinv(uEVdm1(:,akd_1,:,:,:))
                lEDU1(ix,th,f1i,f2i)%m1(ki,ei,:,:,:,:) = calcEdUinv(uEDUdm1(:,akd_1,:,:,:))
            end do
        end if
    
    end subroutine calcEinv



    ! -----------------------------------------------------------------------------------------------------
    ! computes next value of e and respective bounds in ge

    subroutine nxtebds(a,s)
        use motionfns, only : nxte
        implicit none
        integer,intent(in) :: a, s
        integer            :: a1, ei, nei, l

        a1 = a+1

        do l = 1,ngpl
        do ei = 1,ngpe(a,s)
        
            e1(ei,l) = nxte(ge(a,ei,s),l,s)
            
            do nei=1, ngpe(a1,s)
                if (abs(ge(a1,nei,s)-e1(ei,l)) <= tol) then
                    e1lb(ei,l) = nei
                    e1ub(ei,l) = nei
                    exit
                else if (ge(a1,nei,s) > e1(ei,l)) then   !this is not possible if nei=1
                    e1lb(ei,l) = nei-1           
                    e1ub(ei,l) = nei
                    exit
                end if
            end do
            
        end do
        end do

    end subroutine nxtebds


end module slnMod
