! -----------------------------------------------------------------------------------------------------------
! -----------------------------------------------------------------------------------------------------------
! Calculates the variance of estimated structural parameters and exports parameter values and variances to
! text file (numbers to construct tables 8 to 10)
! -----------------------------------------------------------------------------------------------------------
! -----------------------------------------------------------------------------------------------------------


module varMod

use constants
use parameters
use IOpaths
use SimConditions, only : defaultseed
use wrapperMod,    only : SolveAndSim_manySys
use momMod

contains


    ! -----------------------------------------------------------------------------------------------------
    ! Main routine 
    ! calculates the variance of estimated parameters

    subroutine getVariance
        use nag_f_ib, only : f01adf
        implicit none
        real(wp)                    :: Qform
        real(wp), allocatable       :: dMom(:,:), A(:,:), iA(:,:), V(:,:)
        integer                     :: totMom, n, i, j, ifail, npar, unit
        type(structureM)            :: mom
        character(*), parameter     :: outputfile = parampath//'FinalParamAndSE.txt'


        ! solve problem and simulate moments using estimated parameters
        call SolveAndSim_manySys(0, ' ', 1, 1)
        call CalcSimMoments(Qform, totMom, mom)
        write(*,'("Objective value is: ", F18.5)') Qform
        write (*,*)
        npar = param%n
                
        ! calculate derivatives of moments with respect to each estimating parameter
        if (allocated(dMom)) deallocate(dMom);  allocate (dMom (totMom,npar))
        call getDerivatives(1.0e-3_wp, npar, totMom, dMom)

        ! Calculate iA(npar,npar) = dMom' Omega dMom
        if (allocated(A))  deallocate(A);   allocate (A(npar+1,npar))
        if (allocated(iA)) deallocate(iA);  allocate (iA(npar,npar))
        A(1:npar,1:npar) = matmul( matmul(transpose(dMom),Omega) , dMom )
        ifail = 0
        call f01adf(npar, A, npar+1, ifail)
        do j = 1,npar
        do i = j,npar
            iA(i,j) = A(i+1,j)
            iA(j,i) = A(i+1,j)
        end do
        end do

        ! Calculate VarCov matrix
        if (allocated(V)) deallocate(V);  allocate   (V(npar,npar))
        V = (1.0_wp + 1.0_wp/real(nrsim,wp)) * iA

        ! Export to text parameters and variances
        unit = 7942
        open (unit, file=outputfile, action='write')
        n = 0
        do i = 1,npar
            n = n+1
            write(unit, 316) param%name(i), param%val(i), sqrt(V(n,n)), param%text(i)
        end do
        close (unit)
        
        316 format (A12,3F14.6,A50)
        
    end subroutine getVariance



    ! ---------------------------------------------------------------------------------------------------------
    ! Get derivatives of moments with respect to the parameters at estimated values

    subroutine getDerivatives(delta, npar, totMom, dMom)
        implicit none
        real(wp),         intent(in)    :: delta
        integer,          intent(in)    :: npar, totMom
        real(wp),         intent(out)   :: dMom(totMom,npar)
        type(structureM)                :: mom
        real(wp)                        :: estP, change, newM(2,totMom), diff, Qform(2)
        integer                         :: i, j, iter, mi, s, nrMom


        ! Solve, simulate and calculate at estimated parameters
        dMom = 0.0_wp
        
        ! loop over estimating parameters
        do i = 1,npar

            write(*,'(I3,A37)') i, param%text(i)

            ! keep original value of parameter
            estP = param%val(i)

            ! iterate until perturbation large enough to change objective function
            do iter=0,4
            
                change = delta*(10**iter)

                do j = 1,2
                    if (j==1) param%val(i) = max( estP-change, param%lbval(i) ) ! perturb parameter on both direction
                    if (j==2) param%val(i) = min( estP+change, param%ubval(i) )
                    call SolveAndSim_manySys(0, ' ', 1, 1)                      ! solve and simulate individual's decisions
                    call CalcSimMoments(Qform(j), nrMom, mom)                   ! calculate moments & objective function
                    call storeMom(mom, totMom, newM(j,:))                       ! store estimated moments
                end do

                diff =  min(estP+change, param%ubval(i)) - max(estP-change, param%lbval(i))
                write(*,'("  ",4F12.5)') diff, Qform(1), Qform(2), abs(Qform(1)-Qform(2))
                if (abs(Qform(1)-Qform(2)) > tol) exit
            
            end do
            
            do mi=1,totMom
                dMom(mi,i) = ( newM(2,mi) - newM(1,mi) ) / diff
            end do
            
            ! retrieve original value of parameter
            param%val(i) = estP             
            write (*,'(A24,I2,F15.10)') "  converged in iter ", iter, sum(abs(dMom(:,i)))

        end do
        
    end subroutine getDerivatives


    ! ---------------------------------------------------------------------------------------------------------
    ! Arrange moments in correct order to calculate variance

    subroutine storeMom(mom,totMom,Mvec)
        implicit none
        type(structureM), intent(in) :: mom
        integer, intent(in)          :: totMom
        real(wp),intent(out)         :: Mvec(totMom)
        integer                      :: mi, s, j


        ! education moments first (only for s=2,3)
        mi = 0
        do s=2,nts

            do j=1,mom%Sdist%n
                mi = mi+1
                Mvec(mi)  = mom%Sdist%sim(j,s)
            end do

            do j=1,mom%Sregr%n
                mi = mi+1
                Mvec(mi)  = mom%Sregr%sim(j,s)
            end do

        end do

        ! now WL moments
        do s=1,nts
        
            do j=1,mom%E%n
                mi = mi+1
                Mvec(mi)  = mom%E%sim(j,s)
            end do

            do j=1,mom%PT%n
                mi = mi+1
                Mvec(mi)  = mom%PT%sim(j,s)
            end do

            do j=1,mom%UE%n
                mi = mi+1
                Mvec(mi)  = mom%UE%sim(j,s)
            end do

            do j=1,mom%EU%n
                mi = mi+1
                Mvec(mi)  = mom%EU%sim(j,s)
            end do

            do j=1,mom%lW0%n
                mi = mi+1
                Mvec(mi)  = mom%lW0%sim(j,s)
            end do

            do j=1,mom%dlWe%n
                mi = mi+1
                Mvec(mi)  = mom%dlWe%sim(j,s)
            end do

            do j=1,mom%lWregr%n
                mi = mi+1
                Mvec(mi)  = mom%lWregr%sim(j,s)
            end do
 
            do j=1,mom%lWa%n
                mi = mi+1
                Mvec(mi) = mom%lWa%sim(j,s)
            end do

            do j=1,mom%dlWy%n
                mi = mi+1
                Mvec(mi)  = mom%dlWy%sim(j,s)
            end do

            do j=1,mom%lWdist%n
                mi = mi+1
                Mvec(mi)  = mom%lWdist%sim(j,s)
            end do

            do j=1,mom%CC%n
                mi = mi+1
                Mvec(mi)  = mom%CC%sim(j,s)
            end do
                
        end do
        
    end subroutine storeMom

end module varMod

