! -----------------------------------------------------------------------------------------------------------
! -----------------------------------------------------------------------------------------------------------
! Wrapper: solution, simulation and moment calculation routines to be called by higher level routines
! -----------------------------------------------------------------------------------------------------------
! -----------------------------------------------------------------------------------------------------------


module wrapperMod

use slnMod
use simMod
use momMod
use omp_lib 

real(wp) :: QformScale

contains


    ! -------------------------------------------------------------------------------------------------------
    ! Load information to start solution and simulation routines

    subroutine getstartedsolution(prisk)
        use parametersfcs, only : paramCopy, DerivedParameters
        use gridfcs,       only : popgp
        real(wp), intent(in) :: prisk

        call paramCopy(param, prisk)    ! Copy parameters to relevant internal locations
        call DerivedParameters(prisk)   ! Calculate derived parameters and grids
        call popgp
        
    end subroutine getstartedsolution


    subroutine getstartedsimulation(seed)
        use nag_g_ib,      only : g05kbf
        integer, intent(in)    :: seed
        integer                :: igen, iseed(4)

        call simReadInit
        call simClean
        call getShocks(seed)        
        
    end subroutine getstartedsimulation


    subroutine defaultSim
        use SimConditions
        use fortax_data, only : fortax_loadsys_anydate, sys
        use parameters,  only : rent0
        implicit none
        integer :: year, system

        calcFrisch     = 0         ! calculate Frisch elasticities: always set to zero here
        calcMarshall   = 0         ! calculate Marshall elasticities: always set to zero here
        pchangeNetInc  = 0.005_wp  ! proportional change in female net earnings
        rent           = rent0     ! housing rent (set to 0 for estimation)
        dUfee          = 1         ! include university tuition fees
        individualWFTC = 0         ! assess WFTC eligibilty individual income: always set to zero here

        do year = 1991,2008
            system = year*10000 + 407
            call fortax_loadsys_anydate(system)
            allsys(year) = sys
        end do

    end subroutine defaultSim


    ! -------------------------------------------------------------------------------------------------------
    ! Solve individual's problem, simulate LC profiles, calculate and store moments
    ! Tax-benefit systems may change over hte life-cycle
    ! Set systems before calling this routine

    subroutine SolveAndSim_manySys(dwrite, filename, nchar, chooseS)
        implicit none
        integer,intent(in)          :: dwrite, nchar, chooseS
        character(nchar),intent(in) :: filename
        integer                     :: i, a, ai, s, system, age0, age1, rep
        real(8)                     :: x1(nts), x2(nts), x3(nts)

        ! Get initial information
        call getstartedsolution(1.0_wp)
        call getstartedsimulation(defaultseed)

        ! solve for each system and simulate individuals' lifes when reforms arrive unexpectedly
        do system = 1,sysN
        
            call fortax_loadsys_anydate(sysD(system))
            call solve

            do rep=1,nrsim
            do i=1,nind
                call ilocate(i,rep)                                  ! locate relevant parts of matrices to be filled
                age0 = max( sim(i,rep)%sysA0(system), mina )         ! calculate age interval under system
                age1 = min( sim(i,rep)%sysA1(system), maxa )
                if (age1>=age0) then
                    call sim_a0toa1(i,rep,age0,age1,chooseS,1)       ! simulate choices during ages under system
                end if                
            end do
            end do
            
        end do
       
        if (dwrite==1) call writesimstata(filename,nchar)

    end subroutine SolveAndSim_manySys



    ! -------------------------------------------------------------------------------------------------------
    ! Solve individual's problem, simulate LC profiles, calculate and store moments
    ! Same tax and benefit system over entire LC
    ! Select system before calling this routine
    ! Allows for +ve rents

    subroutine SolveAndSim_1Sys(chooseS, MixedAcc)
        use nag_g_ib,  only : g05kbf
        implicit none
        integer,intent(in) :: chooseS, MixedAcc
        integer            :: igen, iseed(4), i, rep


        ! Set seed for NAG and get initial conditions and shocks
        call getShocks(defaultseed)

        ! solve and simulate individuals' lifes under a single system loaded before this routine is called
        if (MixedAcc==0) then

            rent = rent0
            call solve
            do rep=1,nrsim
            do i=1,nind
                call ilocate(i,rep)                             !locate relevant parts of matrices to be filled
                call sim_a0toa1(i, rep, mina, maxa, chooseS, 1)
            end do
            end do

        else
        
            rent = rent0
            call solve
            do rep=1,nrsim
            do i=1,nind
                call ilocate(i,rep)                             !locate relevant parts of matrices to be filled
                call startlife(i, rep, chooseS)
                if (sim(i,rep)%drent == 0) call sim_a0toa1(i, rep, mina, maxa, 0, 0)
            end do
            end do

            rent = rentp
            call solve
            do rep=1,nrsim
            do i=1,nind
                call ilocate(i,rep)                             !locate relevant parts of matrices to be filled
                if (sim(i,rep)%drent == 1) call sim_a0toa1(i, rep, mina, maxa, 0, 0)
            end do
            end do
            
            rent = rent0

        end if
                
    end subroutine SolveAndSim_1Sys


    ! -------------------------------------------------------------------------------------------------------
    ! Solve individual's problem, simulate expected discounted lifetime income by education
    ! Do so for all tax-benefit systems as if they do not change over entire life
    
    subroutine simEY(nsim, filename)
        use nag_g_ib, only : g05lgf, g05kbf
        use stataMod
        implicit none
        integer,intent(in)      :: nsim
        character(*),intent(in) :: filename
        character(9)            :: EY(nts)
        integer                 :: j, i, s, rep, year, system, igen, iseed(4), Xseed(nsim), seed(1), ifail, a
        real(wp)                :: LTinc(nind,nrsim), Einc(nts,nind,1991:2008), unif(nsim)


        ! get seeds for nsim sets of nrsim simulations for each individual
        igen     = 0
        iseed    = 0
        iseed(1) = defaultseed
        call g05kbf(igen,iseed)
        call g05lgf(0.0_wp, 1.0_wp, nsim, unif, igen, iseed, ifail)
        Xseed = int(1000000.0_wp*unif)

        ! initialise solution and simulation routines
        call getstartedsolution(1.0_wp)
        call getstartedsimulation(defaultseed)
        
        ! clean storage matrix
        Einc = 0.0_wp
        
        ! solve & simulate for all tax-benefit systems
        do year = 1991,2008

            ! load system and solve
            system = year*10000 + 407
            write(*,*) 'system year ', year 
            call fortax_loadsys_anydate(system)
            call solve
            
            ! clean storage matrix
            LTinc = 0.0_wp

            ! simulate
            do j=1,nsim
            
                !draw new shocks and clean simulation structure
                call getstartedsimulation(Xseed(j))

                !calculate income over LC for each education level
                do i=1,nind
                    forall (rep=1:nrsim) sim(i,rep)%aa(:)%Oh%system = year   ! simulate all life under current system
                    do rep=1,nrsim     
                        call ilocate(i,rep)                                  ! locate relevant parts of matrices to be filled
                        call sim_bys(i,rep)                                  ! stores income by education level
                        call sim_a0toa1(i,rep,mina,maxa,1,1)                 ! stores outcomes at optimal education choice
                    end do
                end do

                !expected discounted lifetime income by education
                do s=1,nts
                    ! discount to age 19
		            forall (a=as(s):maxa) sim(:,:)%aa(a)%Oh%incbyS(s) = (R**real(19-a,wp)) * sim(:,:)%aa(a)%Oh%incbyS(s)
		            ! cumulate & average
                    forall (i=1:nind) 
                        forall (rep=1:nrsim) LTinc(i,rep) = sum ( sim(i,rep)%aa(as(s):maxa)%Oh%incbyS(s) )
                        Einc(s,i,year) = Einc(s,i,year) + sum(LTinc(i,:))/real(nrsim*nsim)
                    end forall
                end do
                
            end do

        end do

        ! export to STATA
        call saveStata(filename,3,nind)
        call writeStata((/ ((sim(i,1)%id),  i=1, nind) /), 'pid')
        call writeStata((/ ((sim(i,1)%yob), i=1, nind) /), 'yob')
        do s=1,3
            write (EY(s)(1:5), '("EYs", I1, "y")') s
            do year = 1991,2008
                write(EY(s)(6:9), '(I4)') year
                call writeStata((/ ((Einc(s,i,year)), i=1, nind) /), EY(s))
            end do
        end do
        call closeSaveStata
        
    end subroutine simEY


    ! -------------------------------------------------------------------------------------------------------
    ! Solve individual's problem, simulate LC profiles, calculate and store moments

    subroutine SolveSimAndCalcMom(ExpSim, simfile, simchar, PrintMom, momfile, momchar, Qform)
        implicit none
        integer,intent(in)            :: ExpSim, simchar, PrintMom, momchar
        character(simchar),intent(in) :: simfile    ! export simulated LC profiles to this file
        character(momchar),intent(in) :: momfile    ! print simulated moments to this file
        real(wp),intent(out)          :: Qform
        integer                       :: chooseS

        chooseS = 1
        call SolveAndSim_manySys(ExpSim, simfile, simchar, chooseS)
        call getMomForEst(1, PrintMom, momfile, momchar, Qform)
        Qform = Qform / (10.d0**QformScale)
        write(*,'("Objective value (iSE and iVarCov): ", 2F13.5)') Qform

    end subroutine SolveSimAndCalcMom


end module wrapperMod